#include <stdio.h>
#include <stdlib.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "graphic.h"
#include "command.h"

#define Ids(name) ai_ ## name, fi_ ## name ## _discovered

typedef struct {
    area_id_t id;
    flag_id_t discovered_id;
    char *name;
    char *description;
    int (*f)(verb_id_t, noun_id_t, int *exist);
    void (*post)(int time_passed);
    void (*draw)(void);
} area_t;

area_t areas[] = {
    {Ids(title), "タイトル画面",
     "「動詞」もしくは「動詞 名詞」の形式で入力してください。",
     area_title, NULL,
     draw_title,
    },
    {Ids(game_over), "ゲームオーバー",
     "あなたは死にました。",
     area_game_over, NULL,
     draw_game_over,
    },
    {Ids(ending), "クリア画面",
     "おめでとう、あなたは無事に屋敷を脱出した。",
     area_ending, NULL,
     draw_ending,
    },
    {Ids(achievements), "達成度",
     "ここは達成度の部屋だ。この部屋はまだ実装中だ。",
     area_achievements, NULL,
     draw_achievements,
    },


    {Ids(mailbox), "郵便受け",
//     "あなたは屋敷の敷地の入り口にいる。",
     "あなたは屋敷の敷地に踏み込んだところだ。",
     area_mailbox, NULL,
     draw_mailbox,
    },
    {Ids(entrance), "玄関",
     "あなたは玄関の前にいる。",
     area_entrance, NULL,
     draw_entrance,
    },
    {Ids(opened_window), "窓の下",
     "あなたは玄関右手の壁際にいる。",
     area_opened_window, NULL,
     draw_opened_window,
    },
    {Ids(washtub), "洗い桶",
     "あなたは屋敷の玄関右手にいる。",
     area_washtub, NULL,
     draw_washtub,
    },
    {Ids(backdoor), "裏口",
     "あなたは裏口の前にいる。",
     area_backdoor, NULL,
     draw_backdoor,
    },
    {Ids(under_tree), "屋敷裏",
     "あなたは屋敷の裏にいる。",
     area_under_tree, NULL,
     draw_under_tree,
    },
    {Ids(garden), "庭園",
     "あなたは庭園にいる。",
     area_garden, NULL,
     draw_garden,
    },






    {Ids(entrance_hall), "玄関ホール",
     "あなたは玄関ホールにいる。",
     area_entrance_hall, NULL,
     draw_entrance_hall,
    },
    {Ids(corridor_1f_fl), "一階左翼廊下",
     "あなたは一階の廊下にいる。",
     area_corridor_1f_fl, NULL,
     draw_corridor_1f_fl,
    },
    {Ids(corridor_1f_bl), "一階左翼廊下",
     "あなたは一階の廊下にいる。",
     area_corridor_1f_bl, NULL,
     draw_corridor_1f_bl,
    },
    {Ids(corridor_1f_fr), "一階右翼廊下",
     "あなたは一階の廊下にいる。",
     area_corridor_1f_fr, NULL,
     draw_corridor_1f_fr,
    },
    {Ids(corridor_1f_mr), "一階右翼廊下",
     "あなたは一階の廊下にいる。",
     area_corridor_1f_mr, NULL,
     draw_corridor_1f_mr,
    },
    {Ids(corridor_1f_br), "1階右翼廊下",
     "あなたは1階の廊下にいる。",
     area_corridor_1f_br, NULL,
     draw_corridor_1f_br,
    },
    {Ids(bathroom), "浴室",
     "あなたは浴室にいる。",
     area_bathroom, NULL,
     draw_bathroom,
    },
    {Ids(dining), "食堂",
     "あなたは食堂にいる。",
     area_dining, NULL,
     draw_dining,
    },
    {Ids(kitchen), "台所",
     "あなたは台所にいる。",
     area_kitchen, NULL,
     draw_kitchen,
    },
    {Ids(laundry), "洗濯室",
     "あなたは洗濯室にいる。",
     area_laundry, NULL,
     draw_laundry,
    },
    {Ids(living), "居間",
     "あなたは居間にいる。",
     area_living, post_living,
     draw_living,
    },
    {Ids(guestroom_1f), "客室",
     "あなたは客室にいる。",
     area_guestroom_1f, NULL,
     draw_guestroom_1f,
    },
    {Ids(storage), "物置部屋",
     "あなたは物置部屋にいる。",
     area_storage, NULL,
     draw_storage,
    },
    {Ids(playroom), "遊戯室",
     "あなたは遊戯室にいる。",
     area_playroom, NULL,
     draw_playroom,
    },
    {Ids(landing), "踊り場",
     "あなたは階段二階の踊り場にいる。",
     area_landing, NULL,
     draw_landing,
    },
    {Ids(corridor_2f_l), "二階階左翼廊下",
     "あなたは二階の廊下にいる。",
     area_corridor_2f_l, NULL,
     draw_corridor_2f_l,
    },
    {Ids(corridor_2f_c), "二階階廊下中央",
     "あなたは二階の廊下にいる。",
     area_corridor_2f_c, NULL,
     draw_corridor_2f_c,
    },
    {Ids(corridor_2f_cr), "二階階右翼廊下手前",
     "あなたは二階の廊下にいる。",
     area_corridor_2f_cr, post_corridor_2f_cr,
     draw_corridor_2f_cr,
    },
    {Ids(corridor_2f_r), "二階階右翼廊下奥",
     "あなたは二階の廊下にいる。",
     area_corridor_2f_r, NULL,
     draw_corridor_2f_r,
    },



    {Ids(boys_room), "子供部屋",
     "あなたは子供部屋にいる。",
     area_boys_room, NULL,
     draw_boys_room,
    },
    {Ids(bed), "二段ベッド上段",
     "あなたは二段ベッドの梯子の上にいる。",
     area_bed, NULL,
     draw_bed,
    },
    {Ids(girls_room), "子供部屋",
     "あなたは子供部屋にいる。",
     area_girls_room, NULL,
     draw_girls_room,
    },
    {Ids(guestroom_2f), "客室",
     "あなたは客室にいる。",
     area_guestroom_2f, post_guestroom_2f,
     draw_guestroom_2f,
    },
    {Ids(study), "書斎",
     "あなたは書斎にいる。",
     area_study, post_study,
     draw_study,
    },
    {Ids(library), "図書室",
     "あなたは図書室にいる。",
     area_library, post_library,
     draw_library,
    },
    {Ids(bookshelf), "本棚の上",
     "あなたは梯子に上って本棚の上を見ている。",
     area_bookshelf, NULL,
     draw_bookshelf,
    },
    {Ids(bedroom), "寝室",
     "あなたは寝室にいる。",
     area_bedroom, NULL,
     draw_bedroom,
    },
    {Ids(ladys_room), "婦人部屋",
     "あなたは婦人部屋にいる。",
     area_ladys_room, NULL, //post_ladys_room,
     draw_ladys_room,
    },
    {Ids(wardrobe), "洋服箪笥",
     "あなたは洋服箪笥の中にいる。",
     area_wardrobe, NULL, //post_wardrobe,
     draw_wardrobe,
    },
    {Ids(secret_room), "隠し部屋",
     "あなたは隠し部屋にいる。",
     area_secret_room, NULL,
     draw_secret_room,
    },
    {Ids(music_room), "音楽室",
     "あなたは音楽室にいる。",
     area_music_room, NULL,
     draw_music_room,
    },
    {Ids(bathroom_2f), "浴室",
     "あなたは浴室にいる。",
     area_bathroom_2f, NULL,
     draw_bathroom_2f,
    },
    {Ids(balcony), "バルコニー",
     "あなたはバルコニーにいる。",
     area_balcony, NULL,
     draw_balcony,
    },
    {Ids(balcony_down), "バルコニー",
     "あなたはバルコニーから庭園を見下ろしている。",
     area_balcony_down, NULL,
     draw_balcony_down,
    },

    {Ids(cellar), "地下室",
     "あなたは地下室にいる。",
     area_cellar, NULL,
     draw_cellar,
    },
    {Ids(boiler_room), "ボイラー室",
     "あなたはボイラー室にいる。",
     area_boiler_room, NULL,
     draw_boiler_room,
    },
    {Ids(corridor_0f), "地下通路",
     "あなたは地下へ下っていく通路にいる。",
     area_corridor_0f, NULL,
     draw_corridor_0f,
    },
    {Ids(temple), "神殿",
     "あなたは神殿にいる。",
     area_temple, post_temple,
     draw_temple,
    },
    {Ids(temple_ceiling), "神殿の天井",
     "あなたは神殿の天井を見上げている。",
     area_temple_ceiling, NULL,
     draw_temple_ceiling,
    },
    {Ids(cave_entrance), "洞窟の入口",
     "あなたは洞窟の入口に立って神殿を覗き込んでいる。",
     area_cave_entrance, NULL,
     draw_cave_entrance,
    },
    {Ids(cave), "洞窟",
     "あなたは洞窟にいる。",
     area_cave, NULL,
     draw_cave,
    },
    {Ids(under_cliff), "崖下",
     "洞窟を出ると、そこは海岸だった。"
     "振り返ると、波に浸食された海食崖の下に洞窟の口が開いており、"
     "崖の上には、屋敷の駒形切妻屋根が見えている。",
     //     "あなたは海岸の崖下にいる。",
     area_under_cliff, NULL,
     draw_under_cliff,
    },




    {Ids(working), "工事中の部屋",
     "あなたは工事中の部屋にいる。"
     "この部屋はいろいろテストするためのダミー部屋のようだ。",
     area_working, NULL,
     draw_test_room,
    },
};

#define AREA_NUM (sizeof(areas)/sizeof(area_t))

static area_id_t new_area_id = ai_invalid;

void init_area(void){
    int i;

	assert(ai_end <= 256);  // セーブファイルでarea_id_tが8ビットに収まることを想定
    for (i = ai_begin ; i < ai_end ; i++){
        if (areas[i].id != i){
            printf("システムエラー: "
                   "エリア「%s」のID(%d->%d)がずれています。\n", 
                   areas[i].name, i, areas[i].id);
            exit(1);
        }
    }        
}

void describe_area(area_id_t id){
    //    puts_term(areas[id].description);
    message(areas[id].description);
}

int call_area(area_id_t id, verb_id_t vid, noun_id_t nid, int *exist){
    return areas[id].f(vid, nid, exist);
}
void post_area(area_id_t id, int time_passed){
    if (areas[id].post){
        areas[id].post(time_passed);
    }
}
void cancel_move(void){
    new_area_id = ai_invalid;
}
void move_area(area_id_t id){
    new_area_id = id;
}
area_id_t new_area(void){
    return new_area_id;
}
void update_current_area(void){
    if (new_area_id == ai_invalid) return;
    set_area(new_area_id);
    set_flag(areas[new_area_id].discovered_id);
    // 移動と共に死んだ場合(黒い影に捕まったなど)は、移動先の描写をしない
    if (Not(dead)){
        describe_area(current_area());
    }
    draw_graphic();
    new_area_id = ai_invalid;
}
int is_area_discovered(area_id_t aid){
    return is_flag_on(areas[aid].discovered_id);
}

void draw_area(area_id_t id){
     areas[id].draw();
}

void gameover(void){
    message("      + + + + + + +  ゲームオーバー  + + + + + + + +");
    move_area(ai_game_over);
	update_current_area(); // msg_to_title()より前にdescribe_areaを実行したいので明示的にupdate.
    update_achievements();
	msg_to_title();
}

void move_to_title(void){
    clear_status();
    message("Mastery of the Houseの世界へようこそ。");
    move_area(ai_title);
    show_help_basic();
    msg_to_start();
}
void move_to_ending(void){
    message("      + + + + + + +  ゲームクリア  + + + + + + + +");
    move_area(ai_ending);    
	update_current_area(); // msg_to_title()より前にdescribe_areaを実行したいので明示的にupdate.
    update_achievements();
    message("なおこのゲームはフィクションであり、"
            "実在の人物・団体・事件・小説・ゲーム・アニメ・漫画・旧支配者"
            "などには一切関係ありません。");
	msg_to_title();
}
// クリアフラグをロード・セットしてから達成度の部屋に移動
void move_to_achievements(void){
    load_achievements_flags();
    move_area(ai_achievements);
}

void start_game(void){
    msg_start();
    clear_status();
    get_item(ni_newspaper); // 新聞は最初から持っている
    // 初期状態で剣士が置かれているので、これをアバター配置扱いにする
    Set(dining_toy_swordman_put);
    Set(dining_put_boy);
    move_area(ai_mailbox);
}
void die(void){
    message("あなたの意識はそのまま途切れた…。");
    set_wait_return_callback(gameover);
    Set(dead);
}
void die_touching_water(void){
    msg_touching_water();
    Set(death_by_touching_water);
    die();
}
void die_touching_shadow(void){
    msg_touching_shadow();
    Set(death_by_touching_shadow);
    die();
}
void die_killed_by_armor(void){
    msg_killed_by_armor();
    Set(death_by_armor);
    die();
}
void die_exiting_window(void){
    msg_exiting_window();
    Set(death_by_exiting_window);
    die();
}
void die_starved(void){
    msg_starved();
    Set(death_by_starvation);
    die();
}

void game_clear(void){
    message("      + + + + + + +  ゲームクリア  + + + + + + + +");
    message("おめでとう。あなたはとりあえず何かを封印して屋敷を脱出した。");
    msg_to_title();
}

