#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

#define SYMBOL_FLAG_NUM 2
typedef struct {
    noun_id_t nid;
    flag_id_t fids[SYMBOL_FLAG_NUM];
    char *message;
} symbol_t;

symbol_t death_symbols[] = {
    {ni_glasses,   
     {fi_death_by_touching_water, fi_invalid},
     "浴槽の水に触れて死す。"},
    {ni_necklace,  
     {fi_death_by_touching_shadow, fi_invalid},
     "2階の影に触れて死す。"},
    {ni_armor,     
     {fi_death_by_armor, fi_invalid},
     "鎧に切られて死す。"},
    {ni_slingshot,
     {fi_death_by_exiting_window, fi_invalid},
     "窓に挟まれて死す。"},
    {ni_skull,
     {fi_death_by_starvation, fi_invalid},
     "マクドナルドの轍を踏みて死す。"},
    {ni_tentacle,
     {fi_death_by_tentacle, fi_invalid},
     "支配者に捕らわれて死す。"},
};
#define DEATH_SYMBOL_NUM (sizeof(death_symbols)/sizeof(symbol_t))
symbol_t alternative_symbols[] = {
    {ni_fossil,
     {fi_clear_armor_alive, fi_clear_armor_dead},
     "鎧を倒した／倒さずにクリア。"},
    {ni_wooden_key,
     {fi_clear_treasure_chest_unlocked, fi_clear_treasure_chest_locked},
     "宝箱の錠を開けて／壊してクリア。"},
    {ni_portrait,
     {fi_clear_portrait_enabled, fi_clear_portrait_disabled},
     "肖像画を外して／外さずにクリア。"},
    {ni_human_bone,
     {fi_clear_using_human_bone, fi_clear_using_chicken_bone},
     "人の骨／鶏の骨でサイクロトロン機関を起動してクリア。"},
    {ni_telephone,
     {fi_clear_phone_call_answered, fi_clear_phone_call_ignored},
     "電話に出て／出ずにクリア。"},
};
#define ALTERNATIVE_SYMBOL_NUM (sizeof(alternative_symbols)/sizeof(symbol_t))

int is_achievements_on(flag_id_t fids[]);
int count_achievements(int num, symbol_t symbols[]);
void message_active_symbols(int num, symbol_t symbols[]);
void message_achievements_count(char *header, int num, symbol_t symbols[]);
flag_id_t *symbol_nid2fids(noun_id_t nid, int num, symbol_t symbols[]);
char *symbol_nid2message(noun_id_t nid, int num, symbol_t symbols[]);

int area_achievements(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
			message("%nlと%nlがある。",
	  				ni_fireplace, ni_table);
            message("タイトル画面に戻るには、「%v」と入力してください。",
                    vi_backward);
            return 1;
        case vi_backward:
        case vi_down:
		case vi_exit:
            move_to_title();
            return 1;
        default:
            break;
        }
        break;
    case ni_table:
        switch (vid){
        case vi_look:
            message("これは死のテーブルだ。");
            message("テーブルの辺りには\\");
            message_active_symbols(DEATH_SYMBOL_NUM, death_symbols);
            return 1;
        case vi_examine:
            message_achievements_count("死因", DEATH_SYMBOL_NUM, death_symbols);
            return 1;
        default:
            break;
        }
        break;
    case ni_glasses:
    case ni_necklace:
    case ni_armor:
    case ni_slingshot:
    case ni_skull:
        if (!is_achievements_on(symbol_nid2fids(nid, DEATH_SYMBOL_NUM,
                                                death_symbols)))
        {
            ReturnNotExist;
        }
        switch (vid){
        case vi_look:
        case vi_examine:
            message(symbol_nid2message(nid, DEATH_SYMBOL_NUM, death_symbols));
            return 1;
        default:
            break;
        }
    case ni_fireplace:
        switch (vid){
        case vi_look:
            message("これは選択の暖炉だ。");
            message("暖炉の辺りには\\");
            message_active_symbols(ALTERNATIVE_SYMBOL_NUM, alternative_symbols);
            return 1;
        case vi_examine:
            message_achievements_count("選択肢", ALTERNATIVE_SYMBOL_NUM, 
                                       alternative_symbols);
            return 1;
        default:
            break;
        }
        break;
    case ni_fossil:
    case ni_wooden_key:
    case ni_portrait:
    case ni_human_bone:
    case ni_telephone:
        if (!is_achievements_on(symbol_nid2fids(nid, ALTERNATIVE_SYMBOL_NUM,
                                                alternative_symbols)))
        {
            ReturnNotExist;
        }
        switch (vid){
        case vi_look:
        case vi_examine:
            message(symbol_nid2message(nid, ALTERNATIVE_SYMBOL_NUM, 
                                       alternative_symbols));
            return 1;
        default:
            break;
        }
    case ni_wall:
    case ni_floor:
    case ni_ceiling:
        break;
    default:
        *exist = 0;
        break;
    }
    return 0;
}

// フラグID配列fidsの(IDがinvalidでない)フラグがすべてONなら非0、
// OFFのフラグが一つでもあれば0を返す
int is_achievements_on(flag_id_t fids[]){
    int i;
    for (i = 0 ; i < SYMBOL_FLAG_NUM ; i++){
        printf("is_ach_on %d %d %d\n", i, fids[i], is_flag_off(fids[i]));
        if (fids[i] != fi_invalid && is_flag_off(fids[i])) return 0;
    }
    return 1;
}

int count_achievements(int num, symbol_t symbols[]){
    int i, count = 0;
    for (i = 0 ; i < num ; i++){
        printf("%s i=%d\n", symbols[i].message, i);
        // invalidでないフラグでOFFのものが一つでもあれば
        // そのシンボルはカウントしない
        if (is_achievements_on(symbols[i].fids)) count ++;
    }
    return count;
}

void message_active_symbols(int num, symbol_t symbols[]){
    int i, count, outcount = 0;
    count = count_achievements(num, symbols);
    switch (count){
    case 0:
        message("何もない。");
        break;
    default:
        for (i = 0 ; i < num ; i++){
            if (is_achievements_on(symbols[i].fids)){
                outcount ++;
                if (outcount < count-1){
                    message("%nl、\\", symbols[i].nid);
                }
                else if (outcount == count-1){
                    message("%nlと\\", symbols[i].nid);
                }
                else {
                    message("%nlがある。", symbols[i].nid);
                }
            }
        }
        break;
    }
}


void message_achievements_count(char *header, int num, symbol_t symbols[]){
    int count;

    count = count_achievements(num, symbols);
    message("なにか書いてある。");
    message("「%s達成率 %d/%d」", header, count, num);
}

flag_id_t *symbol_nid2fids(noun_id_t nid, int num, symbol_t symbols[]){
    int i;
    for (i = 0 ; i < num ; i++){
        if (symbols[i].nid == nid) return symbols[i].fids;
    }
    assert(0);
    return NULL;
}

char *symbol_nid2message(noun_id_t nid, int num, symbol_t symbols[]){
    int i;
    for (i = 0 ; i < num ; i++){
        if (symbols[i].nid == nid) return symbols[i].message;
    }
    assert(0);
    return NULL;
}

