#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

int area_bathroom(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
            message("向かって右側には%nlが、左側には%nlがある。", 
                    ni_washstand, ni_bathtub);
            message("奥には%nlがある。", ni_toilet);
            message("背後には廊下に続く扉がある。");
            return 1;
        case vi_backward:
        case vi_exit:
            EnterDoor(corridor_1f_fr, bathroom);
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_door:
    case ni_back_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_backward, ni_invalid);
			return 1;
		}
		break;
    case ni_wall:
        switch(vid){
        case vi_look:
        case vi_examine:
            if (Not(mirror_taken)){
                message("洗面台の上には%nlがある。", ni_mirror);
            }
            else {
                message("%nnのあった場所になにか書いてある。", ni_mirror);
            }
            return 1;
        case vi_read:
            if (Is(mirror_taken)){
                msg_read_bathroom_1f_wall();
                Set(bathroom_1f_message_read);
                return 1;
            }
            break;
        default:
            break;
        }
        break;
    case ni_bathtub:
        switch(vid){
        case vi_look:
            message(noun_description(nid));
            if (Not(bathtub_empty)){
                message("浴槽内には、ひどく汚い%nlがたまっている。", ni_water);
                message("水面には、%nlが浮かんでいる。", ni_rubber_duck);
                message("浴槽の縁には細い%nlが取り付けられている。",
                        ni_chain);
            }
            else {
                message("浴槽内は空だ。");
                if (!has_item(ni_rubber_duck)){
                    message("浴槽の底に%nlが転がっている。", ni_rubber_duck);
                }
                if (!has_item(ni_glasses)){
                    message("浴槽の底に%nlが転がっている。", ni_glasses);
                }
            }
            return 1;
        case vi_examine:
            message("中の水を気にしなければ、浴槽自体に変わったところはない。");
            return 1;
        case vi_enter:
            if (Is(bathtub_empty)){
                message("空の浴槽に入ってみた。");
                return 1;
            }
            else {
                die_touching_water();
            }
            return 1;
        default:
            break;
		}
        break;
    case ni_water:
        if (Is(bathtub_empty)){
			*exist = 0;
			return 0;
		}
        switch(vid){
        case vi_look:
            message("汚れが溜まっただけとは思えない、名状し難い色をした水だ。"
                    "表面には不気味な泡が浮かんでいる。");
            return 1;
        case vi_examine:
            message("異様な色の水面を見つめていると、"
                    "水面がわずかに動いているような気がしてくる。");
            return 1;
        case vi_get:
        case vi_touch:
            die_touching_water();
            return 1;
        default:
            break;
        }
    case ni_chain:
        switch(vid){
        case vi_look:
        case vi_examine:
			message(noun_description(nid));
            if (Not(bathtub_empty)){
                message("浴槽の縁から水中に垂れ下がっている。");
            }
            else {
                message("浴槽の栓に繋がっている。");
            }
            return 1;
        case vi_move:
            message("簡単に引っ張れそうだ。");
            return 1;
        case vi_get:
            message("固定されている端を外すのは無理そうだ。");
            return 1;
        case vi_pull:
            if (Not(bathtub_empty)){
                message("鎖を引くと、ぽんと浴槽の栓が外れた。");
                message("ごぼごぼと嫌な音をたてて、"
                        "おぞましい色の水が排水口に吸い込まれていく。");
                message("さいごにうめき声のような音を残して、浴槽は空になった。");
                Set(bathtub_empty);
            }
            else {
                message("もう引っ張っても意味がない。");
            }
            return 1;
        default:
            break;
		}
        break;
    case ni_rubber_duck:
		if (has_item(ni_rubber_duck)){
			*exist = 0;
			return 0;
		}
        switch (vid){
        case vi_look:
            message(noun_description(nid));
            if (Is(bathtub_empty)){
                message("空になった浴槽の底に転がっている。");
            }
            return 1;
        case vi_examine:
            msg_need_to_get();
            return 1;
        case vi_get:
            if (Is(bathtub_empty)){
                command_get_item(ni_rubber_duck, fi_duck_taken);
                return 1;
            }
            else {
                die_touching_water();
            }
            return 1;
        case vi_touch:
            if (Is(bathtub_empty)){
                msg_no_effect();
                return 1;
            }
            else {
                die_touching_water();
            }
            return 1;
        default:
            break;
		}
        break;
    case ni_glasses:
		if (has_item(ni_glasses) || Not(bathtub_empty)){
			*exist = 0;
			return 0;
		}
        switch (vid){
        case vi_look:
            message(noun_description(nid));
            return 1;
        case vi_examine:
            msg_need_to_get();
            return 1;
        case vi_get:
            command_get_item(ni_glasses, fi_glasses_taken);
            return 1;
        case vi_touch:
            msg_no_effect();
            return 1;
        default:
            break;
		}
        break;
    case ni_washstand:
        switch (vid){
        case vi_look:
            message(noun_description(nid));
            if (Not(mirror_taken)){
                message("洗面台の上には%nlがある。", ni_mirror);
            }
            return 1;
        case vi_examine:
        case vi_use:
            message("蛇口をひねっても水は出ない。");
            return 1;
        default:
            break;
		}
        break;
    case ni_mirror:
		if (Is(mirror_taken)){
			*exist = 0;
			return 0;
		}
        switch (vid){
        case vi_look:
            message(noun_description(nid));
            return 1;
        case vi_examine:
        case vi_get:
            message("%nlで壁に固定されている。", ni_screw);
            return 1;
        default:
            break;
        }
        break;
    case ni_screw:
		if (Is(mirror_taken)){
			*exist = 0;
			return 0;
		}
        switch(vid){
        case vi_examine:
        case vi_get:
        case vi_remove:
			message("しっかり止められている。外すには道具が必要だ。");
			return 1;
		default:
			break;
		}
        break;
    case ni_screwdriver:
        if (!has_item(nid)) ReturnNotExist;
        switch(vid){
        case vi_use:
            if (Is(mirror_taken)){
                message("もう外すものはない");
            }
            else {
                message("ねじ回しでねじを外した。");
                command_get_item(ni_mirror, fi_mirror_taken);
            }
            return 1;
		default:
			break;
		}
        break;
    case ni_toilet:
        switch(vid){
        case vi_look:
            message(noun_description(nid));
            return 1;
        case vi_examine:
            message("レバーを押しても水は流れない。\\");
			if (Not(toilet_used)){
	            message("便器や\\");
			}
            message("タンク内も乾ききっている。");
            return 1;
        case vi_use:
            if (Is(toilet_used)){
                message("また用を足した。");
            }
            else {
                message("用を足した。");
                Set(toilet_used);
			}
            return 1;
        default:
            break;
		}
        break;
    case ni_floor:
    case ni_ceiling:
        break;
    default:
		*exist = 0;
        break;
    }
    return 0;
}


