#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

void bathroom_ghost(void);
void bathroom_ghost_end(void);

int area_bathroom_2f(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
            message("正面に%nlと%nlがある。", 
                    ni_bathtub, ni_washstand);
            message("背後には音楽室に続く扉がある。");
            return 1;
        case vi_backward:
        case vi_exit:
            EnterDoor(music_room, bathroom_2f);
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_door:
    case ni_back_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_backward, ni_invalid);
		}
		break;
    case ni_wall:
        switch(vid){
        case vi_look:
            message("%nnの上には%nlが掛かっていたようだ。", 
                    ni_washstand, ni_mirror);
			return 1;
        case vi_examine:
			if (Is(bathroom_2f_ghost_seen)){
				TestAndSet(bathroom_2f_message_found,
						   "人影が指したあたりを調べてみると、"
						   "壁紙が少しめくれ、その下に小さくなにか書いてある。",
						   "壁の文字以外に変わったところはない。");
	            return 1;
			}
		case vi_read:
			if (Is(bathroom_2f_message_found)){
				msg_read_bathroom_2f_wall();
                Set(bathroom_2f_message_read);
				return 1;
			}
			break;
        default:
            break;
        }
        break;
    case ni_floor:
        switch(vid){
        case vi_look:
        case vi_examine:
            message("%nnの周りに%nnの破片が散らばっている。", 
                    ni_washstand, ni_mirror);
            return 1;
        default:
            break;
        }
        break;
    case ni_bathtub:
        switch(vid){
        case vi_look:
            message("すいぶんと古びた%nnだ。",
                    ni_bathtub);
            return 1;
        case vi_examine:
            message("中は空だ。水一滴残っていない。");
            RedoArea(vi_use, ni_bathtub);
            return 1;
        case vi_use:
            message("蛇口をひねっても水は出ない。");
            return 1;
        default:
            break;
        }
        break;
    case ni_washstand:
        switch(vid){
        case vi_look:
            RedoArea(vi_look, ni_wall);
        case vi_examine:
        case vi_use:
            message("蛇口をひねっても水は出ない。");
            return 1;
        default:
            break;
        }
        break;
    case ni_mirror:
        switch(vid){
        case vi_look:
        case vi_examine:
            if (Not(mirror_put)){
                message("粉々に割れている。");
                if (has_item(ni_mirror)){
                    message("手元の鏡は壊れていない。");
                }
            }
            else {
				// このイベントは一回しか見られない
				if (Not(bathroom_2f_ghost_seen)){
					bathroom_ghost();
				}
				else {
					message("もう鏡には、部屋の光景しか映っていない。");
				}
            }
            return 1;
        case vi_get:
            if (Is(mirror_put)){
                // フラグは立てないのでただのget_item()
                message("鏡を再び取った。");
                get_item(ni_mirror);
                Reset(mirror_put);
            }
            else {
                message("こんな破片では役に立ちそうにない。");
            }
            return 1;
        case vi_put:
            if (!has_item(ni_mirror)) ReturnNotExist;
            message("割れた鏡のあったあたりに、1階浴室の鏡を置いてみた。");
            lose_item(ni_mirror);
            Set(mirror_put);
            return 1;
        default:
            break;
        }
        break;
    case ni_ceiling:
    case ni_window:
        break;
    default:
        *exist = 0;
        break;
    }
    return 0;
}

void bathroom_ghost(void){
    message("置いた鏡をのぞき込むと、"
            "あなたの背後にうっすらと女性の姿が映っている。"
			"彼女は壁の一点を指し、もの言いたげな視線をこちらに向けている。");
    Set(bathroom_ghost_appeared);
    set_wait_return_callback(bathroom_ghost_end);
}

void bathroom_ghost_end(void){
    message("やがて女性の姿は、すうっと消えていった。");
    Reset(bathroom_ghost_appeared);
    Set(bathroom_2f_ghost_seen);
}
