#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

void stop_cycrotron(void);

int area_boiler_room(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
			message("目の前には、見たこともない%nlが据え付けられている。",
					ni_machine);
            message("背後には、地下室へと続く%nlがある。",
                    ni_door);
            return 1;
		case vi_backward:
		case vi_exit:
            EnterDoor(cellar, boiler_room);
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
	case ni_machine:
        switch (vid){
        case vi_look:
			message(noun_description(nid));
			return 1;
        case vi_examine:
			if (Is(cycrotron_activated)){
				message("機械は低い唸りをあげて動いている。");
			}
			else {
				message("機械は沈黙している。");
			}
			message("機械の前面パネル%nwは、一部が開くようになっている。"
					"右側には%nlがあり、その上にも%nlがある。",
					ni_panel, ni_round_door, ni_small_door);
			message("右上には赤色と緑色の大きなボタン%nw%nwがある。",
					ni_red_button, ni_green_button);
			return 1;
        case vi_destroy:
        case vi_hit:
        case vi_kick:
			message("よくわからないものを壊すのはやめておこう。");
			return 1;
        default:
            break;
        }
        break;
	case ni_panel:
        switch (vid){
        case vi_look:
			if (Is(cycrotron_panel_open)){
				message("パネルを開いた部分から、機械の中身が見える。");
			}
			else {
				message("金属の前面パネルだ。");
			}
        case vi_examine:
			message("一部が開くようになっている。");
			if (Is(cycrotron_panel_open)){
				message("機械の中は、多数の%nlが張り巡らされている。",
					    ni_cable);
			}
			return 1;
        case vi_open:
        case vi_pull:
			TestAndSet(cycrotron_panel_open, 
					   "パネルを開けた。",
					   "パネルはもう開いている。");
			return 1;
        case vi_close:
        case vi_push:
			TestAndReset(cycrotron_panel_open, 
					   "パネルを閉じた。",
					   "パネルはもう閉じている。");
			return 1;
        default:
            break;
        }
        break;
	case ni_cable:
		if (Not(cycrotron_panel_open)) ReturnNotExist;
        switch (vid){
        case vi_look:
			if (Is(rubber_duck_put)){
				message("ケーブルの間に%nlが挟んである。",
						ni_rubber_duck);
			}
			else {
				message("ケーブルの一本が垂れ下がり、"
						"下の方を走るケーブルと接触している。");
			}
			return 1;
        case vi_examine:
			if (Is(rubber_duck_put)){
				message("ショートしていたケーブルを%nlで絶縁してある。",
						ni_rubber_duck);
			}
			else {
				message("接触しているケーブルの被膜が剥がしてある。"
						"このまま通電すればショートするだろう。");
			}
			return 1;
        default:
            break;
        }
        break;
	case ni_rubber_duck:
		if (!has_item(nid) && Not(rubber_duck_put)) ReturnNotExist;
		if (Not(cycrotron_panel_open)) ReturnNotExist;
		if (Not(rubber_duck_put)){
			if (vid == vi_put){
				message("%nnを挟んで、ショートしているケーブルを絶縁した。", nid);
				Set(rubber_duck_put);
				lose_item(nid);
				return 1;
			}
			break;
		}
		else {
	        switch (vid){
		    case vi_look:
				message("ケーブルの間に挟んである。");
				return 1;
	        case vi_examine:
				message("ショートしていたケーブルを絶縁している。");
				return 1;
//        case vi_get:
	        default:
		        break;
			}
		}
		break;
	case ni_round_door:
        switch (vid){
        case vi_look:
			message(noun_description(nid));
			return 1;
        case vi_examine:
			if (Not(cycrotron_round_door_open)){
				message("取っ手を回せば開きそうだ。");
			}
			else {
				message("中には金属棒の束のようなものが入っている。");
			}
			return 1;
        case vi_open:
        case vi_pull:
			TestAndSet(cycrotron_round_door_open, 
					   "丸扉を開けた。",
					   "丸扉はもう開いている。");
			if (Is(cycrotron_activated)){
				message("「燃料」と書かれたランプが点灯し、\\");			
				stop_cycrotron();
			}
			return 1;
        case vi_close:
        case vi_push:
			TestAndReset(cycrotron_round_door_open, 
					   "丸扉を閉じた。",
					   "丸扉はもう閉じている。");
			return 1;
        default:
            break;
        }
        break;
	case ni_small_door:
        switch (vid){
        case vi_look:
			message(noun_description(nid));
        case vi_examine:
			message("取っ手を回せば開きそうだ。");
			return 1;
        case vi_open:
        case vi_pull:
			TestAndSet(cycrotron_small_door_open, 
					   "小さな扉を開けた。",
					   "小さな扉はもう開いている。");
			if (Is(cycrotron_activated)){
				message("「触媒」と書かれたランプが点灯し、\\");			
				stop_cycrotron();
			}
		return 1;
        case vi_close:
        case vi_push:
			TestAndReset(cycrotron_small_door_open, 
					   "小さな扉を閉じた。",
					   "小さな扉はもう閉じている。");
			return 1;
        default:
            break;
        }
        break;
	case ni_chicken_bone:
	case ni_human_bone:
        // 取っていなければ(所持も設置もしていないから)ここにはない
        if (!((nid == ni_chicken_bone) ?
              Is(chicken_bone_taken) : Is(human_bone_taken))){
            ReturnNotExist;
        }
		if (Not(cycrotron_small_door_open)) ReturnNotExist;
		if (Not(chicken_bone_put) && Not(human_bone_put) ){
			if (vid == vi_put){
				message("機械に%nnを入れた。", nid);
                if (nid == ni_chicken_bone){
                    Set(chicken_bone_put);
                }
                else {
                    Set(human_bone_put);
                }
				lose_item(nid);
				return 1;
			}
			break;
		}
		else {
	        switch (vid){
		    case vi_look:
	        case vi_examine:
				message("機械の中に収まっている。");
				return 1;
//        case vi_get:
	        default:
		        break;
			}
		}
		break;
	case ni_button:
		switch (vid){
		case vi_look:
			message("赤と緑の大きなボタン%nw%nwだ。",
                    ni_red_button, ni_green_button);
			return 1;
		case vi_push:
		case vi_hit:
            message("ボタンは複数ある。");
			return 1;
		default:
		    break;
		}
		break;
	case ni_red_button:
		switch (vid){
		case vi_look:
			message("赤色の大きなボタンだ。");
			return 1;
		case vi_push:
		case vi_hit:
			// すでに停止中なら変化なし
			if (Not(cycrotron_activated)){
				msg_no_effect();
			}
			else {
				stop_cycrotron();
			}
			return 1;
		default:
		    break;
		}
		break;
	case ni_green_button:
		switch (vid){
		case vi_look:
			message("緑色の大きなボタンだ。");
			return 1;
		case vi_push:
		case vi_hit:
			// すでに稼働中なら変化なし
			if (Is(cycrotron_activated)){
				msg_no_effect();
			}
			// ショートしたままなら変化なし
			else if (Not(rubber_duck_put)){
				msg_no_effect();
			}
			else {
				message("低い唸り音と共に、機械が動き出した。\\");
				if ((Not(chicken_bone_put) && Not(human_bone_put)) ||
                    Is(cycrotron_small_door_open)){
					message("しかし、「触媒」と書かれたランプが点灯し、すぐに停止してしまった。");
				}
				else if (Is(cycrotron_round_door_open)){
					message("しかし、「燃料」と書かれたランプが点灯し、すぐに停止してしまった。");
				}
				else {
					message("");
					Set(cycrotron_activated);
				}
			}
			return 1;
		default:
		    break;
		}
		break;
	case ni_door:
		msg_which_door();
        return 1;
    case ni_back_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_backward, ni_invalid);
			return 1;
		}
        break;
    case ni_wall:
        switch (vid){
		case vi_look:
		case vi_examine:
			message("何か書いてある。");
			return 1;
		case vi_read:
            msg_read_boiler_room_wall();
			Set(boiler_room_note_read);
			return 1;
        default:
            break;
        }
    case ni_ceiling:
        switch (vid){
		case vi_look:
			message("機械から延びたパイプが、天井へと繋がっている。");
			return 1;
        default:
            break;
        }
        break;
    case ni_floor:
        break;
    default:
        *exist = 0;
        break;
    }
    return 0;
}

void stop_cycrotron(void){
	message("機械の動作音が止まった。");
	Reset(cycrotron_activated);
}

