#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

int area_boys_room(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
            message("向かって右側には%nlがあり、左側には%nlがある。", 
                    ni_bed, ni_treasure_chest);
            message("背後には廊下に続く扉がある。");
            return 1;
        case vi_backward:
		case vi_exit:
            EnterDoor(corridor_2f_c, boys_room);
            return 1;
        case vi_up:
            message("二段ベッドにでも上がってみるか。");
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_door:
    case ni_back_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_backward, ni_invalid);
        }
    // ベッドは別室に異なる種類のものがあるため、
    // 二段ベッド固有の処理はここに書く
    case ni_bed:
        switch(vid){
        case vi_look:
            message("二段ベッドだ。\\");
            if (Not(ladder_taken)){
                message("%nlが取り付けられている。", ni_ladder);
            }
            else {
                message("");
            }
            return 1;
        case vi_examine:
            message("なにもないようだが…。");
            return 1;
        case vi_up:
            if (Not(ladder_taken)){
                RedoArea(vi_up, ni_ladder);
            }
            else {
                message("よじ登るのは難しそうだ。");
            }
            return 1;
        default:
            break;
        }
        break;
    case ni_ladder:
        if (has_item(ni_ladder)){
            if (vid == vi_put){
                message("%nnを元に戻した。", ni_ladder);
                lose_item(ni_ladder);
                Reset(ladder_taken);
                return 1;
            }
            else {
				*exist = 0;
                return 0;
            }
        }
        else {
			if (Is(ladder_taken)){
				*exist = 0;
				return 0;
			}
            switch (vid){
            case vi_examine:
                msg_nothing_special(); // msg_need_to_getをオーバライド
            case vi_get:
                command_get_item(ni_ladder, fi_ladder_taken);
                return 1;
            case vi_up:
                message("梯子を登った。");
                move_area(ai_bed);
                return 1;
            break;
            default:
                break;
            }
        }
    case ni_wooden_key:
        if (has_item(nid)){
            switch(vid){
            case vi_use:
				if (Not(treasure_chest_unlocked) && Not(treasure_chest_broken)){
					message("鍵が開いた。");
					Set(treasure_chest_unlocked);
					lose_item(nid);
				}
				else {
					message("もう鍵は掛かっていない。");
				}
                return 1;
            default:
                break;
            }
            return 0;
        }
		else {
            ReturnNotExist;
		}
        break;
    case ni_treasure_chest:
        switch (vid){
        case vi_look:
            message(noun_description(nid));
            if (Is(treasure_chest_opened)){
                message("中は空だ。");
            }
			if (Is(treasure_chest_moved) &&
	            Not(slingshot_taken))
		    {
			    message("最初に宝箱があったところには、%nlがある",
	                    ni_slingshot);
		    }
            return 1;
        case vi_examine:
            message("大きな鍵穴がある。");
            return 1;
        case vi_get:
            msg_too_large();
            return 1;
        case vi_hit:
        case vi_kick:
        case vi_destroy:
            if (Is(treasure_chest_unlocked)){
                message("もう鍵は掛かっていない。");
            }
            else if (Is(treasure_chest_broken)){
                message("もう鍵は壊れている。");
            }
            else {
				Set(treasure_chest_broken);
                message("鍵を叩き壊した。");
            }
            return 1;
        case vi_open:
            if (Not(treasure_chest_opened)){
                if (Is(treasure_chest_unlocked) || Is(treasure_chest_broken)){
                    message("宝箱の中から%nlと%nlが出てきた。",
                            ni_box, ni_billiard_ball);
                    Set(treasure_chest_opened);
                }
                else {
                    message("鍵が掛かっている。");
                }
            }
            else {
                message("宝箱はもう開いている。");
            }
            return 1;
        case vi_close:
            TestAndReset(treasure_chest_opened,
                         "宝箱を閉じた。",
                         "宝箱は閉まっている。");
            return 1;
        case vi_move:
        case vi_push:
            TestAndSet(treasure_chest_moved,
                       "宝箱を動かした。",
                       "宝箱はもう動かしてある。");
            return 1;
        default:
            break;
        }
        break;
    case ni_billiard_ball:
        ReturnIfHasItem;
        if (Is(billiard_ball_taken) ||
            Not(treasure_chest_opened))
        {
			ReturnNotExist;
		}
        switch (vid){
		case vi_get:
			command_get_item(nid, fi_billiard_ball_taken);
			return 1;
		default:
			break;
		}
		break;
    case ni_slingshot:
        ReturnIfHasItem;
        if (Is(slingshot_taken) ||
            Not(treasure_chest_moved))
        {
			ReturnNotExist;
		}
        switch (vid){
		case vi_get:
			command_get_item(nid, fi_slingshot_taken);
			return 1;
		default:
			break;
		}
		break;
    case ni_box:
        if (Not(treasure_chest_opened)) ReturnNotExist;
        switch (vid){
		case vi_look:
            message("子供向けの玩具の箱だ。"
                    "剣を持ったロボットと鳥・ライオン型のメカが合体する絵が描かれている。");
			return 1;
		case vi_examine:
		case vi_open:
            message("箱の中身は空だ。");
			return 1;
		case vi_get:
            message("箱だけ持って行っても役には立たないだろう。");
            return 1;
		default:
			break;
		}
        break;
    case ni_floor:
        switch (vid){
		case vi_look:
			message("床の上には\\");
			if (Not(treasure_chest_opened)){
		        message("%nlがある。",
			            ni_treasure_chest);
			}
			else{
				if (Is(billiard_ball_taken)){
			        message("%nlと%nlがある。",
				            ni_treasure_chest, ni_box);
				}
				else {
			        message("%nlと%nl、%nlがある。",
				            ni_treasure_chest, ni_box, ni_billiard_ball);
				}
			}
			if (Is(treasure_chest_moved) &&
	            Not(slingshot_taken))
		    {
			    message("最初に宝箱があったところには、%nlがある",
	                    ni_slingshot);
		    }
        return 1;
		default:
			break;
		}
		break;
    case ni_wall:
    case ni_ceiling:
	case ni_window:
        break;
    default:
		*exist = 0;
        break;
    }
    return 0;
}
