#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

typedef enum {
    ap_begin, ap_man=ap_begin, ap_woman, ap_girl, ap_boy, ap_end
} avator_pos_t;

typedef struct {
    noun_id_t nid;
    flag_id_t fid;
    avator_pos_t pos;
} avator_info_t;

#define Avator(name, pos)  {ni_ ## name, fi_dining_ ## name ## _put, ap_ ## pos}

avator_info_t avator_infos[] = {
    //    {ni_glasses, fi_dining_glasses_put, ap_man},
    Avator(glasses, man),
    Avator(necklace, woman),
    Avator(plush_cat, girl),
    Avator(large_robot, boy),
    Avator(fishing_rod, man),
    Avator(bottle, woman),
    Avator(fossil, man),
    Avator(screwdriver, man),
    Avator(girl_doll, girl),
    Avator(witch_doll, girl),
    Avator(marble, boy),
    Avator(poker, man),
    Avator(toy_swordman, boy),
    Avator(toy_lion, boy),
    Avator(toy_bird, boy),
    Avator(robot, boy),
    Avator(toy_dragon, boy),
    Avator(billiard_ball, man),
    Avator(rubber_duck, man),
    Avator(slingshot, boy),
    Avator(ladder, man),
    Avator(diary, girl),
    Avator(score, girl),
    Avator(origami_plane, boy),
    Avator(essay, boy),
    Avator(paper, girl),
    Avator(lantern, man),
    Avator(mirror, woman),
    Avator(medal, girl),
};
#define AVATOR_INFO_NUM (sizeof(avator_infos)/sizeof(avator_info_t))
static flag_id_t avator_flags[] = {
    fi_dining_put_man,
    fi_dining_put_woman,
    fi_dining_put_girl,
    fi_dining_put_boy
};
flag_id_t correct_avators[] = {
    fi_dining_glasses_put,
    fi_dining_necklace_put,
    fi_dining_plush_cat_put,
    fi_dining_large_robot_put
};


void dining_move_backward(void);
avator_info_t *dining_get_avator_info(noun_id_t nid);
// 指定したアバター位置にあるもののnidを返す
// なければni_invalidを返す
noun_id_t get_avator_nid(avator_pos_t pos);
void dining_put_avator(noun_id_t nid);
int is_avator_put(noun_id_t nid);
// アバターとして置かれたものをとる
void dining_get_avator(noun_id_t nid);
void dining_look_chair(void);
void necromancy(void);
void necromancy_end(void);

int area_dining(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
    *exist = 1;

    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
			message("%nlの周りに%nlがいくつか並んでいる。",
	  				ni_table, ni_chair);
            message("背後と左右に扉%nw%nw%nwがある。",
                    ni_back_door, ni_left_door, ni_right_door);
            
            if (Is(hall_dining_door_opened)){
                message("背後の扉は玄関ホールに通じている。");
            }
            if (Is(dining_living_door_opened) &&
                Is(dining_kitchen_door_opened))
            {
                message("左右の扉は、それぞれぞれ居間と台所に通じている。");
            }
            else if (Is(dining_living_door_opened)){
                message("左側の扉は居間に通じている。");
            }
            else if (Is(dining_kitchen_door_opened)){
                message("右側の扉は台所に通じている。");
            }
            return 1;
        case vi_backward:
			if (Not(hall_dining_door_unlocked)){
				msg_door_unlocked_by_opening();
		        Set(hall_dining_door_unlocked);
			}
		    EnterDoor(entrance_hall, hall_dining);
			return 1;
        case vi_left:
            EnterDoor(living, dining_living);
            return 1;
        case vi_right:
            EnterDoor(kitchen, dining_kitchen);
            return 1;
		case vi_pray:
			necromancy();
			return 1;
        default:
            break;
		}
		break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_door:
        msg_which_door();
        return 1;
    case ni_back_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_backward, ni_invalid);
		}
        break;
    case ni_left_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_left, ni_invalid);
		}
        break;
    case ni_right_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_right, ni_invalid);
		}
        break;
	case ni_table:
        switch (vid){
    	case vi_look:
			if (Is(candle_lit)){
				message("テーブルの上には、食事をした後の食器が並んでいる。\\");
				if (Not(chicken_bone_taken)){
					message("中央の皿には、%nlが残っている。",
				     		ni_chicken_bone);
				}
				else {
					message("");
				}
			}
			else {
				message("テーブルの上には食器が並び、これから食事を始めようとしているかのようだ。");
			}
			if (Not(candle_taken)){
				message("テーブルの端の燭台には%nlが立っている。",
						ni_candle);
			}
            return 1;
		default:
			break;
		}
		break;
	case ni_chicken_bone:
		if (Not(candle_lit)) ReturnNotExist;
		if (has_item(nid)) ReturnNotExist;
        switch (vid){
    	case vi_get:
			command_get_item(nid, fi_chicken_bone_taken);
            return 1;
		default:
			break;
		}
		break;
	case ni_chair:
        switch(vid){
    	case vi_look:
            dining_look_chair();
            return 1;
        default:
            break;
        }
		break;
	case ni_chicken:
        if (has_item(nid)){
            switch(vid){
            case vi_put:
				command_put_item(nid, fi_chicken_put, NULL);
                return 1;
            default:
                break;
            }
            return 0;
        }
		else {
			if (Not(chicken_put)){
				ReturnNotExist;
			}
            switch (vid){
            case vi_get:
                // 再取得なのでfi_chicken_takenはすでにON
                command_get_item(nid, fi_invalid);
                Reset(chicken_put);
                return 1;
            default:
                break;
            }
		}
        break;
    // 正解の依代
    case ni_glasses:         // 教授の眼鏡
    case ni_necklace:        // 夫人のネックレス
    case ni_large_robot:     // 男の子の巨大ロボット
    case ni_plush_cat:       // 女の子のぬいぐるみ。
    // 間違った依代
    case ni_medal:           // おもちゃのメダル

	case ni_toy_swordman:    // 男の子の巨大ロボットパーツ
    case ni_toy_lion:        // 男の子の巨大ロボットパーツ
    case ni_toy_bird:        // 男の子の巨大ロボットパーツ
    case ni_robot:           // 男の子の巨大ロボットパーツ
    case ni_toy_dragon:      // 男の子の巨大ロボットパーツ
    case ni_girl_doll:       // 赤毛の人形。ダミー
    case ni_witch_doll:      // 魔女の人形。ダミー
    case ni_slingshot:       // ぱちんこ
    case ni_marble:          // ビー玉。ぱちんこの弾
    case ni_poker:           // 火かき棒
	case ni_rubber_duck:     // ゴムのアヒル
    case ni_mirror:          // 浴室の鏡
    case ni_screwdriver:     // ねじ回し
    case ni_billiard_ball:   // ビリヤードの玉
    case ni_origami_plane:   // 紙飛行機
	case ni_score:           // 楽譜
    case ni_fishing_rod:     // 釣竿
    case ni_ladder:          // 二段ベッドの梯子
	case ni_bottle:          // 燻香の瓶
    case ni_diary:           // !教授娘の日記帳
    case ni_paper:           // !紙(教授娘の日記帳のページ、分解した紙飛行機)
    case ni_lantern:         // ランタン
	case ni_essay:           // !教授息子の作文(開いた紙屑)
    case ni_fossil:          // 化石


    // ここで置く可能性のないアイテム
    //   すでに使用済
    //case ni_brass_key:       // 玄関の鍵
    //   必ず未入手
    // 交霊イベント終了まで明かりがないから地下と隠し部屋には行けない
    //case ni_hat:             // 大佐の帽子
    //case ni_belt:            // 犬の首輪
    //case ni_amulet:          // 旧神の印
    // 交霊イベント後に出現 
	//case ni_chicken_bone:    // 鶏の骨

    //   現バージョンでは入手できない
	//case ni_postcard:        // !大佐から教授への葉書
	//case ni_logbook:         // !看護人の日誌
    //case ni_screw:           // 浴室の鏡のねじ。今のところ入手アイテムではない
	//case ni_cuckoo:          // 鳥型ロボと入れ替わった鳩時計のカッコー。未登場

        if (has_item(nid)){
            switch(vid){
            case vi_put:
				dining_put_avator(nid);
                return 1;
            default:
                break;
            }
        }
		else {
			if (!is_avator_put(nid)){
				ReturnNotExist;
			}
            switch(vid){
            case vi_get:
                if (Is(candle_lit)){
                    message("もう必要ないだろう。");
                }
                else {
                    puts("get avator");
                    dining_get_avator(nid);
                }
                return 1;
            default:
                break;
            }
		}
		break;
    // あきらかに依代ではないもの
    case ni_newspaper:       // 最初から持っている新聞の切り抜き
    case ni_leaflet:         // 郵便受けのちらし
    case ni_wooden_key:      // 玄関前のダミーの鍵。宝箱の鍵。
    case ni_bronze_key:      // 2階踊り場の鍵
	case ni_steel_key:       // 書斎の机の鍵
	case ni_golden_key:      // 化粧台の鍵
    case ni_silver_key:      // 廃止アイテム。裏シナリオ用？
    case ni_branch:          // 二股の木の枝。ぱちんこの柄の代用品
    case ni_rubber_strip:    // ぱちんこのゴム紐
    case ni_sponge:          // スポンジ
	case ni_wastepaper:      // !紙屑
        if (has_item(nid)){
            switch(vid){
            case vi_put:
                message("これは関係なさそうだ。");
                return 1;
            default:
                break;
            }
        }
        else {
            ReturnNotExist;
        }
		break;
        /*
    case ni_toy_swordman:
		if (has_item(nid)){
		    switch (vid){
			default:
				break;
			}
		}
		else {
	        switch (vid){
			case vi_get:
				command_get_item(nid, fi_swordman_taken);
				return 1;
			default:
				break;
			}
		}
		break;
        */
	case ni_candle:
		if (has_item(nid)){
		    switch (vid){
			case vi_put:
				message("蝋燭を戻した。");
				lose_item(ni_candle);
				Reset(candle_taken);
				return 1;
			default:
				break;
			}
		}
		else {
	        switch (vid){
			case vi_get:
				command_get_item(nid, fi_candle_taken);
				return 1;
			default:
				break;
			}
		}
		break;
    case ni_wall:
    case ni_floor:
    case ni_ceiling:
        break;
    default:
        *exist = 0;
        break;
    }
    return 0;
}


avator_info_t *dining_get_avator_info(noun_id_t nid){
    int i;
    for (i = 0 ; i < AVATOR_INFO_NUM ; i++){
        if (avator_infos[i].nid == nid){
            printf("avator found i=%d nid=%d fid=%d pos=%d\n", i, nid, avator_infos[i].fid, avator_infos[i].pos);
            return &avator_infos[i];
        }
    }
    assert(0); // 置けるものについてのみこの関数を呼ぶはず
    return NULL;
}

noun_id_t get_avator_nid(avator_pos_t pos){
    int i;
    for (i = 0 ; i < AVATOR_INFO_NUM ; i++){
        if (is_flag_on(avator_infos[i].fid) && avator_infos[i].pos == pos){
            //            printf("avator found i=%d nid=%d fid=%d pos=%d\n", i, nid, avator_infos[i].fid, avator_infos[i].pos);
            return avator_infos[i].nid;
        }
    }
    return ni_invalid;
}


void dining_put_avator(noun_id_t nid){
    avator_info_t *avator_info;

    avator_info = dining_get_avator_info(nid);
    assert(avator_info != NULL);

    if (is_flag_on(avator_flags[avator_info->pos])){
        message("すでに別のものが置かれている。");
        return;
    }
    command_put_item(nid, avator_info->fid, NULL);
    set_flag(avator_flags[avator_info->pos]);
    return ;
}

int is_avator_put(noun_id_t nid){
    avator_info_t *avator_info;

    avator_info = dining_get_avator_info(nid);
    assert(avator_info != NULL);

    printf("is_avator_put:%d\n", is_flag_on(avator_info->fid));

    return is_flag_on(avator_info->fid);
}

void dining_get_avator(noun_id_t nid){
    avator_info_t *avator_info;

    if (nid == ni_toy_swordman){
        Set(toy_swordman_taken);
    }

    avator_info = dining_get_avator_info(nid);
    assert(avator_info != NULL);

    reset_flag(avator_info->fid);
    reset_flag(avator_flags[avator_info->pos]);
    get_item(nid);
    message("%nnを取った。", nid);
}

void dining_look_chair(void){
    int m=1, n, i;
    noun_id_t nid;

    message("4脚の椅子がある。\\");
    n = count_on_flags(4, 
                       fi_dining_put_man,
                       fi_dining_put_woman,
                       fi_dining_put_girl,
                       fi_dining_put_boy);
    //                message("%d個のものがある。", n);
    switch (n){
    case 0:
        message("");
        break;
    case 1:
        for (i = ap_begin ; i < ap_end ; i++){
            if ((nid = get_avator_nid(i)) != ni_invalid) break;
        }
        assert(nid != ni_invalid);
        message("椅子の一つには%nlが置いてある。", nid);
        break;
    case 2:
        for (i = ap_begin ; i < ap_end ; i++){
            if ((nid = get_avator_nid(i)) != ni_invalid){
                if (m == 1){
                    message("椅子の上には%nlと\\", nid);
                    m++;
                }
                else {
                    message("%nlが置いてある。", nid);
                    break;
                }
            }
        }
        break;
    case 3:
    case 4:
        message("椅子の上には\\");
        for (i = ap_begin ; i < ap_end ; i++){
            if ((nid = get_avator_nid(i)) != ni_invalid){
                if (m < n){
                    message("%nl、\\", nid);
                    m++;
                }
                else {
                    message("%nlが置いてある。", nid);
                    break;
                }
            }
        }
        break;
    default:
        assert(0);
    }
    return;
}
#ifdef DUMMY
            // !!! げんざいのっているものを表示
			if (Not(toy_swordman_taken)){
				message("椅子の一つには%nlが乗っている。",
						ni_toy_swordman);
			}
            else {
                message("椅子の上にはなにも乗っていない。");
            }
			return 1;
		default:
			break;
		}
#endif


void necromancy(void){
    int i, avator_check = 0;

    for (i = 0 ; i < 4 ; i++){
        if (is_flag_on(correct_avators[i])){
            avator_check += (1 << i);
        }
    }
    printf("avator check value:%d\n", avator_check);

    switch (avator_check){
    case 1+2+4+8: // 全部正解
		if (Not(chicken_put)){
			message("誰かが空腹に耐えかねて呻いたような気がした。");
			return;
		}
		if (Is(candle_taken)){
			message("誰かが無念そうな溜息を漏らした気がした。");
			return;
		}
		// 降霊中、いったん依代の表示を消す
        for (i = 0 ; i < 4 ; i++){
            reset_flag(correct_avators[i]);
            //		Reset(necklace_put);
            //		Reset(plush_cat_put);
            //		Reset(large_robot_put);
        }
		Set(candle_lit);
		Set(ghost_appeared);
		message("音もなく、幾人かの人影が食堂に入ってきた。");
		message("中年の男女と子供たち…\\");
        if (Is(photo_checked)){
            message("写真のヨシノ教授一家だ。");
        }
        else {
            message("誰だろう。");
        }
		message("彼らが食卓につくと蝋燭が灯り、"
				"食事が始まった。"
				"楽しげに会話を交わしているようだが、"
				"明らかにこの世の者ではない彼らの声は、あなたには聞こえない。");
		set_wait_return_callback(necromancy_end);
        break;
    case 2+4+8:
		message("かすかに、男性のすすり泣くような声が聞こえる。");
        break;
    case 1+4+8:
		message("かすかに、女性のすすり泣くような声が聞こえる。");
        break;
    case 1+2+8:
		message("かすかに、少女のすすり泣くような声が聞こえる。");
        break;
    case 1+2+4:
		message("かすかに、少年のすすり泣くような声が聞こえる。");
        break;
    default:  // 依代が2つ以上不足
		message("かすかに、複数の人がすすり泣くような声が聞こえる。");
        break;
    }
}
void necromancy_end(void){
    int i;

    message("やがて食事が終わると、\\");
    if (Is(photo_checked)){
        message("夫人\\");
    }
    else {
        message("女性\\");
    }
    message("と子供たちは食堂を出て行った。"
            "残った\\");
    if (Is(photo_checked)){
        message("ヨシノ教授\\");
    }
    else {
        message("男性\\");
    }
    message("は何か言いたげに蝋燭を指すと、"
            "妻子の後を追った。");


	// 依代は残る
    for (i = 0 ; i < 4 ; i++){
        set_flag(correct_avators[i]);
    }
    //	Set(glasses_put);
    //	Set(necklace_put);
    //	Set(plush_cat_put);
    //	Set(large_robot_put);
	// 食事が終わってから丸焼きを骨に変える
	Reset(chicken_put);
	// 幽霊退場
	Reset(ghost_appeared);
}
