#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

int area_entrance(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
    *exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
            message("入り口は重々しい木製の扉%nwで閉ざされている。", ni_door);
            if (Not(flowerpot_destroyed)){
                message("扉の脇には%nlがある。", ni_flowerpot);
            }
            else {
                message("扉の脇には%nlの残骸がある。", ni_flowerpot);
            }
            return 1;
        case vi_forward:
            message("扉が閉まっている。");
            return 1;
        case vi_backward:
            move_area(ai_mailbox);
            return 1;
        case vi_left:
            move_area(ai_garden);
            return 1;
        case vi_right:
            move_area(ai_opened_window);
            return 1;
        default:
            break;
        }
        break;
    case ni_wall:
        break;
    case ni_door:
        switch(vid){
        case vi_look:
            message("分厚い扉だ。壊すのは無理だろう。");
            return 1;
        case vi_examine:
            msg_nothing_special();
            return 1;
        case vi_open:
        case vi_push:
        case vi_pull:
            if (Is(entrance_unlocked)){
                message("重い扉を開いて屋敷内に踏み込んだ。");
                message("バタン。数歩踏み出すと、"
                            "背後で扉が大きな音を立てて閉まった。");
                Set(entered_house);
                move_area(ai_entrance_hall);
            }
            else {
                message("鍵が掛かっていてびくともしない。");
            }
            return 1;
        case vi_close:
            message("最初から閉まっている。");
            return 1;
        case vi_hit:
            message("扉を叩いてみたが、誰も答えない。");
            return 1;
        default:
            break;
        }
        break;
    case ni_flowerpot:
        switch(vid){
        case vi_look:
            if (Is(flowerpot_destroyed)){
                message("砕けた植木鉢の残骸だ。");
                if (!has_item(ni_brass_key)){
                    message("残骸の中に%nlがある。", ni_brass_key);
                }
            }
            else {
                message("素焼きの植木鉢だ。"
                        "植えられていた植物は枯れてしまっている。"
						"かつては右側にも一つあったらしく、"
						"破片が散らばっている。");
            }
            if ((Is(flowerpot_moved) ||
				 Is(flowerpot_destroyed)) &&
                 !has_item(ni_wooden_key))
            {
                message("植木鉢のあったところに%nlが落ちている。", 
                        ni_wooden_key);
            }
            return 1;
        case vi_get:
            if (Is(flowerpot_destroyed)){
                message("すでに粉々になっている。");
            }
            else {
                message("持って行くには重すぎる。");
            }
            return 1;
        case vi_move:
        case vi_push:
        case vi_pull:
            if (Is(flowerpot_moved)){
                message("植木鉢はもう動かしてある。");
            }
			else if (Is(flowerpot_destroyed)){
                message("もう粉々だ。");
			}
            else {
                message("植木鉢の下から%nlが出てきた。", ni_wooden_key);
                Set(flowerpot_moved);
            }
            return 1;
        case vi_destroy:
        case vi_hit:
        case vi_kick:
            TestAndSet(flowerpot_destroyed,
                       "ガシャン。植木鉢を叩き壊した。",
                       "植木鉢はすでに壊れている。");
            return 1;
        default:
            break;
        }
        break;
    case ni_brass_key:
        if (has_item(nid)){
            switch(vid){
            case vi_use:
                TestAndSet(entrance_unlocked,
                           MSG_DOOR_UNLOCKED,
                           MSG_DOOR_ALREADY_UNLOCKED);
                lose_item(nid);
                return 1;
            default:
                break;
            }
            return 0;
        } 
        if (Is(flowerpot_destroyed)){
            switch(vid){
            case vi_look:
                message(noun_description(nid));
                return 1;
            case vi_get:
                command_get_item(ni_brass_key, fi_brass_key_taken);
                return 1;
            default:
                break;
            }
        }
        else { // 植木鉢を破壊していない
            *exist = 0;
        }
        break;
    case ni_wooden_key:
        if (has_item(ni_wooden_key)){
            switch(vid){
            case vi_use:
                msg_key_unmatched();
                return 1;
            default:
                break;
            }
			*exist = 0;
            return 0;
        } 
        if ((Is(flowerpot_moved) ||
             Is(flowerpot_destroyed)))
        {
            switch(vid){
            case vi_look:
                message(noun_description(nid));
                return 1;
            case vi_get:
                command_get_item(ni_wooden_key, fi_wooden_key_taken);
                return 1;
            default:
                break;
            }
        }
        else { // 植木鉢を動かしていない
			*exist = 0;
        }
        break;
    default:
        *exist = 0;
        break;
    }
    return 0;
}
