#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

int area_kitchen(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
			message("正面には%nlがあり、その左右に%nlと%nlがある。",
				    ni_counter, ni_cupboard, ni_icebox);
            message("左右に扉%nw%nwがある。",
                    ni_left_door, ni_right_door);
            if (Is(dining_kitchen_door_opened) &&
                Is(kitchen_corridor_door_opened))
            {
                message("左右の扉はそれぞれ食堂と廊下に通じている。");
            }
            else if (Is(dining_kitchen_door_opened)){
                message("左側の扉は食堂に通じている。");
            }
            else if (Is(kitchen_corridor_door_opened)){
                message("右側の扉は廊下に通じている。");
            }
            return 1;
        case vi_left:
            EnterDoor(dining, dining_kitchen);
            return 1;
        case vi_right:
            EnterDoor(corridor_1f_br, kitchen_corridor);
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_door:
        msg_which_door();
        return 1;
    case ni_left_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_left, ni_invalid);
		}
        break;
    case ni_right_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_right, ni_invalid);
		}
        break;
    case ni_counter:
        switch (vid){
        case vi_look:
            message("%nlと%nlが据え付けられている。",
                    ni_sink, ni_range);
            return 1;
        default:
            break;
        }
        break;
    case ni_sink:
        switch (vid){
        case vi_use:
            message("蛇口をひねっても水は出ない。");
            return 1;
        default:
            break;
        }
        break;
    case ni_range:
        switch (vid){
        case vi_look:
            message("コンロの下に%nlが内蔵されている。",
                    ni_oven);
            return 1;
        case vi_use:
            message("コンロはつかない。ガスが来ていないようだ。");
            return 1;
        default:
            break;
        }
        break;
    case ni_oven:
        switch (vid){
    	case vi_look:
    	case vi_examine:
			if (Is(oven_opened)){
				if (Not(chicken_taken)){
					message("オーブンの中に%nlがある。", ni_chicken);
				}
				else {
					message("オーブンの中には、もう何もない。");
				}
				return 1;
			}
			else {  // 蓋が閉まっているならデフォルトのlook処理
				return 0;
			}
        case vi_use:
            message("オーブンはつかない。ガスが通じてないようだ。");
            return 1;
        case vi_open:
            TestAndSet(oven_opened,
                       "オーブンを開けた。",
                       "オーブンはもう開いている。");
            return 1;
        case vi_close:
            TestAndReset(oven_opened,
                         "オーブンを閉じた。",
                         "オーブンは閉まっている。");
            return 1;
        default:
            break;
        }
        break;
	case ni_chicken:
		if (Is(chicken_taken)){
			*exist = 0;
			return 0;
		}
		if (Is(oven_opened)){
	        switch(vid){
			case vi_get:
                command_get_item(ni_chicken, fi_chicken_taken);
				return 1;
	        default:
	            break;
	        }
		}
		else { // オーブンが開いていない
			*exist = 0;
		}
		break;
    case ni_icebox:
        switch (vid){
        case vi_examine:
            message("中は冷えていない。電気が通じていないようだ。");
            return 1;
        case vi_open:
            message("中には干からびた野菜屑ぐらいしかない。");
            return 1;
        default:
            break;
        }
        break;
    case ni_cupboard:
        switch (vid){
        case vi_look:
        case vi_examine:
            message("中の食器類は粉々に割れてしまっている。");
            return 1;
        case vi_open:
            message("中には使えそうなものはない。");
            return 1;
        default:
            break;
        }
        break;
    default:
        *exist = 0;
        break;
    }
    return 0;
}

