#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

int describe_rods(void);
void msg_dog_growl(void);
int command_callback_talk_dog(char *line);

int area_laundry(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
            message("正面と背後に扉%nw%nwがある。\\",
                    ni_front_door, ni_back_door);
            if (Is(dog_stood_up)){
                message("床には%nlがある。\\",
                        ni_hatch);
            }
            message("背後の扉は廊下に\\");
            if (Is(laundry_cellar_door_opened)){
                message("、床のハッチは地下室に、それぞれ\\");
            }
            message("続いている。");
            message("左側には%nlと%nlがある。",
                    ni_washer, ni_dryer);
            if (Not(dog_stood_up)){
                message("右側奥には%nlが座っている。", ni_dog);
            }
            else if (Not(dog_gone)){
                message("奥には%nlが立っている。", ni_dog);
            }
            return 1;
        case vi_forward:
            message("この扉は開けられそうにない。");
            return 1;
        case vi_backward:
            EnterDoor(corridor_1f_br, laundry_corridor);
            return 1;
        case vi_down:
            if (Is(dog_stood_up)){
                if (has_light()){
#ifndef DEBUG
                    Set(save_forbidden);
#endif
                    EnterDoor(cellar, laundry_cellar);
                }
                else {
                    msg_no_light();
                }
            }
            else {
                msg_dog_growl();
            }
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_door:
        msg_which_door();
        return 1;
    case ni_front_door:
        switch (vid){
        case vi_look:
            message("内側から板が打ちつけられている。");
            return 1;
        case vi_open:
        case vi_push:
        case vi_pull:
        case vi_destroy:
        case vi_examine:
            message("厳重に封じられている。これを開けるのは無理そうだ。");
            return 1;
        default:
            break;
        }
        break;
    case ni_back_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_backward, ni_invalid);
			return 1;
		}
        break;
    case ni_hatch:
        switch (vid){
        case vi_look:
            message("階下への入り口のようだ。");
            return 1;
        case vi_examine:
			if (Is(dog_stood_up)){
                message("手で開けられる簡単なハッチだ。"
                        "その下には地下室への階段があるようだ。");
			}
			else {
                msg_dog_growl();
			}
            return 1;
        case vi_open:
        case vi_enter:
        case vi_pull:
            RedoArea(vi_down, ni_invalid);
			return 1;
        default:
            break;
        }
        break;
    case ni_washer:
        switch (vid){
        case vi_look:
			if (Is(washer_opened)){
	            message("中には%nl一式が入っている。",
					    ni_clothes);
                describe_rods();
			}
			else {
                message("ありふれた電動の洗濯機だ。");
                describe_rods();
			}
            return 1;
		case vi_examine:
			if (Is(washer_opened)){
	            message("中には%nl一式が入っている。",
					    ni_clothes);
			}
			else {
				message("何か入っているようだ。");
			}
            return 1;
        case vi_open:
			if (Not(washer_opened)){
				message("%nnの蓋を開けた", ni_washer);
				Set(washer_opened);
			}
			else {
				message("%nnの蓋はもう開いている", ni_washer);
			}
			return 1;
        case vi_close:
			if (Is(washer_opened)){
				message("%nnの蓋を閉めた", ni_washer);
				Reset(washer_opened);
			}
			else {
				message("%nnの蓋は閉まっている", ni_washer);
			}
			return 1;
        case vi_use:
            message("電源がつかない。");
            return 1;
        default:
            break;
        }
        break;
    case ni_dryer:
        switch (vid){
        case vi_look:
			if (Is(dryer_opened)){
	            message("中は空だ。");
			}
			else {
                message("ありふれた電動の乾燥機だ。");
			}
            return 1;
		case vi_examine:
			if (Is(dryer_opened)){
	            message("何も入っていない。");
			}
			else {
                message("何も入っていないようだ。");
			}
            return 1;
        case vi_open:
			if (Not(dryer_opened)){
				message("%nnの蓋を開けた", ni_dryer);
				Set(dryer_opened);
			}
			else {
				message("%nnの蓋はもう開いている", ni_dryer);
			}
			return 1;
        case vi_close:
			if (Is(dryer_opened)){
				message("%nnの蓋を閉めた", ni_dryer);
				Reset(dryer_opened);
			}
			else {
				message("%nnの蓋は閉まっている", ni_dryer);
			}
			return 1;
        case vi_use:
            message("電源がつかない。");
            return 1;
        default:
            break;
        }
        break;
	case ni_clothes:
        switch (vid){
        case vi_look:
			message("男性用の衣類一式だ。靴まである。");
			if (Is(man_checked)){
				message("写真のヨシノ教授が着ていた服に似ている。");
			}
			return 1;
        case vi_examine:
			if (Not(bronze_key_found)){
				message("上着のポケットに%nlがあった",
						ni_bronze_key);
				Set(bronze_key_found);
			}
			else {
				message("他には何もないようだ。");
			}
			return 1;
        case vi_get:
			message("着替えは必要ないだろう。");
			return 1;
		default:
			break;
		}
		break;
    case ni_fishing_rod:
        ReturnIfHasItem;
        if (Is(fishing_rod_taken)) ReturnNotExist;
        switch (vid){
        case vi_get:
            command_get_item(nid, fi_fishing_rod_taken);
            return 1;
		default:
			break;
		}
		break;
    case ni_poker:
        ReturnIfHasItem;
        if (Is(poker_taken)) ReturnNotExist;
        switch (vid){
        case vi_get:
            command_get_item(nid, fi_poker_taken);
            return 1;
		default:
			break;
		}
		break;
	case ni_bronze_key:
		if (Not(bronze_key_found) || Is(bronze_key_taken)){
			*exist = 0;
			return 0;
		}
		if (vid == vi_get){
			command_get_item(nid, fi_bronze_key_taken);
			return 1;
		}
		break;
    case ni_dog:
		if (Is(dog_gone)){
			*exist = 0;
			return 0;
		}
        switch (vid){
        case vi_look:
            message("%nlをしている。飼い犬のようだ。",
                    ni_belt);
			return 1;
        case vi_examine:
            message("おとなしく調べさせてくれそうにない。");
            return 1;
        case vi_talk:
            message("何を話す?");
            set_command_callback(command_callback_talk_dog);
            set_term_prompt(QUERY_PROMPT);
            return 1;
		case vi_hit:
		case vi_touch:
		case vi_push:
		case vi_pull:
			message("あなたの手は空を切った。");
			return 1;
		case vi_kick:
			message("あなたの足は空を切った。");
			return 1;
        default:
            break;
        }
        break;
    case ni_hat:
        if (!has_item(nid)) ReturnNotExist;
        switch (vid){
        case vi_use:
        case vi_throw:
        case vi_put:
        case vi_give:
        case vi_show:
            message("帽子を目の前におくと、犬はくんくんと匂いを嗅いだ。"
                    "そして一声悲しげな声をあげると、すうっと消えていった。");
            lose_item(ni_hat);
            Set(dog_gone);
            return 1;
        default:
            break;
        }
    case ni_chicken:
    case ni_chicken_bone:
        if (!has_item(nid)) ReturnNotExist;
        switch (vid){
        case vi_use:
        case vi_throw:
        case vi_put:
        case vi_give:
        case vi_show:
			message("犬はちらりと%nnを見たが、"
					"そのままそっぽを向いてしまった。",
					nid);
            return 1;
        default:
            break;
        }
    case ni_belt:
        if (Is(belt_taken)) ReturnNotExist;
		if (Not(dog_gone)){
			RedoArea(vi_examine, ni_dog);
		}
        switch (vid){
        case vi_look:
            message("オリオンの首輪だ。");
            return 1;
        case vi_get:
            command_get_item(ni_belt, fi_belt_taken);
            return 1;
        default:
            break;
        }
        break;
    case ni_wall:
        switch (vid){
        case vi_look:
            if (describe_rods() == 0){
                msg_nothing_any_more();
            }
            return 1;
        default:
            break;
        }
        break;
    case ni_floor:
        switch (vid){
        case vi_look:
        case vi_examine:
            if (Is(laundry_cellar_door_opened)){
                message("右手の床には地下室への%nlがある。",
                        ni_hatch);
            }
            else {
                message("右手の床に%nlがある。", ni_hatch);
            }
            if (Is(dog_gone) && Not(belt_taken)){
                message("床の上に%nlが落ちている", ni_belt);
            }
            return 1;
        default:
            break;
        }
        break;
    case ni_ceiling:
        break;
    default:
        *exist = 0;
        break;
    }
    return 0;
}


int describe_rods(void){
    if (Not(fishing_rod_taken)){
        message("洗濯機の前に、%nlが立てかけてある。",
                ni_fishing_rod);
        return 1;
    }
    else if (Not(poker_taken)){
        message("洗濯機の前に、%nlが立てかけてある。",
                ni_poker);
        return 1;
    }
    return 0;
}

void msg_dog_growl(void){
    message("%nnの上に座り込んでいる犬が、"
            //            "唸り声を上げてこちらを睨んでいる。",
            "こちらを睨んでいる。",
            ni_hatch);
}

// 犬がまだいることが前提。昇天後はarea_laundry()側ではじく
int command_callback_talk_dog(char *line){
    char *noun;
    noun_id_t nid;

    if (*line != '\0'){
        parse_noun(line, &noun);
        nid = get_noun_id(noun);
        switch (nid){
        case ni_orion:
            if (Not(dog_stood_up)){
                message("名前を呼ぶと、"
                        "それまでこちらに無関心だった犬が立ち上がった。");
                Set(dog_stood_up);
            }
            else {
                message("心なしか嬉しげに、ゆっくりと尾を振っている。");
            }
            break;
        default:
            if (Is(dog_stood_up)){
                message("じっとこちらを見つめている。");
            }
            else {
                message("そっぽを向いたまま、何の反応も示さない。");
            }
            break;
        }
    }
    //    clear_command_callback();
    //    set_console_prompt(NULL);
    return 0;
}
