#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

void stop_phone_call(void);
void describe_silver_key(void);
void describe_fireplace_msg(void);
int command_callback_adjust_clock(char *line);
	
int area_living(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
    *exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
			message("正面には%nlがあり、"
					"その左側には%nlが、"
					"右側の隅には%nlがある。",
					ni_fireplace, ni_tv, ni_telephone);
            message("右側と背後に扉%nw%nwが\\",
                    ni_right_door, ni_back_door);
            if (Is(dining_living_door_opened) &&
                Is(living_corridor_door_opened))
            {
                message("あり、それぞれぞれ食堂と廊下に通じている。");
            }
            else if (Is(dining_living_door_opened)){
                message("あり、右側の扉は食堂に通じている。");
            }
            else if (Is(living_corridor_door_opened)){
                message("あり、背後の扉は廊下に通じている。");
            }
            else {
                message("ある。");
            }
            return 1;
        case vi_backward:
		case vi_exit:
			stop_phone_call();
            EnterDoor(corridor_1f_bl, living_corridor);
            return 1;
        case vi_right:
			stop_phone_call();
            EnterDoor(dining, dining_living);
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_wall:
		switch (vid){
		case vi_look:
			message("%nnの上の壁には%nlが掛かっている。",
					ni_fireplace, ni_clock);
			return 1;
		default:
			break;
		}
		break;
    case ni_floor:
		switch (vid){
		case vi_look:
			message("%nnの脇に%nlが転がっている。",
					ni_tv, ni_videotape);
			return 1;
		default:
			break;
		}
        break;
    case ni_videotape:
		switch (vid){
		case vi_look:
		case vi_examine:
			message("市販のビデオソフトだ。");
			message("一本はロボットアニメのようで、"
                    "パッケージにはドラゴンとロボットが合体する絵が描かれている。"
					"もう一本は子供向けの漫画映画で、"
                    "茶色い鼠が%nnで灰色の猫を狙っている絵が描かれている。",
                    ni_slingshot);
			return 1;
		case vi_get:
            message("役に立ちそうにない。");
            return 1;
		default:
			break;
		}
        break;
    case ni_door:
        msg_which_door();
        return 1;
    case ni_back_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_backward, ni_invalid);
        }
        break;
    case ni_right_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_right, ni_invalid);
        }
        break;
	case ni_fireplace:
		switch(vid){
        case vi_look:
			message(noun_description(nid));
            if (Not(fossil_taken)){
                message("暖炉の上には%nlと%nlが飾ってある。", 
                        ni_photo, ni_fossil);
            }
            else {
                message("暖炉の上には%nlが飾ってある。", 
                        ni_photo);
            }
            describe_silver_key();
            describe_fireplace_msg();
			return 1;
        case vi_examine:
			message("暖炉の火は消えている。"
					"薪は燃え尽きて%nlだけが積もっている。", 
					ni_ash);
            describe_silver_key();
            describe_fireplace_msg();
			return 1;
        case vi_read:
            if (Is(silver_key_taken)){
                message("「奴は火を嫌う」");
                message("なんのことだろう。");
                Set(fireplace_message_read);
            }
            else {
                msg_cannot_read();
            }
            return 1;
		default:
			break;
		}
        break;
	case ni_clock:
		switch (vid){
		case vi_look:
			message("鳩時計のようだ。\\");
			if (Not(clock_working)){
				message("%nlの動きは止まっている。",
                        ni_clock_hands);
			}
			else {
				message("%nlはカチカチと時を刻んでいる。",
                        ni_clock_hands);
			}
			if (Is(clock_bird_appeared)){
				if (Not(toy_bird_taken)){
                    message("上の小窓から%nlが飛び出している。",
                            ni_toy_bird);
                }
                else {
                    message("上の小窓から飛び出たバネには、"
                            "もう何も乗っていない。");
                }
            }
			return 1;
		case vi_examine:
			message("この時計は錘式だ。"
					"下に錘のついた%nlが下がっている",
					ni_chain);
			return 1;
		case vi_adjust:
            message("何時何分に合わせる?");
			msg_clock_adjusting_guide();
			set_command_callback(command_callback_adjust_clock);
			set_term_prompt(QUERY_PROMPT);
			return 1;
		default:
			break;
		}
        break;
    case ni_clock_hands:
        switch (vid){
        case vi_move:
        case vi_turn:
            RedoArea(vi_adjust, ni_clock);
        default:
            break;
        }
        break;
	case ni_chain:
		switch (vid){
		case vi_look:
			message("時計から何本か%nnが下がっており、"
                    "いくつかには錘が付いている。",
                    ni_chain);
			return 1;
		case vi_examine:
			message("錘の重量で時計を動かす仕組みだ。");
			return 1;
		case vi_pull:
		case vi_move:
			if (Not(clock_working)){
				message("鎖を引いて錘を巻き上げると、時計は動きだした。");
				Set(clock_working);
				if (Is(clock_adjusted)){
					Set(clock_striking);
				}
			}
			else {
				message("時計はまだ動いている。"
						"しばらくは鎖を引く必要はないだろう。");
			}
			return 1;
		default:
			break;
		}
        break;
	case ni_toy_bird:
		ReturnIfHasItem;
        if (Not(clock_bird_appeared)){
			ReturnNotExist;
		}
		switch (vid){
		case vi_get:            
			command_get_item(nid, fi_toy_bird_taken);
			return 1;
		default:
			break;
		}
		break;
	case ni_tv:
		switch (vid){
        case vi_look:
            message(noun_description(nid));
			message("脇に%nlが転がっている。",
					ni_videotape);
            return 1;
        case vi_use:
			message("電源スイッチを押しても反応しない。"
					"電気が通じていないようだ。");
			return 1;
        case vi_get:
			message("こんな古くて重いテレビは要らない。");
			return 1;
		default:
			break;
		}
        break;
	case ni_telephone:
		switch(vid){
        case vi_examine:
			message("電話機に変わったところはないが、"
					"壁から伸びる電話線が途中で切断されている。");
			if (Not(phone_call_stopped)){
                message("ふと気づくと電話は沈黙していた。");
                Set(phone_call_stopped);
			}
			return 1;
        case vi_use:
			if (Not(phone_call_stopped)){
				msg_answer_phone_call();
				Set(phone_call_answered);
				Set(phone_call_stopped);
			}
			else {
				message("電話は沈黙している。");
			}
			return 1;
		default:
			break;
		}
        break;
	case ni_photo:
		switch(vid){
		case vi_look:
			message(noun_description(nid));
			message("夫婦らしい中年の男女%nw%nwと、"
					"姉弟らしい子供二人%nw%nwが写っている。",
					ni_man, ni_woman, ni_girl, ni_boy);
			Set(photo_checked);
			return 1;
		case vi_examine:
			message("写真の裏に、なにか書いてある。");
			return 1;
		case vi_read:
			message("「1995年秋 アイシャ、ミリカ、アーサーと共に」");
			return 1;
		case vi_get:
			message("これはそっとしておこう。");
			return 1;
		case vi_touch:
			message("写真に触ってみたが、\\");
			msg_no_effect();
			return 1;
		case vi_hit:
		case vi_kick:
		case vi_destroy:
		case vi_push:
		case vi_pull:
			message("写真にあたるのはやめよう。");
			return 1;
		default:
			break;
		}
        break;
	case ni_fossil:
        //        ReturnIfHasItem;
		if (Is(fossil_taken)){
			*exist = 0;
			return 0;
		}
		switch(vid){
        case vi_get:
            command_get_item(nid, fi_fossil_taken);
            return 1;
		default:
			break;
        }
        break;
	case ni_ash:
		switch (vid){
        case vi_look:
            message(noun_description(nid));
            describe_silver_key();
            return 1;
		case vi_examine:
		case vi_move:
		case vi_remove:
            if (Is(silver_key_found)){
                msg_nothing_any_more();
            }
            else {
                message("何か、かき回す道具を見つけてきた方がいい。");
            }
			return 1;
		default:
			break;
		}
        break;
    case ni_poker:
        if (!has_item(nid)) ReturnNotExist;
        switch (vid){
        case vi_use:
            if (Not(silver_key_found)){
                message("%nnをかき回すと%nlが出てきた。",
                        ni_ash, ni_silver_key);
                Set(silver_key_found);
                lose_item(nid);
            }
            else {
                msg_nothing_any_more();
            }
            return 1;
		default:
			break;
		}
        break;
    case ni_silver_key:
        ReturnIfHasItem;
        if (Not(silver_key_found) || Is(silver_key_taken)) ReturnNotExist;
        switch (vid){
        case vi_get:
            command_get_item(nid, fi_silver_key_taken);
            return 1;
		default:
			break;
		}
		break;
	case ni_man:
		switch (vid){
        case vi_look:
            message(noun_description(nid));
			Set(man_checked);
            return 1;
		default:
			break;
		}
		break;
	case ni_woman:
		switch (vid){
        case vi_look:
            message(noun_description(nid));
			Set(woman_checked);
            return 1;
		default:
			break;
		}
		break;
	case ni_girl:
		switch (vid){
        case vi_look:
            message(noun_description(nid));
			Set(girl_checked);
            return 1;
		default:
			break;
		}
		break;
	case ni_boy:
		switch (vid){
        case vi_look:
            message(noun_description(nid));
			Set(boy_checked);
            return 1;
		default:
			break;
		}
		break;
    case ni_ceiling:
    case ni_window:
        break;
    default:
		*exist = 0;
        break;
    }
    return 0;
}


void stop_phone_call(void){
	if (Not(phone_call_stopped)){
		Set(stop_phone_call);
	}
}

void post_living(int time_passed){
    // 電話イベント
    if (Is(phone_call_stopped)){
    }
    else if (Is(stop_phone_call)){
        msg_phone_call_stopped();
        Set(phone_call_stopped);
        Reset(stop_phone_call);
    }
    else if (Is(phone_call_continued)){
        msg_phone_call_continued();
    }
    // 時計イベント
    if (time_passed){
        if (Is(clock_striking)){
            msg_clock_door_opened();
            Set(clock_bird_appeared);
            Reset(clock_striking);
            Reset(clock_adjusted);
        }
        else if (Is(clock_bird_appeared)){
            msg_clock_door_closed();
            Reset(clock_bird_appeared);
        }
    }
}

void describe_silver_key(void){
    if (Is(silver_key_found) && Not(silver_key_taken)){
        message("%nnの中に%nlがある。",
                ni_ash, ni_silver_key);
    }
}
void describe_fireplace_msg(void){
    if (Is(silver_key_taken)){
        message("%nnがあったあたりに、なにか彫ってある。",
                ni_silver_key);
    }
}

int command_callback_adjust_clock(char *line){
	int i, t, h, m;

    // 数字を入れずにリターンキーで入力中止
	if (strlen(line) == 0){
        message("時計を合わせるのをやめた。");
		return 0;
	}
	if (strlen(line) != 4){
		msg_clock_adjusting_guide();
		return 1;
	}
	for (i = 0 ; i < 4 ; i++){
		if (!isdigit(line[i])){
			msg_clock_adjusting_guide();
            //			clear_command_callback();
            //			set_console_prompt(NULL);
			return 1;
		}
	}
	t = atoi(line);
	h = t / 100;
	m = t % 100;
	if (h >= 24 || m >= 60){
		msg_clock_adjusting_guide();
        //		clear_command_callback();
        //		set_console_prompt(NULL);
		return 1;
	}
	message("時計を%d時%d分に合わせた。", h, m);
	if (m == 0){
		set_flag(fi_clock_adjusted);
		if (is_flag_on(fi_clock_working)){
			set_flag(fi_clock_striking);
            //			set_post_message(msg_clock_door_opened);
		}
	}
    //    clear_command_callback();
    //    set_console_prompt(NULL);
    return 0;
}


