#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

void music_room_move_backward(void);

int area_music_room(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
			message("部屋の中には%nlと%nlがある。",
					ni_piano, ni_chair);
			message("右側手前には子供部屋に続く扉%nwがある。",
					ni_right_door);
			message("正面と背後には扉%nw%nwが\\", 
					ni_front_door, ni_back_door);
			if (Is(balcony_door_opened) && Is(music_room_door_opened)){
				message("あり、それぞれバルコニーと廊下に続いている。");
			}
			else if (Is(balcony_door_opened)){
				message("あり、正面の扉はバルコニーに続いている。");
			}
			else if (Is(music_room_door_opened)){
				message("あり、背後の扉は廊下に続いている。");
			}
			else {
	            message("ある。");
			}
			message("左側にも扉%nwが\\", 
					ni_left_door);
			if (Is(bathroom_2f_door_opened)){
				message("あり、浴室に続いている。");
			}
            else {
	            message("ある。");
            }
            return 1;
        case vi_forward:
			EnterDoor(balcony, balcony);
            return 1;
        case vi_backward:
		case vi_exit:
			music_room_move_backward();
            return 1;
        case vi_left:
		    EnterDoor(bathroom_2f, bathroom_2f);
            return 1;
        case vi_right:
		    EnterDoor(girls_room, music_room_right);
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_door:
        msg_which_door();
        return 1;
    case ni_front_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_forward, ni_invalid);
		}
        break;
    case ni_back_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_backward, ni_invalid);
		}
        break;
    case ni_left_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_left, ni_invalid);
		}
        break;
    case ni_right_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_right, ni_invalid);
		}
        break;
	case ni_piano:
		switch (vid){
		case vi_examine:
			if (Not(score_played)){
				msg_nothing_special();
			}
			else {
				message("Dの音のハンマーに%nlが結わえられている。",
						ni_steel_key);
				Set(steel_key_found);
			}
			return 1;
		case vi_use:
			if (Not(score_set)){
				message("あなたは「ねこふんじゃった」を弾いた。");
			}
			else {
				message("あなたは楽譜を見ながらなんとか「エリーゼのために」を弾いた。");
				message("おや、Dの音が鳴っていない。");
				Set(score_played);
			}
			return 1;
		default:
			break;
		}
		break;
	case ni_steel_key:
		if (has_item(nid)){
			*exist = 0;
			return 0;
		}
		switch (vid){
		case vi_get:
			if (Is(steel_key_found)){
				command_get_item(nid, fi_steel_key_taken);
				return 1;
			}
			else {
				ReturnNotExist;
			}
		default:
			break;
		}
		break;
	case ni_score:
        if (has_item(nid)){
            switch (vid){
            case vi_put:
				TestAndSet(score_set,
                           "あなたは譜面台に楽譜を置いた。",
                           "楽譜はもう置いてある。");
                lose_item(nid);
                return 1;
            default:
                break;
            }
            return 0;
        }
		else {
			if (Not(score_set)){
				ReturnNotExist;
			}
            switch (vid){
            case vi_get:
                message("もう持ち歩く必要もないだろう。");
                return 1;
            case vi_read:
				// 楽譜台に置いた後も読めるように
				command_read_item(nid);
                return 1;
            default:
                break;
            }
		}
        break;
    case ni_chair:
    case ni_wall:
    case ni_floor:
    case ni_ceiling:
	case ni_window:
        break;
    default:
		*exist = 0;
        break;
    }
    return 0;
}

void music_room_move_backward(void){
	if (Not(music_room_door_unlocked)){
        msg_door_unlocked_by_opening();
        Set(music_room_door_unlocked);
    }
    enter_door(ai_corridor_2f_l, fi_music_room_door_opened);
}
