#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

int area_secret_room(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
            message("部屋の中央には怪しい%nlがある。"
                    "隅には小さな%nlがある。"
                    "奥の壁には%nlがもたれかかっている。",
                    ni_magic_circle, ni_table, ni_skeleton);
            message("背後には図書室に続く隠し扉がある。");
            return 1;
        case vi_backward:
		case vi_exit:
            if (Not(bookshelf_moved)){
                msg_locked_in_secret_room();
                if (Is(secret_room_note_read)){
                    die_starved();
                }
            }
            else {
                Reset(save_forbidden);
                move_area(ai_library);
            }
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_floor:
        switch (vid){
        case vi_look:
			message("中央には怪しい%nlが描かれている。",
					ni_magic_circle);
			if (Is(pocket_notebook_dropped)){
                msg_looking_pocket_notebook();
			}
			return 1;
		default:
			break;
		}
		break;
    case ni_table:
        switch (vid){
        case vi_look:
			message("%nlと%nlが置かれている。",
					ni_book, ni_letter);
			return 1;
		default:
			break;
		}
		break;
    case ni_book:
        switch (vid){
        case vi_look:
            message("古ぼけた本だ。%nlと題がついている。",
                    ni_another_world);
			return 1;
        case vi_examine:
			message("ところどころ虫に喰われている。");
            return 1;
        case vi_get:
            msg_no_need_to_carry();
			return 1;
        case vi_read:
            msg_read_secret_room_book();
			Set(secret_room_book_read);
			return 1;
		default:
			break;
		}
		break;
    case ni_another_world:
        RedoArea(vid, ni_book);
    case ni_letter:
        switch (vid){
        case vi_look:
			message("古い手紙だ。");
			return 1;
        case vi_examine:
            message("マクドナルド氏宛の手紙だ。"
                    "差出人はラヴクラフトとなっている。");
            return 1;
        case vi_get:
            //			command_get_item(nid, fi_postcard_taken);
            msg_no_need_to_carry();
			return 1;
        case vi_read:
            msg_read_letter();
			Set(secret_room_letter_read);
            return 1;
		default:
			break;
		}
		break;
	case ni_magic_circle:
        switch (vid){
        case vi_look:
			message("%nnのあちこちには奇妙な記号が描かれている。"
					"中央には%nlがある。",
					ni_magic_circle, ni_altar);
        case vi_examine:
            if (Is(magic_circle_invoked)){
                msg_feeling_magic_circle_power();
            }
            else {
                message("床の上に直接描かれている。");
            }
			return 1;
		default:
			break;
		}
		break;
	case ni_altar:
        switch (vid){
        case vi_look:
			if (Is(candle_put)){
				message("%nnの上には燭台があり、蝋燭が灯っている。",
						ni_altar);
			}
			else {
				message("%nnの上には燭台がある。",
						ni_altar);
			}
			return 1;
		default:
			break;
		}
		break;
	case ni_skeleton:
        switch (vid){
        case vi_look:
			message("白骨死体だ。"
					"男の服を着ている。"
					);
			if (Is(pocket_notebook_dropped)){
                msg_looking_pocket_notebook();
			}
			return 1;
        case vi_examine:
			message("かなり古いもののようだ。袖や裾から%nlが覗いている。"
					"衣服は虫に食われているが、かなり上等な品だ。",
                    ni_human_bone);
			if (Not(pocket_notebook_dropped)){
				message("死体を調べていると、懐から何かが落ちた。");
				Set(pocket_notebook_dropped);
			}
			else {
                msg_looking_pocket_notebook();
			}
			return 1;
        case vi_hit:
        case vi_kick:
        case vi_destroy:
			message("死者の眠りを妨げるのはやめておこう。");
			return 1;
		default:
			break;
		}
		break;
	case ni_human_bone:
		if (Not(pocket_notebook_dropped)) ReturnNotExist;
        ReturnIfHasItem;
        // @note アイテムとしての骨をとっても、まだ骨が残っているので、
        // 他のアイテムのように以下のコードを使うことはできない
        //		if (Is(human_bone_taken)){
        //			*exist = 0;
        //			return 0;
        //		}
		switch (vid){
		case vi_look:
			message("相当な年月がたったのか、完全に白骨化している。");
			return 1;
		case vi_examine:
            message("あまりいじり回すのは冒涜的な気がする。");
			return 1;
		case vi_get:
            if (Is(human_bone_taken)){
                message("あとは安らかに眠らせておこう。");
            }
            else {
                command_get_item(nid, fi_human_bone_taken);
            }
			return 1;
		default:
			break;
		}
		break;
	case ni_pocket_notebook:
		if (Not(pocket_notebook_dropped)) ReturnNotExist;
		switch (vid){
		case vi_look:
			message("古びているが、革表紙の立派な手帳だ。");
			return 1;
		case vi_examine:
			message("ぱらぱらとめくると、"
                    "なにかぎっしりと書いてあるページが見つかった。");
			return 1;
		case vi_read:
            msg_read_mcdonald_notebook();
			Set(secret_room_note_read);
			return 1;
		case vi_get:
            msg_no_need_to_carry();
			return 1;
		default:
			break;
		}
		break;
	case ni_bottle:
        if (!has_item(nid)) ReturnNotExist;
        switch (vid){
        case vi_use:
            if (Not(magic_circle_invoked)){
                if (Is(candle_put)){
                    message("小瓶の中の粉をひとつまみ蝋燭の火にくべると、"
                            "あたりに奇妙な香りが広がった。");
                    msg_feeling_magic_circle_power();
                    Set(magic_circle_invoked);
                }
                else {
                    message("祭壇に火が必要だ。");
                }
            }
            else {
                message("これ以上、香を焚く必要もないだろう。");
            }
            return 1;
        default:
            break;
        }
        break;
	case ni_candle:
		if (Is(candle_put)){
		    switch (vid){
			case vi_look:
				message("蝋燭は燭台に置かれている。");
				return 1;
			case vi_put:
				message("蝋燭はもう置かれている。");
				return 1;
			case vi_get:
				command_get_item(nid, fi_invalid);
				Reset(candle_put);
                if (Is(magic_circle_invoked)){
                    message("魔法陣から力を感じなくなった。");
                    Reset(magic_circle_invoked);
                }
				return 1;
			default:
				break;
			}
		}
		else if (has_item(nid)){
		    switch (vid){
			case vi_put:
				message("蝋燭を燭台に置いた。");
				lose_item(ni_candle);
				Set(candle_put);
				return 1;
			default:
				break;
			}
		}
		else {
			ReturnNotExist;
		}
		break;
		
		/*
case ni_amulet:
        if (!has_item(nid)) ReturnNotExist;
        switch (vid){
        case vi_put:
            command_put_item(nid, fi_amulet_put, "魔法陣の中央に石を置いた");
            game_clear();
            return 1;
        default:
            break;
        }
        break;
*/
    case ni_wall:
    case ni_ceiling:
        break;
    default:
		*exist = 0;
        break;
    }
    return 0;
}

