#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

int storage_window_blocked(void);

int area_storage(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
            message("%nlや%nl、%nlといった家具が詰め込まれている。",
                    ni_chest, ni_table, ni_chair);
            message("背後には廊下に続く扉がある。");
            return 1;
        case vi_backward:
		case vi_exit:
            EnterDoor(corridor_1f_mr, storage);
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        case vi_examine:
            RedoArea(vid, ni_floor);
        default:
            break;
        }
        break;
    case ni_door:
    case ni_back_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_backward, ni_invalid);
        }
        break;
    case ni_chest:
        switch (vid){
        case vi_look:
            message("%nlが並んだ箪笥だ。",
                    ni_drawer);
            return 1;
        case vi_examine:
            RedoArea(vi_examine, ni_drawer);
        case vi_open:
        case vi_close:
            RedoArea(vid, ni_drawer);
        case vi_push:
        case vi_pull:
        case vi_move:
            msg_no_space_to_move();
            return 1;
        default:
            break;
        }
        break;
    case ni_drawer:
        switch (vid){
        case vi_look:
        case vi_examine:
            message("外側に変わったところはない。");
            if (Not(storage_drawers_closed)){
                message("中は空だ。");
            }
            return 1;
        case vi_close:
            TestAndSet(storage_drawers_closed,
                       "引き出しを閉じた。",
                       "もう閉まっている。");
            return 1;
        case vi_open:
            TestAndReset(storage_drawers_closed,
                       "引き出しを開けた。",
                       "もう開いている。");
            return 1;
        case vi_move:
            if (Is(storage_drawers_closed)){
                RedoArea(vi_open, nid);
            }
            else {
                RedoArea(vi_close, nid);
            }
        default:
            break;
        }
        break;
    case ni_table:
        switch (vid){
        case vi_look:
            return 1;
        case vi_push:
            if (Not(storage_drawers_closed)){
                msg_no_space_to_move();
                return 1;
            }
            TestAndSet(storage_table_moved,
                       "テーブルを奥に移動した。",
                       "これ以上奥には動かせない。");
            return 1;
        case vi_pull:
            if (Not(storage_drawers_closed)){
                msg_no_space_to_move();
                return 1;
            }
            TestAndReset(storage_table_moved,
                       "テーブルを元の位置に戻した。",
                       "これ以上、手前には動かせない。");
            return 1;
        case vi_move:
            if (Not(storage_drawers_closed)){
                msg_no_space_to_move();
                return 1;
            }
            if (Is(storage_table_moved)){
                RedoArea(vi_pull, nid);
            }
            else {
                RedoArea(vi_push, nid);
            }
        default:
            break;
        }
        break;
    case ni_chair:
        switch (vid){
        case vi_look:
            return 1;
        case vi_push:
        case vi_pull:
        case vi_move:
            msg_no_space_to_move();
            return 1;
        default:
            break;
        }
        break;
	case ni_window:
        switch (vid){
        case vi_look:
            message("ガラスの一部が割れて穴が開いている。");
            if (!storage_window_blocked()){
                message("窓の下には%nlがある。",
                        ni_washtub);
            }
            return 1;
        case vi_examine:
            if (storage_window_blocked()){
                msg_blocked_by_furniture();
            }
            else {
                message("掌大のものなら、穴を通りそうだ。");
            }
            return 1;
        default:
            if (storage_window_blocked()){
                msg_blocked_by_furniture();
                return 1;
            }
            else {
                break;
            }
        }
        break;
    case ni_washtub:
        if (storage_window_blocked()) ReturnNotExist;
        switch (vid){
        case vi_look:
        case vi_examine:
            message("%nlが溜まっている。",
                    ni_water);
            return 1;
        case vi_get:
        case vi_touch:
        case vi_push:
        case vi_pull:
        case vi_move:
        case vi_hit:
        case vi_kick:
        case vi_destroy:
            message("穴から手を出したぐらいでは届きそうにない。");
            return 1;
        default:
            break;
        }
    case ni_water:
        if (storage_window_blocked()) ReturnNotExist;
        switch (vid){
        case vi_look:
        case vi_examine:
            message("雨水が貯まったのだろうか。"
                    "底に落ち葉が沈んでいるが、澄んだ水だ。");
            return 1;
        case vi_get:
        case vi_touch:
        case vi_push:
        case vi_pull:
        case vi_move:
        case vi_hit:
        case vi_kick:
        case vi_destroy:
            message("穴から手を出したぐらいでは届きそうにない。");
            return 1;
        default:
            break;
        }
    case ni_fishing_rod:
        if (!has_item(nid)) ReturnNotExist;
        switch (vid){
        case vi_use:
            if (storage_window_blocked()){
                msg_blocked_by_furniture();
                return 1;
            }
            if (Not(fishing_rod_with_sponge)){
                message("あなたは窓の穴から%nnを突き出し、%nnに釣り糸を垂れた。",
                        ni_fishing_rod, ni_washtub);
                message("･･･何も釣れない。魚はいないようだ。");
            }
            else {
                if (Not(sponge_wet)){
                    message("あなたは窓の穴から%nnを突き出し、"
                            "釣り糸の先の%nnを%nnに降ろした。",
                            ni_fishing_rod, ni_sponge, ni_washtub);
                    message("しばらくして引き上げると、"
                            "%nnはたっぷりと水を吸っていた。",
                            ni_sponge);
                    Set(sponge_wet);
                }
                else {
                    message("%nnはすでに水浸しだ。",
                            ni_sponge);
                }
            }
            return 1;
        default:
            break;
        }
    case ni_lantern:
		if (has_item(nid)){
			*exist = 0;
			return 0;
		}
		switch (vid){
		case vi_get:
			command_get_item(nid, fi_lantern_taken);
			return 1;
		default:
			break;
		}
		break;
    case ni_floor:
        switch (vid){
        case vi_look:
        case vi_examine:
            message("部屋の隅に%nlが置いてある。",
                    ni_lantern);
			return 1;
		default:
			break;
		}
		break;
    case ni_ceiling:
    case ni_wall:
    default:
        *exist = 0;
        break;
    }
    return 0;
}


int storage_window_blocked(void){
    if (Not(storage_drawers_closed) || Not(storage_table_moved)){
        return 1;
    }
    return 0;
}
