#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

int upset_armor_in_study(void);
void calm_armor_in_study(void);

int area_study(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
            if (Is(armor_in_study)){
                msg_armor_standing();
            }
			message("右側手前には%nlと%nlが\\",
                    ni_desk, ni_chair);
            if (Not(armor_in_study) && Not(armor_dead)){
                message("あり、その奥には%nlが飾られている。",
                        ni_armor);
            }
            else {
                message("ある。");
            }
            message("正面の壁には%nlがあり、その左側には扉%nwがある。",
                    ni_bookshelf, ni_left_door);
            message("背後には廊下に続く扉%nwがある。",
                    ni_back_door);
            return 1;
        case vi_backward:
		case vi_exit:
            calm_armor_in_study();
            EnterDoor(corridor_2f_cr, study_corridor);
            //            move_dragon_in_corridor_2f();
            return 1;
        case vi_left:
			if (Not(study_library_door_closed)){
                calm_armor_in_study();
	            // 図書室との扉は最初から開いている
		        move_area(ai_library);
			}
			else {
				msg_door_closed();
			}
            return 1;
        default:
            break;
        }
        break;
    case ni_room:
        switch (vid){
        case vi_look:
        case vi_exit:
            RedoArea(vid, ni_invalid);
        default:
            break;
        }
        break;
    case ni_door:
        msg_which_door();
        return 1;
    case ni_left_door:
        switch (vid){
        case vi_look:
            message("扉の向こうは図書室のようだ。");
            return 1;
        case vi_open:
        case vi_push:
            TestAndReset(study_library_door_closed,
                         "左奥の扉を開けた。",
                         "扉はもう開いている。");
            return 1;
        case vi_close:
        case vi_pull:
            TestAndSet(study_library_door_closed,
                       "左奥の扉を閉めた。",
                       "扉はもう閉まっている。");
            return 1;
		case vi_enter:
            RedoArea(vi_left, ni_invalid);
        default:
            break;
        }
        break;
    case ni_back_door:
		if (vid == vi_open || vid == vi_push || vid == vi_pull){
            RedoArea(vi_backward, ni_invalid);
		}
        break;
    case ni_armor:
        if (Is(armor_dead)) ReturnNotExist;
        switch (vid){
        case vi_look:
            if (Not(armor_in_study)){
                message("座った形で飾ってある。"
                        "腰には%nlを差している。",
                        ni_sword);
            }
            else {
                msg_look_standing_armor();
            }
            return 1;
        case vi_examine:
        case vi_touch:
        case vi_get:
            upset_armor_in_study();
            return 1;
        default:
            break;
        }
        break;
    case ni_sword:
        switch (vid){
        case vi_look:
            if (Not(armor_in_study)){
                message("鞘に納まったまま、鎧の腰に差してある。");
            }
            else {
                msg_look_standing_armor_sword();
            }
            return 1;
        case vi_examine:
        case vi_touch:
        case vi_get:
            upset_armor_in_study();
            return 1;
        default:
            break;
        }
        break;
    case ni_desk:
        if (vid != vi_look){
            if (upset_armor_in_study()) return 1;
        }
        switch (vid){
        case vi_look:
        case vi_examine:
			message("%nlの付いた机だ。",
					ni_drawer);
			message("一枚の%nlが置かれている。",
					ni_postcard);
			return 1;
        default:
            break;
        }
        break;
    case ni_drawer:
        if (vid != vi_look){
            if (upset_armor_in_study()) return 1;
        }
        switch (vid){
        case vi_look:
        case vi_examine:
            if (Not(study_desk_opened)){
                message("右端に鍵穴がある。");
            }
            else {
                if (Not(screwdriver_taken)){
                    message("抽斗の中に%nlと%nlがある。",
                            ni_notebook, ni_screwdriver);
                }
                else {
                    message("抽斗の中に%nlがある。",
                            ni_notebook);
                }
            }
			return 1;
        case vi_open:
        case vi_pull:
            if (Is(study_desk_unlocked)){
                TestAndSet(study_desk_opened,
                           "机の抽斗を開けた。",
                           "抽斗はもう空いている。");
            }
            else {
                message("鍵が掛かっている。");
            }
            return 1;
        case vi_close:
        case vi_push:
            TestAndReset(study_desk_opened,
                         "机の抽斗を閉めた。",
                         "抽斗は閉まっている。");
            return 1;
        default:
            break;
        }
        break;
    case ni_steel_key:
        if (!has_item(nid)) ReturnNotExist;
        switch (vid){
        case vi_use:
            if (upset_armor_in_study()) return 1;
            if (Not(study_desk_unlocked)){
                message("鍵が開いた。");
                Set(study_desk_unlocked);
                lose_item(nid);
            }
            else {
                message("もう鍵は掛かっていない。");
            }
            return 1;
        default:
            break;
        }
        break;
	case ni_postcard:
        //        ReturnIfHasItem;
        switch (vid){
        case vi_look:
            message(noun_description(nid));
            return 1;
        case vi_examine:
            message("サンダース大佐からヨシノ教授に送られたものだ。");
            return 1;
        case vi_get:
            //			command_get_item(nid, fi_postcard_taken);
            msg_no_need_to_carry();
			return 1;
        case vi_read:
            msg_read_postcard();
            Set(postcard_read);
            return 1;
        default:
            break;
        }
        break;
	case ni_notebook:
        if (vid != vi_look){
            if (upset_armor_in_study()) return 1;
        }
        if (upset_armor_in_study()) return 1;
        //        ReturnIfHasItem;
        switch (vid){
        case vi_look:
            message(noun_description(nid));
            return 1;
        case vi_examine:
            message("何かの研究ノートのようだ。");
            return 1;
        case vi_get:
            msg_no_need_to_carry();
			return 1;
        case vi_read:
            msg_read_yoshino_notebook();
            Set(study_notebook_read);
            return 1;
        default:
            break;
        }
        break;
	case ni_screwdriver:
        ReturnIfHasItem;
        if (vid != vi_look){
            if (upset_armor_in_study()) return 1;
        }
		switch (vid){
		case vi_get:
			if (Is(study_desk_opened)){
				command_get_item(nid, fi_screwdriver_taken);
				return 1;
			}
			else {
				ReturnNotExist;
			}
		default:
			break;
		}
		break;
    case ni_chair:
        break;
    case ni_bookshelf:
		switch (vid){
		case vi_look:
            message("物理学の専門書が並んでいる。");
            return 1;
		default:
			break;
		}
		break;
    case ni_wall:
    case ni_floor:
    case ni_ceiling:
	case ni_window:
    case ni_book:
        break;
    case ni_brass_key:
    case ni_wooden_key:
    case ni_bronze_key:
    case ni_golden_key:
        if (has_item(nid)){
            switch(vid){
            case vi_use:
                msg_key_unmatched();
                return 1;
            default:
                break;
            }
            return 0;
        } 
		else {
			ReturnNotExist;
		}
        break;
    default:
		*exist = 0;
        break;
    }
    return 0;
}

int upset_armor_in_study(void){
    if (Not(armor_dead)){
        // メッセージはコマンドへのメッセージの後で出したいので、フラグセットのみ
        TestAndSet(start_armor_to_study,
                   NULL, NULL);
        if (Is(armor_moving)){
            // このターン末で図書室/客室から戻ってくるなら今回のコマンドは無効
            if (Is(armor_restoring_library) || Is(armor_moving_from_guestroom)){
                return 1;
            }
            return 0;
        }
        else {
            return 1;
        }
    }
    else {
        return 0;
    }
}

void calm_armor_in_study(void){
    //    if (Is(start_armor_to_study) || Is(armor_in_study)){
    if (Is(armor_in_study)){
        message("部屋を出ようとすると、"
                "鎧は満足したかのように元の場所に戻っていった。");
        Reset(armor_moving);
        Reset(armor_in_study);
    }
}

void post_study(int time_passed){
    if (!time_passed) return;
    
    // 鎧イベント
    if (Is(armor_in_study) && Staying(study)){
        die_killed_by_armor();
    }
    else if (Is(start_armor_to_study)){
        if (Not(armor_moving)){
            message("座っていた鎧が動き出し、あなたに向かってきた!");
            Set(armor_moving);
            Set(armor_in_study);
            Reset(start_armor_to_study);
        }
    }
}

