#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "config.h"
#include "command.h"
#include "term.h"
#include "status.h"
#include "area.h"
#include "noun.h"
#include "verb.h"
#include "message.h"
#include "graphic.h"

static int (*command_callback)(char *line);
// vi_combine処理時に使うフラグ
int combine_flags[ni_end];
// vi_combineのときの名詞ID(合体・結合のベースアイテム)
noun_id_t combine_base_nid;
// vi_combine時の追加入力回数
int combine_query_count;
// vi_combine時の指定アイテム数
int combine_item_count;
/*
void (*exec_command_post_message)(void) = NULL;
*/

typedef struct {
	flag_id_t flag;
	noun_id_t nid;
	 void (*msg)(void);
} recall_entry_t;

static recall_entry_t recalls[] = {
	{fi_postcard_read, ni_postcard, msg_read_postcard},
	{fi_study_notebook_read, ni_study_note, msg_recall_yoshino_notebook},
	{fi_easy_necromancy_read, ni_easy_necromancy, msg_recall_easy_necromancy},
	{fi_bathroom_1f_message_read, ni_bathroom_1f_wall, msg_read_bathroom_1f_wall},
	{fi_bathroom_2f_message_read, ni_bathroom_2f_wall, msg_read_bathroom_2f_wall},
	{fi_secret_room_note_read, ni_secret_room_note, msg_recall_mcdonald_notebook},
	{fi_secret_room_letter_read, ni_secret_room_letter, msg_read_letter},
	{fi_secret_room_book_read, ni_another_world, msg_recall_secret_room_book},
	{fi_boiler_room_note_read, ni_boiler_room_note, msg_recall_boiler_room_wall},
    {fi_logbook_read, ni_logbook, msg_read_logbook},
};
#define RECALL_ENTRY_NUM (sizeof(recalls)/sizeof(recall_entry_t))

void init_command(void){
    noun_init();
    verb_init();
    clear_command_callback();
}

void interpret_line(char *line){
    char *verb, *noun;
    verb_id_t vid;
    noun_id_t nid;
    int time_passed;

    if (command_callback){
        // command_callbackが0を返したら、
        // コールバック登録を消去して通常のコマンド入力に戻る
        // 複数回コールバックしたいときは、コールバック関数内で
        // 次のコールバックをセットし非0を返せばよい
        // ※リピートしないコールバック関数内で%wを使うと、
        //   [リターンキーを...]は出ない。
        //   以下でconsole_promptが初期化されるため。
        if (command_callback(line) == 0){
            clear_command_callback();
            //            set_console_prompt(NULL);
        }
    }
    // 入力なしでリターンキーだけ押したときはコマンド処理しない
	else if (*line != '\0'){
	    parse_command(line, &verb, &noun);
		nid = get_noun_id(noun);
	    vid = get_verb_id(verb, nid);
		//    printf("verb:%s:%d: noun:%s:%d:\n", verb, vid, noun, nid);
	    exec_command(vid, nid, verb, noun);
//    if (command_callback == NULL){
        time_passed = (vid == vi_look ||
					   vid == vi_inventory ||
					   vid == vi_load ||
					   vid == vi_save ||
                       vid == vi_unknown ||
                       nid == ni_unknown)
					   ? 0 : 1;
        post_area(current_area(), time_passed);
//    }
        // この回移動があるなら、フラグ系列進行は移動後に適用
        // 死んだときはフラグ系列進行やコールバックはしない
        if (Not(dead)){
            process_flag_sequences(time_passed);
        }
		update_current_area();
	}
    draw_graphic();
/*
	if (exec_command_post_message != NULL){
   		printf("set %p\n", exec_command_post_message);
		exec_command_post_message();
		exec_command_post_message = NULL;
	}
	*/
}

// lineからverb, nounを取り出す
// lineの指す先は書き換えられる。
// verb, nounはline中の該当箇所を指すように変更される
// 動詞のみの場合、nounはNULLとなる
// !!! 大文字は小文字に変換する処理を入れる？
void parse_command(char *line, char **verb, char **noun){
    char *p = line;

    printf("|%s|\n", p);
    // nounのデフォルト値はNULL
    *noun = NULL;
    p = parse_verb(p, verb);
    if (p){
        parse_noun(p, noun);
    }
}
char *parse_verb(char *line, char **verb){
    char *p = line;

    // 動詞前にスペースがあったら読み飛ばす
    while (*p == ' '){
        p++;
    }
    *verb = p;
    while (*p != '\0' && *p != ' '){
        p++;
    }
    // 名詞句がないなら終了
    if (*p == '\0'){
        return NULL;
    }
    // 動詞末をターミネート
    *p = '\0';
    return p+1;
}
void parse_noun(char *line, char **noun){
    char *p = line;
    char *q;

    // 先頭(動詞後)のスペースをすべて読み飛ばす
    while (*p == ' '){
        p++;
    }
    // スペース後にやっぱり名詞句がないなら終了
    if (*p == '\0'){
        return;
    }
    // 残り行末まで名詞扱い
    *noun = p;
    // 名詞は空白を含むことができるので、複数語からなる名詞句でもよい
    // 複数の空白が連続する場合は空白を一つに変換しておく
    while (*p != '\0'){
        if (*p == ' '){
            p++;
            q = p;
            while (*p == ' '){
                p++;
            }
            strcpy(q, p);
            p = q;
        }
        else {
            p++;
        }
    }
    p--;
    // 行末に空白があれば消す
    while (*p == ' '){
        *p = '\0';
        p--;
    }
}




// コマンドを実行する
// 基本的にIDの方で処理するが、メッセージに入力文字列が必要な場合に備え、
// verb, nounも受け取る
void exec_command(verb_id_t vid, noun_id_t nid, char *verb, char *noun){
    int exist = 0;
    assert(vid != vi_invalid);

    if (vid == vi_unknown){
        message("%sという動詞は使えない。", verb);
        return;
    }
    if (nid == ni_unknown){
        message("%sという名詞は使えない。", noun);
        return;
    }
    if (call_area(current_area(), vid, nid, &exist)) return;
    if (nid == ni_invalid){
        switch (vid){
        case vi_forward:
        case vi_backward:
        case vi_left:
        case vi_right:
            message("そちらには進めない。");
            return;
        case vi_up:
            message("上れそうなところはない。");
            return;
        case vi_down:
            message("降りられそうなところはない。");
            return;
        case vi_recall:
            command_recall(nid);
            return;
        case vi_help:
            show_help(nid, noun);
            return;
        case vi_inventory:
            show_inventory();
            return;
        case vi_load:
            if (load_data() == 0){
                message("ゲームをロードしました。");
                move_area(current_area());
            }
            else {
                message("ゲームのロードに失敗しました。");
            }
            return;
        case vi_save:
			if (Is(save_forbidden)){
                message("そんな状況ではなさそうだ。");
                //				message("そんな余裕はない！");
				return;
			}
            if (save_data() == 0){
                message("ゲームをセーブしました。");
            }
            else {
                message("ゲームのセーブに失敗しました。");
            }
            return;
        case vi_title:
            move_to_title();
            return;
        case vi_score:
            show_normal_score();
            //            show_maniac_score();
            return;
        case vi_config:
            config(nid, noun);
            return;
        case vi_quit:
            command_quit();
            return;
#ifdef DEBUG
        case vi_debug:
            show_debug();
            return;
        case vi_test:
            move_area(ai_working);
            return;
#endif
		case vi_wait:
			message("あなたは少し時間を潰した。");
			return;
		case vi_examine:
        case vi_get:
        case vi_put:
        case vi_push:
        case vi_pull:
        case vi_destroy:
            message("何を？");
            return;
        case vi_pray:
            message("神は答えてくれない。");
            return;
        default:
            break;
        }
    }
    else {  // 名詞が指定されている場合
		// その場に存在していない名詞に対して可能なコマンド処理
		switch (vid){
        case vi_config:
            config(nid, noun);
            return;
        case vi_help:
            show_help(nid, noun);
            return;
        case vi_recall:
            command_recall(nid);
            return;
#ifdef DEBUG
		case vi_teleport:
            command_teleport(nid);
            return;
        case vi_summon:
            command_summon(nid);
            return;
        case vi_flag:
            command_flag(nid);
            return;
#endif
		default:
			break;
		}
        // 所持アイテムなら存在フラグを立てる
        if (IsNounItem(nid) && has_item(nid)){
            exist = 1;
        }
		if (IsNounItem(nid)){
            if (exec_command_item(vid, nid, exist)) return;
        }
        if (!exist){
            msg_no_object();
            return;
        }
        switch (vid){
        case vi_look:
            message(noun_description(nid));
//            msg_nothing_special();
            return;
        case vi_examine:
            switch(nid){
            case ni_ceiling:
                message("この位置から調べるのは無理だ。");
                return;
            default:
                msg_nothing_special();
                return;
            }
            return;
        case vi_get:
            message("それは取れない。");
            return;
        case vi_hit:
        case vi_kick:
        case vi_touch:
            switch (nid){
            case ni_ceiling:
                message("届かない。");
                return;
            default:
                msg_no_effect();
                return;
            }
        case vi_up:
			switch (nid){
			case ni_table:
			case ni_chair:
			case ni_bed:
			case ni_desk:
			case ni_cabinet:
			case ni_chest:
			case ni_wardrobe:
			case ni_dressing_table:
			case ni_bookshelf:
                message("行儀の悪いことはやめておこう。");
                return;
            default:
                break;
            }
            break;
        case vi_use:
			switch (nid){
			case ni_bed:
				message("ぐぅ。…いかん、寝ている場合ではない。");
				return;
			default:
				break;
			}
			break;
        case vi_read:
            msg_cannot_read();
            return;
        case vi_destroy:
            msg_cannot_destroy();
            return;
		case vi_open:
			switch (nid){
			case ni_window:
				message("枠が錆びついているのか、固くて開かない。");
				return;
			default:
				break;
			}
			break;
		case vi_close:
            switch(nid){
            case ni_door:
            case ni_front_door:
            case ni_back_door:
            case ni_left_door:
            case ni_right_door:
				msg_door_already_closed();
				return;
			default:
				break;
			}
        default:
            break;
        }
    }
    msg_cannot_do();
}
/*
void set_post_message(void (*msg)(void)){
	printf("set %p\n", msg);
	exec_command_post_message = msg;
}*/

int exec_command_item(verb_id_t vid, noun_id_t nid, int exist){
    switch (vid){
    case vi_look:
        // 所持アイテムかその場に存在していればlookコマンドを実行
        // 手に取る前と後で違うメッセージを出したい場合はarea_xxxで処理しておく
        if (exist){
            command_look_item(nid);
            return 1;
        }
        break;
    case vi_examine:
        if (has_item(nid)){
            command_examine_item(nid);
            return 1;
        }
        else if (exist){
            msg_need_to_get();
            return 1;
        }
        break;
    case vi_get:
        // 有効なgetはarea_xxxで処理されるので、
        // ここに来るのは入手済みの場合か、対象がその場にない場合
        if (has_item(nid)){
            message("%nnはもう持っている。", nid);
            return 1;
        }
        break;
    case vi_put:
        // 有効なputはarea_xxxで処理されるので、
        // ここに来るのは入手済みの場合か、対象がその場にない場合
        if (has_item(nid)){
            msg_cannot_do();
            return 1;
        }
        break;
    case vi_use:
        // 有効なuseはarea_xxxで処理されるので、
        // ここに来るのは未入手の場合か、対象がその場にない場合か、
        // 使えない場合
        if (has_item(nid)){
            command_use_item(nid);
            return 1;
        }
        break;
    case vi_read:
        if (has_item(nid)){
            command_read_item(nid);
            return 1;
        }
        else if (exist){
            msg_need_to_get();
            return 1;
        }
        break;
    case vi_touch:
        if (has_item(nid)){
            // とりあえずさわってどうにかなるアイテムはない
            msg_no_effect();
            return 1;
        }
        break;
    case vi_combine:
        if (has_item(nid)){
            command_combine_item(nid);
            return 1;
        }
        break;
    case vi_dismantle:
        if (has_item(nid)){
            command_dismantle_item(nid);
            return 1;
        }
        break;
    case vi_remove:
        command_remove_item(nid);
        return 1;
    default:
        break;
    }
    return 0;
}

void show_help(noun_id_t nid, char *noun){
    switch(nid){
    case ni_invalid:
        message("  %s %s: 基本コマンド", vid2verb(vi_help), nid2noun(ni_basic));
        message("  %s %s: 動詞一覧", vid2verb(vi_help), nid2noun(ni_verb));
        message("  %s %s: 名詞一覧", vid2verb(vi_help), nid2noun(ni_noun));
        message("  %s %s: ヒント", vid2verb(vi_help), nid2noun(ni_tips));
        message("  %s %s: キー操作", vid2verb(vi_help), nid2noun(ni_key));
        message("  %s %s: 補助", vid2verb(vi_help), nid2noun(ni_assistance));
		// ※補助機能とするとvcppで「能」が化ける
        break;
    case ni_basic:
        show_help_basic();
        break;
    case ni_verb:
        show_help_verb();
        break;
    case ni_noun:
        show_help_noun();
        break;
    case ni_tips:
        show_help_tips();
        break;
    case ni_key:
        show_help_key();
        break;
    case ni_assistance:
        show_help_assistance();
        break;
    default:
        if (noun != NULL){
            message("%sは助けられない。", noun);
        }
    }
}

void show_inventory(void){
    // !!! 持ち物が表示行数を超えることがあるので、
    // とりあえずバッファがあふれないように大きめにしておく
    // console側で、表示行数を超えるようならリターン待ちをするようにする
    char buf[CONSOLE_CHR_WIDTH * CONSOLE_CHR_HEIGHT * 4 ];
    int i, item_num;
	noun_id_t nid;
    char *p;

    puts_console("[持ち物]");
    buf[0] = '\0';
    p = buf;
	item_num = get_item_num();
    for (i = 0 ; i < item_num ; i++){
		nid = get_item_id(i);
        printf("%d\n", i);
        assert(nid2noun(nid) != NULL);
        sprintf(p, "%s[%s] ", noun_name(nid), nid2noun(nid));
        while (*p != '\0') p++;
    }
    if (p == buf){
        sprintf(buf, "(なし)");
    }
    puts(buf);
    puts_console(buf);
}

void show_debug(void){
    //    char buf[CONSOLE_CHR_WIDTH * CONSOLE_CHR_HEIGHT];
    char fbuf[fi_end+1];
    char ibuf[ni_item_end+1];
    int i;

    for (i = fi_begin ; i < fi_end ; i++){
        if (is_flag_on(i)){
            fbuf[i] = '1';
        }
        else {
            fbuf[i] = '0';
        }
    }
    fbuf[i] = '\0';
    puts(fbuf);
    puts_console(fbuf);

    for (i = ni_item_begin ; i < ni_item_end ; i++){
        if (has_item(i)){
            ibuf[i] = '1';
        }
        else {
            ibuf[i] = '0';
        }
    }
    ibuf[i] = '\0';
    puts(ibuf);
    puts_console(ibuf);

}

void config(noun_id_t nid, char *noun){
    switch(nid){
    case ni_invalid:
        message("以下の設定が可能です。");
        message("  %s %s: 入力言語設定", vid2verb(vi_config), nid2noun(ni_input));
        break;
    case ni_input:
        message("どの入力方式に切り替えますか?(%nw%nw%nw)", 
                ni_english, ni_hiragana, ni_katakana);
        set_command_callback(command_callback_input);
        set_term_prompt(QUERY_PROMPT);
        break;
    default:
        message("%sは設定できません。", noun);
    }
}
void command_quit(void){
    message("プログラムを終了しますか(%nw/%nw)?", ni_yes, ni_no);
    set_command_callback(command_callback_quit);
    set_term_prompt(QUERY_PROMPT);
}

int command_callback_quit(char *line){
    char *noun;
    noun_id_t nid;
    int result = 1;

    if (*line != '\0'){
        parse_noun(line, &noun);
        nid = get_noun_id(noun);
        switch (nid){
        case ni_yes:
            result = 1;
            break;
        default:
            result = 0;
            break;
        }
    }
    //    command_callback = NULL;
    //    set_console_prompt(NULL);
    if (result){
        puts("bye.");
        EzExitEventLoop();
    }
    return 0;
}
void command_recall(noun_id_t nid){
	int i;
	for (i = 0 ; i < RECALL_ENTRY_NUM ; i++){
		if (nid == recalls[i].nid && is_flag_on(recalls[i].flag)){
			recalls[i].msg();
			return;
		}
	}
	list_recall();
}
void list_recall(void){
    int ctr = 0;
	int i;
	for (i = 0 ; i < RECALL_ENTRY_NUM ; i++){
		if (is_flag_on(recalls[i].flag)){
	        message("%nl \\", recalls[i].nid);
			ctr ++;
		}
	}
    if (ctr > 0){
        message("");
        message("以上のことを思い出せる。");
    }
    else {
        message("まだ思い出せるようなことはない。");
    }
}

	
/*	
	if (Is(postcard_read)){
        message("%nl \\", ni_postcard);
        ctr++;
    }
    if (Is(easy_necromancy_read)){
        message("%nl \\", ni_easy_necromancy);
        ctr++;
    }
	if (Is(bathroom_2f_message_read)){
        message("%nl \\", ni_bathroom_2f_wall);
        ctr++;
    }
    if (Is(study_notebook_read)){
        message("%nl \\", ni_study_note);
        ctr++;
    }
    if (Is(secret_room_note_read)){
        message("%nl \\", ni_secret_room_note);
        ctr++;
    }
    if (Is(secret_room_book_read)){
        message("%nl \\", ni_secret_room_book);
        ctr++;
    }
    if (Is(secret_room_book_read)){
        message("%nl \\", ni_secret_room_book);
        ctr++;
    }
	
*/


void command_look_item(noun_id_t nid){
    switch(nid){
    case ni_newspaper:
		msg_look_newspaper();
        Set(newspaper_looked);
        return;
    case ni_candle:
		if (Is(candle_lit)){
			message("火のついた%nnだ。", nid);
		}
		else {
			message("%nnだ。", nid);
		}
		return;
	case ni_lantern:
        if (Is(lantern_lit)){
            message("灯りのついた%nnだ。", nid);
        }
        else {
            message("持ち歩き用の%nnだ。", nid);
        }
        return;
    case ni_slingshot:
		if (Is(slingshot_repaired)){
			message("%nnだ。", nid);
		}
		else {
			message("壊れた%nnだ。", nid);
		}
		return;
    case ni_belt:
        message("大型犬用の首輪だ。小さな革袋がぶら下がっている。");
        return;
    case ni_fishing_rod:
        message("普通の%nnだ。\\", nid);
		if (Is(fishing_rod_with_sponge)){
			message("釣り針に%nnを引っかけてある。",
                    ni_sponge);
		}
        else {
            message("");
        }
		return;
    case ni_sponge:
        if (Is(fishing_rod_with_sponge)){
            message("釣り針に引っかけた、\\");
        }
        message("入浴用のスポンジだ。\\");
		if (Is(sponge_wet)){
			message("水をたっぷりと含んでいる。");
		}
		else {
            message("");
		}
		return;
	default:
		break;
	}
    message(noun_description(nid));
}
void command_examine_item(noun_id_t nid){
    switch(nid){
    case ni_glasses:
        if (Is(man_checked)){
            message("写真のヨシノ教授がかけていたものに似ている。");
        }
        else {
            msg_nothing_special();
        }
        break;
    case ni_toy_swordman:
    case ni_toy_lion:
    case ni_toy_bird:
    case ni_toy_dragon:
        message("変形合体できるようになっている。");
        break;
    case ni_robot:
        message("剣士・ライオン・鳥の玩具を変形合体させたものだ。");
        break;
    case ni_large_robot:
        message("ロボットとドラゴンの玩具を変形合体させたものだ。");
        break;
	case ni_score:
		message("ピアノ用の楽譜のようだ。");
		break;
        //    case ni_card:
        //        message("スペードのジャックのカードだ。なにか書いてある。");
        //        break;
    case ni_diary:
        msg_examine_diary();
        break;
    case ni_paper:
        message("日記帳のページを破いたもののようだ。");
        break;
	case ni_wastepaper:
		message("広げてみると、なにか書いてある。");
		message("どうやら子供の%nlのようだ。", ni_essay);
        get_item(ni_essay);
        lose_item(ni_wastepaper);
		break;
	case ni_slingshot:
		message("ゴムの力で小石などを飛ばす玩具だ。\\");
		if (Is(slingshot_repaired)){
			message("折れた柄を%nnで代用してある。",
		            ni_branch);
		}
		else {
			message("残念ながら、柄が折れているため使えない。");
		}
		break;
    case ni_belt:
        message("首輪にぶら下げられた革袋の口を開けると、"
                "中から小さな石版のようなものが出てきた。");
        lose_item(ni_belt);
        get_item(ni_amulet);
        return;
	case ni_amulet:
		message("灰白色の小さな石だ。奇妙に歪んだ星型の印が彫られている。");
		break;   
	case ni_fossil:
		message("ずっしりと重い。");
		break;   
	case ni_hat:
        if (Is(newspaper_looked)){
            message("新聞の写真で大佐がかぶっていたものと同じだ。\\");
        }
        message("乾いた血のような、茶色い染みが広がっている。");
        break;

    case ni_origami_plane:
        message("字が書いてある。"
                "ノートのページかなにかを折って作ったようだ。");
        break;
	case ni_mirror:
		message("裏側になにか書いてある。");
		break;
	case ni_chicken:
		message("ずいぶん古いもののはずだが、"
				"不思議なことに腐りもせず、今でも食べられそうに見える。");
		break;
	case ni_bottle:
		message("中には奇妙な香りのする黒い粉末が入っている。香だろうか。");
		break;
	case ni_lantern:
        if (Is(lantern_lit)){
            message("ランタンは点いている。\\");
        }
        else {
            message("ランタンは消えている。\\");
        }
		message("油はまだ十分入っている。");
		break;
    default:
        msg_nothing_special();
    }
}
void command_get_item(noun_id_t nid, flag_id_t fid){
    message("%nnを手に入れた。", nid);
	get_item(nid);
	if (fid != fi_invalid){
		set_flag(fid);
	}
}
void command_put_item(noun_id_t nid, flag_id_t fid, char *msg){
	if (msg == NULL){
		message("%nnを置いた。", nid);
	}
	else {
		message(msg, nid);
	}
	lose_item(nid);
	if (fid != fi_invalid){
		set_flag(fid);
	}
}
void command_use_item(noun_id_t nid){
    switch(nid){
    case ni_slingshot:
        if (Not(slingshot_repaired)){
            message("パチンコは壊れている。");
        }
        else if (!has_item(ni_marble)){
            message("弾にするものがない。");
        }
        else {
            message("撃つ相手がいない。");
        }
        return;
    case ni_lantern:
        if (Is(lantern_lit)){
            message("%nnはもう点いている。", nid);
        }
        else {
            if (Has(candle) && Is(candle_lit)){
                message("%nnを使って%nnを点けた。", ni_candle, nid);
                Set(lantern_lit);
            }
            else {
                message("燃料の油は入っているが、火種がない。");
            }
        }
        return;
	default:
        msg_cannot_do();
		break;
	}
}
void command_read_item(noun_id_t nid){
    switch(nid){
    case ni_leaflet:
        msg_read_leaflet();
        break;
    case ni_newspaper:
        msg_read_newspaper();
        break;
    case ni_score:
        message("「エリーゼのために」の楽譜だ。");
        break;
        //    case ni_card:
        //        msg_read_card();
        //        break;
    case ni_origami_plane:
        message("折りたたんであるので、このままでは読めない。");
        break;
        //    case ni_postcard:
        //        msg_read_postcard();
        //        break;
    case ni_essay:
        msg_read_essay();
        break;
    case ni_diary:
        msg_read_diary();
        break;
    case ni_paper:
        msg_read_diary_piece();
        break;
    case ni_letter:
        msg_read_letter();
        break;
    case ni_mirror:
        msg_read_mirror();
        break;
        //    case ni_easy_necromancy:
        //        msg_read_easy_necromancy();
        //        break;
        //    case ni_logbook:
        //        msg_read_logbook();
        //        break;
    default:
        msg_cannot_read();
    }
}
void command_combine_item(noun_id_t nid){
    clear_combine_flags();
    if (nid == ni_toy_swordman ||
        nid == ni_toy_lion ||
        nid == ni_toy_bird)
        {
            combine_query_count = 2;
        }
    set_combine_flag(nid);
    combine_base_nid = nid;
    message("何と合体させる?");
    set_command_callback(command_callback_combine);
    set_term_prompt(QUERY_PROMPT);
}

void command_dismantle_item(noun_id_t nid){
    switch (nid){
    case ni_robot:
        lose_item(ni_robot);
        get_item(ni_toy_swordman);
        get_item(ni_toy_lion);
        get_item(ni_toy_bird);
        break;
    case ni_large_robot:
        lose_item(ni_large_robot);
        get_item(ni_robot);
        get_item(ni_toy_dragon);
        break;
	case ni_slingshot:
		lose_item(ni_slingshot);
		get_item(ni_rubber_strip);
		if (Is(slingshot_repaired)){
			get_item(ni_branch);
		}
        break;
    case ni_origami_plane:
        lose_item(ni_origami_plane);
        //        get_item(ni_diary_piece);
        get_item(ni_paper);
        break;
    case ni_fishing_rod:
    case ni_sponge:
        if (Is(fishing_rod_with_sponge)){
            message("%nnを%nnから外した。", ni_sponge, ni_fishing_rod);
            Reset(fishing_rod_with_sponge);
            return;
        }
        else {
            message("%nnは分解できない。", nid);
            return;
        }
    default:
        message("%nnは分解できない。", nid);
        return;
    }
    message("%nnを分解した。", nid);
}
        

void command_remove_item(noun_id_t nid){
    switch (nid){
    case ni_sponge:
        if (has_item(ni_fishing_rod) && Is(fishing_rod_with_sponge)){
            command_dismantle_item(ni_fishing_rod);
        }
        break;
    default:
        message("%nnは外せない。", nid);
    }
}


#ifdef DEBUG
void command_teleport(noun_id_t nid){
    switch(nid){
    case ni_entrance:
        move_area(ai_entrance);
        break;
    case ni_entrance_hall:
        move_area(ai_entrance_hall);
        break;
    case ni_kitchen:
        move_area(ai_kitchen);
        break;
    case ni_dining:
        move_area(ai_dining);
        break;
    case ni_living:
        move_area(ai_living);
        break;
    case ni_bathroom:
        move_area(ai_bathroom);
        break;
    case ni_boys_room:
        move_area(ai_boys_room);
        break;
    case ni_music_room:
        move_area(ai_music_room);
        break;
    case ni_study:
        move_area(ai_study);
        break;
    case ni_library:
        move_area(ai_library);
        break;
    case ni_cellar:
        move_area(ai_cellar);
        break;
    case ni_secret_room:
        move_area(ai_secret_room);
        break;
    case ni_cave:
        move_area(ai_cave);
        break;
    case ni_working_room:
        move_area(ai_working);
        break;
    default:
        message("%nnへは転移できない。", nid);
        break;
    }
}
void command_summon(noun_id_t nid){
    if (IsNounItem(nid)){
        if (has_item(nid)){
            message("%nnはもう持っている。", nid);
            return;
        }
        else {
            message("アイテム「%nn」を召喚した", nid);
            get_item(nid);
        }
    }
}
void command_flag(noun_id_t nid){
    switch(nid){
    case ni_candle:
        toggle_flag(fi_candle_lit);
        break;
    case ni_easy_necromancy:
        toggle_flag(fi_ghost_appeared);
        break;
    case ni_armor:
        toggle_flag(fi_armor_dead);
        break;
    case ni_sponge:
        toggle_flag(fi_sponge_wet);
        break;
    case ni_magic_circle:
        toggle_flag(fi_magic_circle_invoked);
        break;
    case ni_machine:
        toggle_flag(fi_cycrotron_activated);
        break;
    case ni_water:
        toggle_flag(fi_great_old_one_gone);
        toggle_flag(fi_temple_unsunken);
        break;
	case ni_car:
		toggle_flag(fi_temple_ceiling_destroyed);
		break;
	case ni_lantern:
		toggle_flag(fi_temple_burning);
		break;
    default:
        message("%nnというフラグはない。", nid);
        break;
    }
}
#endif
void set_command_callback(int (*callback)(char *)){
    command_callback = callback;
}
void clear_command_callback(void){
    command_callback = NULL;
    set_term_prompt(NULL);
}
void enter_door(area_id_t id, flag_id_t flag){
    message("扉を開けて移動した。");
    if (flag != fi_invalid){
        set_flag(flag);
    }
    move_area(id);
}

int command_callback_input(char *line){
    char *noun;
    noun_id_t nid;

    if (*line != '\0'){
        parse_noun(line, &noun);
        nid = get_noun_id(noun);

        //        message("combine callbacked %s:%d:%s", line, nid, noun);
        switch (nid){
        case ni_english:
            message("英語入力に切り替えました。");
            set_term_input_mode(im_english);
            break;
        case ni_hiragana:
            message("ひらがな入力に切り替えました。");
            set_term_input_mode(im_hiragana);
            break;
        case ni_katakana:
            message("カタカナ入力に切り替えました。");
            set_term_input_mode(im_katakana);
            break;
        default:
            message("他の言語はわかりません。");
        }
    }
    //    clear_command_callback();
    //    set_console_prompt(NULL);
    return 0;
}



void clear_combine_flags(void){
    int i;
    for (i = 0 ; i < ni_end ; i++){
        combine_flags[i] = 0;
    }
    combine_query_count = 1;
    combine_item_count = 0;
}
void set_combine_flag(noun_id_t nid){
    assert(nid != ni_invalid);
    if (combine_flags[nid] == 0){
        combine_item_count ++;
        combine_flags[nid] = 1;
    }
}
int command_callback_combine(char *line){
    char *noun;
    noun_id_t nid;

    parse_noun(line, &noun);
    nid = get_noun_id(noun);

//    message("combine callbacked %s:%d:%s q:%d i:%d", line, nid, noun,
//            combine_query_count, combine_item_count);

    combine_query_count --;
    set_combine_flag(nid);
    if (combine_query_count > 0) return 1;
    switch (combine_base_nid){
    case ni_toy_swordman:
    case ni_toy_lion:
    case ni_toy_bird:
        if (combine_item_count < 3){
            message("これは三体合体する玩具のようだ。");
        }
        else if (combine_flags[ni_toy_swordman] &&
                 combine_flags[ni_toy_lion] &&
                 combine_flags[ni_toy_bird])
        {
            message("玩具を三体合体させて%nlにした。", ni_robot);
            get_item(ni_robot);
            lose_item(ni_toy_swordman);
            lose_item(ni_toy_lion);
            lose_item(ni_toy_bird);
        }
        else {
			msg_combine_unmatched();
        }
        break;
    case ni_robot:
    case ni_toy_dragon:
        if (combine_flags[ni_robot] && combine_flags[ni_toy_dragon]){
            message("玩具を合体させて%nlにした。", ni_large_robot);
            get_item(ni_large_robot);
            lose_item(ni_robot);
            lose_item(ni_toy_dragon);
        }
        else {
			msg_combine_unmatched();
        }
        break;
	case ni_branch:
	case ni_rubber_strip:
        if (combine_flags[ni_branch] && combine_flags[ni_rubber_strip]){
            message("%nnに%nnを結びつけて%nlを作った。",
					ni_branch, ni_rubber_strip, ni_slingshot);
			Set(slingshot_repaired);
            get_item(ni_slingshot);
            lose_item(ni_branch);
            lose_item(ni_rubber_strip);
        }
        else {
			msg_combine_unmatched();
        }
        break;
	case ni_fishing_rod:
	case ni_sponge:
        if (combine_flags[ni_fishing_rod] && combine_flags[ni_sponge]){
            if (Not(fishing_rod_with_sponge)){
                message("釣り針に%nnを引っかけた。",
                        ni_sponge);
                Set(fishing_rod_with_sponge);
            }
            else {
                message("%nnはもう釣り針に引っかけてある。",
                        ni_sponge);
            }
        }
        else {
			msg_combine_unmatched();
        }
        break;
    default:
		msg_combine_unmatched();
        break;
    }
    //    command_callback = NULL;
    //    set_console_prompt(NULL);
    return 0;
}


void strip_space(char *line){
    unsigned int code;
    int byte;
    char buf[1024];
    char *lp = line, *bp = buf;

    while (*lp != '\0'){
        code = jstrcode(lp, &byte);
        if (byte > 1){
            jstrappend(bp, code, byte);
            bp += byte;
        }
        else {
            if (*lp != ' '){
                jstrappend(bp, code, byte);
                bp += byte;
            }
        }
        lp += byte;
    }
    *bp = '\0';
    strcpy(line, buf);
}
