#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "console.h"
#include "jstr.h"
#include "jwrap.h"

// 表示行の最初と最後
static int line_num;
static int line_width;
static char **text;
static char *buf;
static int head;
static int tail;
static int offset;
static int show_height;
static int getline_ix;

static char *output_buf;
static char *output_buf_tail;

// インデックス値ixをリングバッファ内に折り返す
// ※負のインデックス式が与えられる可能性もあるため、CONSOLE_LINE_NUMを足してから
//   moduloを求める
#define Idx(ix) (((ix) + line_num) % line_num)
#define IncIdx(ix) (ix = ((ix)+1) % line_num)
#define DecIdx(ix) (ix = ((ix) + line_num - 1) % line_num)

int console_init(int lnum, int lwidth, int sheight){
    int i;
    line_num = lnum;
    line_width = lwidth;
    show_height = sheight;

    jwrap_init();

    text = malloc(sizeof(char*) * line_num);
    buf = malloc((JBufSize(line_width)+1) * line_num);
    output_buf = malloc((JBufSize(line_width)+1) * line_num);
    if (text == NULL || buf == NULL || output_buf == NULL){
        return 1;
    }
    for (i = 0 ; i < line_num ; i++){
        text[i] = buf + (JBufSize(line_width)+1) * i;
    }
    tail = 0;
    head = Idx(tail-show_height);
    output_buf_tail = output_buf;
    clear_console();
    return 0;
}


void clear_console(void){
    int i;
    for (i = 0 ; i < line_num ; i++){
        text[i][0] = '\0';
    }
    tail = 0;
    console_reset_scroll();
    //    head = Idx(tail-show_height);
    //	rollback = 0;
    //    clear_input_line();
//	shift_down = 0;
//	control_down = 0;
}


// consoleの各表示行を取得する
// console_reset_getline();
// while ((line = console_getline()) != NULL){ lineを処理 }
// で表示範囲の各行を上から順に処理できる。

void console_reset_getline(void){
    getline_ix = Idx(head + offset);
}
char *console_getline(void){
    char *line;
    if (getline_ix == Idx(tail+offset)) return NULL;
    //    printf("getline_ix:%d\n", getline_ix);
    line = text[getline_ix];
    IncIdx(getline_ix);
    return line;
}


// 改行付きでコンソールに出力
void puts_console(char *str){
    int is_line_head = 1;
    char *p = output_buf;

    strcpy(output_buf_tail, str);
    while (*p != '\0'){
        jwrap_slice(&p, text[tail], line_width, line_width, is_line_head);
        IncIdx(tail);
        IncIdx(head);
        is_line_head = 0;
    }
    output_buf_tail = output_buf;
}

// 改行なしでコンソールに出力
// strは実際にはtextではなくoutput_bufに追加されていき、
// 以降最初のputs_consoleに先だって出力される
void print_console(char *str){
    strcpy(output_buf_tail, str);
    output_buf_tail += strlen(str);
    printf("output_buffer:%s:\n", output_buf);
}

int console_scroll_up(int n){
    while (n > 0){
        if (Idx(head+offset) == Idx(tail+1)) return 1;
        offset--;
        n--;
    }
    return 0;
}
int console_scroll_down(int n){
    while (n > 0){
        if (Idx(head+offset) == Idx(tail-show_height)) return 1;
        offset++;
        n--;
    }
    return 0;
}
int console_reset_scroll(void){
    offset = 0;
    return 0;
}


#ifdef DEBUG
void show_console_buf(void){
    int i;
    printf("----- BEGIN -----\n");
    for (i = Idx(tail+1) ; i != tail ; IncIdx(i)){
        printf("%d:%s\n", i, text[i]);
    }
    printf("----- END -----\n");
}
#endif

