#include <stdio.h>
#include <EzGraph.h>
#include "config.h"
#include "game_config.h"
#include "file.h"
#include "term.h"
#include "map.h"

int load_config(void){
    FILE *fp;
    unsigned char file_major_version = 0;
    unsigned char file_minor_version = 0;
    int current_version = GetVersion(MAJOR_VERSION, MINOR_VERSION);
    unsigned char cval;

    if ((fp = open_data(CONFIG_FILE, "rb", APPLICATION_ID, APPLICATION_ID_SIZE,
                        &file_major_version, &file_minor_version)) == NULL)
    {
        return 0;
    }
    // 旧バージョンなら読み込み中止
    if (GetVersion(file_major_version, file_minor_version) < current_version){
        log_warn("Cannot read old configuration file (Ver%d.%2d)... renamed.\n",
                 file_major_version, file_minor_version);
        fclose(fp);
        rename_data(CONFIG_FILE);
        return 0;
    }
    fread(&cval, sizeof(unsigned char), 1, fp);
    set_term_input_mode(cval);
    fread(&cval, sizeof(unsigned char), 1, fp);
    set_show_minimap(cval);
    fread(&cval, sizeof(unsigned char), 1, fp);
    set_overlay_minimap(cval);
    fread(&cval, sizeof(unsigned char), 1, fp);
    set_minimap_location(cval);
    fclose(fp);
    return 1;

}

int save_config(void){
    FILE *fp;
    unsigned char file_major_version = MAJOR_VERSION;
    unsigned char file_minor_version = MINOR_VERSION;
    unsigned char cval;

    if ((fp = open_data(CONFIG_FILE, "wb", APPLICATION_ID, APPLICATION_ID_SIZE,
                        &file_major_version, &file_minor_version)) == NULL)
    {
        return 0;
    }
    cval = (unsigned char)get_term_input_mode();
    fwrite(&cval, sizeof(unsigned char), 1, fp);
    cval = (unsigned char)is_minimap_shown();
    fwrite(&cval, sizeof(unsigned char), 1, fp);
    cval = (unsigned char)is_overlay_minimap();
    fwrite(&cval, sizeof(unsigned char), 1, fp);
    cval = (unsigned char)get_minimap_location();
    fwrite(&cval, sizeof(unsigned char), 1, fp);
    fclose(fp);
    return 1;
}
