#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "input_line.h"
#include "jstr.h"

// 文字列中の各文字の情報
typedef struct {
    int ix;      // その文字の位置(line[]のインデックス)
    int width;   // 先頭からその文字までの幅
    //    int byte;    // 先頭からその文字までのバイト数
} chr_attr_t;
// 文字列中の各単語の情報

// 文字列領域
static char *line;
// 文字列長さ(バイト)
static int line_byte;
// 文字列幅(等幅フォント表示幅)
static int line_width;
// 文字列幅上限
static int line_width_max;
// 文字列中の文字数
static int chr_num;
// 文字列中の単語(空白文字で区切られた領域)数
static int word_num;

// 文字列中の各文字の情報
static chr_attr_t *chr_attrs;
// 文字列中の各単語の先頭位置
// (単語単位でchar_attrsの中身を消しやすいようにchr_attrs[]のインデックス)
static int *word_ixs;

// 現在の末尾文字が空白なら非0
// 文字列が空文字列(line_byte==0)の時も非0
static int is_tail_spc;

//static char prompt[JBufSize(PROMPT_WIDTH_MAX)+1];
static int prompt_byte = 0;
static int prompt_width = 0;

static char cursor = '_';


void terminate_input_line(void);

int input_line_init(int width_max){
    line_width_max = width_max;
    chr_attrs = malloc(sizeof(chr_attr_t)*width_max);
    word_ixs = malloc(sizeof(int)*width_max);
    line = malloc(JBufSize(width_max)+2);
    if (line == NULL) return 1;
    // フラグ設定とmalloc領域クリアのためclear()が必要
    input_line_clear();
    return 0;
}

// プロンプト・カーソルを含む行全体を取得(表示用・文字列の内容変更不可)
char *input_line_get_line(void){
    return line;
}
// 入力内容(プロンプト・カーソルを除く行)を引数領域にコピー
// inputの先は必要な大きさを確保しておくこと
int input_line_get_input(char *input){
    strncpy(input, &line[prompt_byte], line_byte-prompt_byte);
    input[line_byte-prompt_byte] = '\0';
    return 0;
}

int input_line_get_byte(void){
    return line_byte;
}
int input_line_get_width(void){
    return line_width;
}
int input_line_get_chr_num(void){
    return chr_num;
}
// 単語数(is_tail_spcが偽のときは、入力中の単語も含む
int input_line_get_word_num(void){
        return word_num;
}
int input_line_is_tail_spc(void){
        return is_tail_spc;
}


int input_line_clear(void){
    //    printf("clear %d %d\n", prompt_byte, prompt_width);
    line_byte = prompt_byte;
    line_width = prompt_width;
    chr_num = 0;
    word_num = 0;
    is_tail_spc = 1;
    terminate_input_line();
    return 0;
}

void terminate_input_line(void){
    line[line_byte] = cursor;
    //    line[line_byte] = '_';
    line[line_byte+1] = '\0';
}

int input_line_append_chr(char chr){

    if (line_width >= line_width_max) return 1;

    line[line_byte] = chr;
    chr_attrs[chr_num].ix = line_byte;
    chr_attrs[chr_num].width = line_width;
    //    char_bytes[char_num] = 1;
    line_byte ++;
    line_width ++;
    chr_num ++;
    terminate_input_line();

    if (chr == ' '){
        is_tail_spc = 1;
    }
    else {
        if (is_tail_spc){
            word_ixs[word_num] = chr_num-1; //chr_ixs[char_num-1];
            word_num ++;
            is_tail_spc = 0;
        }
    }
    return 0;
}

int input_line_append_code(unsigned int code, int byte){

    if (byte == 1) return input_line_append_chr((char)code);

    if (line_width + 2 >= line_width_max) return 1;

    jstrappend(&line[line_byte], code, byte);
    chr_attrs[chr_num].ix = line_byte;
    chr_attrs[chr_num].width = line_width;
    line_byte += byte;
    line_width += 2;
    chr_num ++;
    terminate_input_line();

    if (is_tail_spc){
        word_ixs[word_num] = chr_num-1; //chr_ixs[char_num-1];
        word_num ++;
        is_tail_spc = 0;
    }
    return 0;
}

int input_line_set_str(char *str){
    input_line_clear();
    return input_line_append_str(str);
}

int input_line_append_str(char *str){
    unsigned int code;
    int byte;
    while (*str != '\0'){
        code = jstrcode(str, &byte);
        input_line_append_code(code, byte);
        str += byte;
    }
    return 0;
}


int input_line_backspace(void){
    if (line_byte == prompt_byte) return 1;

    //    printf("bs chr_num:%d word_num:%d is_tail:%d  (%d, %d)\n", chr_num, word_num, is_tail_spc, word_ixs[word_num-1], chr_attrs[chr_num-1].ix);
    //    if (word_ixs[word_num-1] == chr_attrs[chr_num-1].ix){
    if (word_ixs[word_num-1] == chr_num-1){
        word_num --;
        is_tail_spc = 1;
    }
    else {
        is_tail_spc = 0;
    }
    chr_num --;
    assert(chr_num >= 0);
    line_byte = chr_attrs[chr_num].ix;
    line_width = chr_attrs[chr_num].width;
    //    printf("bs2 chr_num:%d word_num:%d is_tail:%d\n", chr_num, word_num, is_tail_spc);
    /*
    byte = char_bytes[char_num];
    line_byte -= byte;
    line_width -= (byte==1) ? 1 : 2;
    */
    terminate_input_line();
    return 0;
}

// 末尾の単語を削除
int input_line_delete_word(void){
    if (word_num == 0) return 1;

    word_num --;
    is_tail_spc = 1;
    chr_num = word_ixs[word_num];
    line_byte = chr_attrs[chr_num].ix;
    line_width = chr_attrs[chr_num].width;
    terminate_input_line();
    return 0;
}

int input_line_set_prompt(char *new_prompt){
    int new_width, new_len, len;

    new_width = jstrwidth(new_prompt);
    if (new_width > PROMPT_WIDTH_MAX) return 1;
    new_len = strlen(new_prompt);
    // カーソルも移動するため+1
    len = line_byte - prompt_byte + 1;
    // '\0'の分を+1
    memmove(line+new_len, line+prompt_byte, len+1);

    line_width -= prompt_width;
    prompt_width = new_width;
    line_width += prompt_width;

    line_byte -= prompt_byte;
    prompt_byte = new_len;
    line_byte += prompt_byte;

    assert(prompt_byte < JBufSize(PROMPT_WIDTH_MAX));
    memcpy(line, new_prompt, new_len);


    //    printf("set prompt %d %d %d %d|%s|%s|\n", prompt_width, prompt_byte, new_len, len, new_prompt, line);
    return 0;
}

int input_line_set_cursor(char new_cursor){
    //    printf("change cursor %c->%c\n", cursor, new_cursor);
    cursor = new_cursor;
    terminate_input_line();
    return 0;
}

#ifdef DEBUG
void input_line_print_line(void){
    int i;
    printf("%d %d|%s|\n", prompt_byte, line_byte, line);
    printf("chrs:%d:", chr_num);
    for (i = 0 ; i < chr_num ; i++){
        printf("%2d(%2d, %1d)", i, chr_attrs[i].ix, chr_attrs[i].width);
    }
    puts("");
    printf("words:%d(%d):", word_num, is_tail_spc);
    for (i = 0 ; i < word_num ; i++){
        printf("%2d, ", word_ixs[i]);
    }
    puts("");
}
#endif


