#include <stdio.h>
#include <string.h>
#include "jwrap.h"
#include "jstr.h"

unsigned int forbid_tail_codes[4][FORBID_CHR_NUM_MAX];
int forbid_tail_code_num[4];
unsigned int forbid_head_codes[4][FORBID_CHR_NUM_MAX];
int forbid_head_code_num[4];
//unsigned int forbid_dangling_codes[4][FORBID_CHR_NUM_MAX];
//int forbid_dangling_code_num[4];

static char is_initialized = 0;

int jwrap_init(void){
    char *p;
    unsigned int code;
    int byte;

    if (is_initialized) return 1;

    p = FORBID_TAIL_CHRS;
    while ((code = jstrcode(p, &byte)) != 0){
        //        printf("%d %p ", code, p);
        forbid_tail_codes[byte][forbid_tail_code_num[byte]] = code;
        forbid_tail_code_num[byte] ++;
        p += byte;
    }
    //    puts("");
    p = FORBID_HEAD_CHRS;
    while ((code = jstrcode(p, &byte)) != 0){
        //        printf("%d %p ", code, p);
        forbid_head_codes[byte][forbid_head_code_num[byte]] = code;
        forbid_head_code_num[byte] ++;
        p += byte;
    }
    //    puts("");
    /*
    p = FORBID_DANGLING_CHRS;
    while ((code = jcode_slice_code(&p, &byte)) != 0){
        printf("%d %p ", code, p);
        forbid_dangling_codes[byte][forbid_dangling_code_num[byte]] = code;
        forbid_dangling_code_num[byte] ++;
    }
    puts("");
    */
    is_initialized = 1;
    return 0;
}



int jwrap_slice(char **src, char *dest, int width_slimit, int width_hlimit,
                int is_line_head)
{
    int byte, width = 0, i;
    char buf[8];
    char *dest_tail, *src_tail;
    int slimit_num = 0, slimit_bytes[1024];
    unsigned int code = 0;
    int append, forbidden_tail;

    if (is_line_head == 0){
        while (**src == ' '){
            (*src) ++;
        }
    }

    // soft limitまではそのまま切り出す
    // 追い出し禁則用に各文字のバイト数をcbytesに記録
    while (**src != '\0' && width < width_slimit){
        code = jstrcode(*src, &byte);
        *src += byte;
        width += (byte == 1) ? 1 : 2;
        slimit_bytes[slimit_num] = byte;
        slimit_num ++;
        jstrappend(dest, code, byte);
        dest += byte;
    }
    // 最後に追加した文字が行末禁則ならフラグを立てる(さらに文字追加が必要)
    forbidden_tail = 0;
    for (i = 0 ; i < forbid_tail_code_num[byte] ; i++){
        if (code == forbid_tail_codes[byte][i]){
            forbidden_tail = 1;
        }
    }
    src_tail = *src;
    dest_tail = dest;
    // hard limitまでぶら下げ禁則を行う
    //    printf("ぶら下げ禁則: %d (%d:%d)\n",
    //           forbidden_tail, width, width_hlimit);
    //    while (**src != '\0' && width < width_hlimit){
    while (**src != '\0'){
        code = jstrcode(*src, &byte);
        *src += byte;
        append = 0;
        // 直前が行末禁則文字の場合
        if (forbidden_tail){
            // 直前の文字が行末に来ないよう、この文字は出力
            append = 1;
            // 行末禁則文字でなければ、行末禁則文字の並びは終了
            // さらに行末禁則文字が続くなら、フラグを立て続ける
            forbidden_tail =0;
            for (i = 0 ; i < forbid_tail_code_num[byte] ; i++){
                if (code == forbid_tail_codes[byte][i]){
                    //                    jcode_append_code(&dest, code, byte);
                    //                    width += (byte == 1) ? 1 : 2;
                    forbidden_tail = 1;
                    break;
                }
            }
        }
        // 直前が行末禁則文字でない場合
        // 行頭禁則文字でなければ、ここで終わってよい
        else {
            // 行頭禁則文字なら、次行頭になるのを防ぐため出力に追加する
            for (i = 0 ; i < forbid_head_code_num[byte] ; i++){
                if (code == forbid_head_codes[byte][i]){
                    //                jcode_append_code(&dest, code, byte);
                    //                width += (byte == 1) ? 1 : 2;
                    append = 1;
                    break;
                }
            }
        }
        // 今回の文字が追加されなければ、禁則処理は終了
        if (append == 0){
            *src -= byte;
            break;
        }
        // 追加されたときは、さらに続ける
        else {
            //if (forbidden_tail){
            jstrappend(dest, code, byte);
            dest += byte;
            width += (byte == 1) ? 1 : 2;
            //          break;
        }
    }
    if (width > width_hlimit){
        //        printf("ぶらさげ禁則失敗(%d>%d)\n", width, width_hlimit);
        slimit_num --;
        dest_tail -= slimit_bytes[slimit_num];
        src_tail -= slimit_bytes[slimit_num];
        *src = src_tail;
        //        width = width_slimit;
        width = width_slimit - slimit_bytes[slimit_num];
        while (1){
            code = jstrcode(*src, &byte);
            jstrappend(buf, code, byte);
            buf[byte] = '\0';
            //            printf("checking tail |%s|(%d)\n", buf, byte);
            /*
            // 現在の行末文字が行末禁則ならさらにさかのぼる
            for (i = 0 ; i < forbid_tail_code_num[byte] ; i++){
                if (code == forbid_tail_codes[byte][i]){
                    slimit_num --;
                    dest_tail -= slimit_bytes[slimit_num];
                    *src -= slimit_bytes[slimit_num];
                    printf("cont|%s|\n", *src);
                    continue;
                }
            }
            */
            // 現在の行末文字が行末禁則ならさらにさかのぼる
            for (i = 0 ; i < forbid_head_code_num[byte] ; i++){
                if (code == forbid_head_codes[byte][i]){
                    slimit_num --;
                    dest_tail -= slimit_bytes[slimit_num];
                    *src -= slimit_bytes[slimit_num];
                    width -= slimit_bytes[slimit_num];
                    //                    printf("cont|%s|\n", *src);
                    continue;
                }
            }
            /*
            code = jcode_get_code(*src - slimit_bytes[slimit_num-1], &byte);
            jcode_code2str(buf, code, byte);
            printf("checking head |%s|(%d)\n", buf, byte);
            for (i = 0 ; i < forbid_head_code_num[byte] ; i++){
                if (code == forbid_head_codes[byte][i]){
                    slimit_num --;
                    dest_tail -= slimit_bytes[slimit_num];
                    *src -= slimit_bytes[slimit_num];
                    continue;
                }
            }
            */
            break;
        }
        *dest_tail = '\0';
    }   

    *dest = '\0';
    return width;
}
