#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "config.h"
#include "message.h"
#include "layout.h"
#include "console.h"
#include "jstr.h"
#include "command.h"
#include "verb.h"

void message(char *msg, ...){
    va_list ap;
    char buf[MESSAGE_MAX_LEN];
    char *msgp = msg, *bufp = buf, *p;
    int n;
    noun_id_t nid;
    verb_id_t vid;
    int byte;
    unsigned int code;

    printf("message:%s:\n", msg);

	va_start(ap, msg);
    while (*msgp != '\0'){
        //        printf("bufp:%p msgp:%s:\n", bufp, msgp);
        code = jstrcode(msgp, &byte);
        msgp += byte;
        if (byte > 1){
            jstrappend(bufp, code, byte);
            bufp += byte;
        }
        else if (code == '\\'){
            printf("!!!\n");
            *bufp = '\0';
            print_term(buf);
            bufp = buf;
        }
        else if (code == '%'){
            switch(*msgp){
            case 's':
                p = va_arg(ap, char *);
                strcpy(bufp, p);
                bufp += strlen(p);
                break;
            case 'd':
                n = va_arg(ap, int);
                bufp += sprintf(bufp, "%d", n);
                break;
            // idで指定した名詞を表示
            case 'n':
                nid = va_arg(ap, noun_id_t);
                msgp ++;
                switch(*msgp){
                // 名詞の名称・単語を表示
                case 'l':
                    p = noun_name(nid);
                    strcpy(bufp, p);
                    bufp += strlen(p);
                    *bufp++ = '[';
                    p = nid2noun(nid);
                    strcpy(bufp, p);
                    bufp += strlen(p);
                    *bufp++ = ']';
                    break;
                // 名詞の名称を表示
                case 'n':
                    p = noun_name(nid);
                    strcpy(bufp, p);
                    bufp += strlen(p);
                    break;
                // 名詞の単語を表示
                case 'w':
                    *bufp++ = '[';
                    p = nid2noun(nid);
                    strcpy(bufp, p);
                    bufp += strlen(p);
                    *bufp++ = ']';
                    break;
                default:
                    printf("%%n%c is not supported in message().", *msgp);
                    exit(1);
                }
                break;
            case 'v':
                vid = va_arg(ap, verb_id_t); 
                p = vid2verb(vid);
                //                printf("hoge:%d:%s:\n", vid, p);
                strcpy(bufp, p);
                bufp += strlen(p);
                break;
			// その位置でメッセージ出力を一時中断しリターンキーの入力待ちをする
            // 実際の処理はtermに任せる。
            case 'w':
                *bufp = '\0';                
                puts_term_waited(buf);
                bufp = buf;
                break;
            default:
                printf("%%%c is not supported in message().", *msgp);
                exit(1);
            }
            msgp ++;
        }
        else {
            *bufp++ = code;
        }
    }
    *bufp = '\0';
    va_end(ap);

    // message("")は改行するためputs_termする
    // メッセージが空でないがbufに文字列が貯まっていないのは
    // すでにputs/print済みなので、余計な改行を追加しないようputsしない
    if (bufp != buf || *msg == '\0'){
        puts_term(buf);
    }
}

Msg(cannot_do, "ここではできない。");
Msg(cannot_destroy, "壊すのは無理そうだ。");
Msg(cannot_read, "それは読めない。");
Msg(cannot_move, "びくともしない。");
Msg(cannot_climb, "よじ登るのは無理だ。");
Msg(no_effect, "なにも起こらなかった。");
Msg(nothing_special, "とくに変わったところはない。");
Msg(no_object, "ここには見あたらない。");
Msg(nothing_any_more, "もう何もない。");
Msg(need_to_get, "まずは手に取ってみたほうがいい。");
Msg(no_need_to_carry, "持って行く必要はないだろう。");
Msg(which_door, "扉は複数ある。");
Msg(which_bookshelf, "本棚は複数ある。");
Msg(door_locked, "扉には鍵が掛かっている。");
Msg(door_unlocked, MSG_DOOR_UNLOCKED);
Msg(door_already_unlocked, MSG_DOOR_ALREADY_UNLOCKED);
Msg(door_closed, "扉が閉まっている。");
Msg(door_already_closed, "扉はすでに閉まっている。");
Msg(door_unlocked_by_opening, "扉の取っ手を回すと、ガチャリと鍵が外れた。");
Msg(key_unmatched, "この鍵は鍵穴に合わない。");
Msg(too_large, "持っていくには大きすぎる。");
Msg(too_heavy, "重すぎる。");
Msg(too_far, "ここからでは遠すぎる。");
Msg(combine_unmatched, "これは組み合わせられない。");
Msg(bootstrap, "そんな真似ができるのはミュンヒハウゼン男爵ぐらいだ。");
Msg(multiple_climb, "これ以上どこへ登ろうというのか。");

void msg_armor_standing(void){
    message("目の前には%nnが立っている。", ni_armor);
}
void msg_look_standing_armor(void){
    message("%nlを構え、今にもあなたに斬りかかろうとしている！",
            ni_sword);
}
Msg(look_standing_armor_sword, "切れ味が良さそうだ。");
Msg(no_magic_circle, "もう魔法陣は残っていない。");
Msg(blocked_by_furniture, "並んだ家具が邪魔で近づけない。");
Msg(no_space_to_move, "動かす場所がない。");


void msg_to_start(void){
    message("「%v」(最初から)または"
			"「%v」(前回の続き)と入力してください。", vi_start, vi_load);
}
void msg_start(void){
    message("あなたは海辺の崖の上に立つ古い屋敷にやってきた。");
    message("ここは大戦前に大富豪が建てた家で、莫大な財宝が眠ると言われている。"
            "しかし、失踪者が相次ぐ呪われた館としても有名であり、"
            "最近も著名な冒険家であるサンダース大佐がここで行方不明になったと"
            "新聞で報じられたばかりだ。");
    message("手元には新聞の切り抜き%nwがある。", ni_newspaper);
}

void msg_to_title(void){
    message("タイトルに戻るなら「%v」、"
            "データをロードするなら「%v」と入力してください。",
            vi_title, vi_load);
}
void msg_read_newspaper(void){
    message("「『サンダース大佐、幽霊屋敷で行方不明か！？』");
    message("　冒険家のサンダース大佐が、"
            "アーラムの旧マクドナルド邸を調査に出かけたまま"
            "帰宅していないことが判明した。"
            "同邸は1930年に"
            "大富豪の氏が妻の療養のために建てたが、"
            "夫人病死後に氏が失踪したのを皮切りに行方不明者が相次ぎ、"
            "1995年のヨシノ教授一家失踪事件後は空き家となっている。"
            "以前より財宝の噂がある一方、"
            "入った者が帰ってこない呪われた屋敷として、"
            "地元では怖れられていた」");
}
void msg_look_newspaper(void){
       message("サンダース大佐の行方不明を報じた記事だ。"
                "「今年の夏、愛犬と共に中央アジアから帰国した"
                "ケネス・サンダース大佐」と"
                "見出しのついた写真に、大佐らしき人物が写っている。");
}
void msg_read_essay(void){
    message("「ぼくのお姉ちゃん　　アーサー・ヨシノ");
    message("ぼくのお姉ちゃんはわがままです。"
            "大きくなってぼくと別の部屋になったとき、"
            "この家にあったあんてーくのベッドをもらいました。"
            "でも、いまでも二段ベッドのうえは"
            "お姉ちゃんのりょうちだといって、"
            "よく本を読みにきます。"
            "ぼくがあがるとすごく怒ります。"
            "このまえ、こっそりぬいぐるみのしっぽを結んでおいたらぶたれました。"
            "ぼうくんだとおもいます。でもやさしいときもあります"
            "」");
}
void msg_read_diary(void){
	message("日記の最後の方を読んでみた。");
    message("「12月22日。"
			"アーサーがまた叱られた。"
            "図書室に入ったお父さんが本棚のかげのおもちゃにつまづいたみたい。"
			"おもちゃが軽かったから怪我しなくてよかった。");
    message("　12月23日。"
            "最近よく変な夢を見る。"
            "ベッドで寝ていると、"
            "地下室から何か近づいてくるような。"
            "明日はクリスマスイブ。"
		    "みんなで楽しく過ごせるといいな"
            "」");
            message("これ以降のページは白紙だ。");
}
void msg_examine_diary(void){
    message("日記をぱらぱらめくると、破られたページがあった。");
    message("「10月7日。"
			"アーサーがここに上がりこんで遊んでいた。"
			"部屋が別になっても上のベッドは私のだって言ってるのに。"
			"おまけに、大事なダイアナの」");
    message("この次のページが根本から破りとられている。");
}

void msg_read_diary_piece(void){
	message("「しっぽが結んであったから、"
			"思わず引っぱたいてしまった。"
			"アーサーには悪いことをしちゃった。あやまらなくちゃ」");
}
void msg_read_logbook(void){
    message("看護人によるマクドナルド夫人の看護記録だ。");
    message(
            "最初の頃はまだ元気だったらしく、音楽室でピアノを弾いたり、"
            "食堂で夫と食事をとったりする様が書かれている。"
            "しかし徐々に衰弱が進み、"
            "途中からは音楽室隣の寝室でほとんど寝たきりとなっている。"
            "マクドナルド氏も金を惜しまず手を尽くしたが"
            "治療の目処が立たず、憔悴しきっていたらしい。"
            "記録の最後の方では、怪しげなオカルト儀式に没頭する氏を、"
            "看護人がひどく気味悪がっている。"
            );
}
void msg_read_postcard(void){
	message("「こちらは元気でやっている。"
			"オリオン"
			"(ちなみに3代目だ。"
			"君が会った2代目は昨年天に召された)"
			"と遺跡の探索に明け暮れているよ。");
    message("  君の手紙にあったマクドナルド邸の話は、"
		    "何やら危険な臭いがする。"
			"戻りしだい手を貸すが、"
			"こちらが片付くまで時間がかかりそうだ。"
			"あまり無理をせず、変わったことがあればすぐに屋敷を出たほうがいい」"
		);
}
void msg_read_sanders_note(void){
    message("「ジャックへ");
    message("君がこれを読んでいるなら、"
            "私の身になにかあったということだろう。"
            "深入りは危険だ。"
            "いざというときは、車に積んであるガソリンを使ってくれ。");
    message("犬が無事だったら連れ帰って欲しい。"
            "無愛想な奴だが、名前を呼べば言うことを聞くだろう。");
    message("　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　ケニー」");
}
void msg_read_easy_necromancy(void){
	message("『5分でできる交霊術』という、うさんくさい内容が書かれている。");
    msg_recall_easy_necromancy();
}
void msg_recall_easy_necromancy(void){
	message("「術式の場には、死者の想いの深い場所を選ぶべし");
	message("　死者と縁の深い品を、依代として配すべし");
	message("　供物と燭台を整えたのち、祈るべし」");
}

void msg_read_yoshino_notebook(void){
	message("ほとんどのページには数式や図が書き散らされているが、"
            "最後の方に手記のようなものが記されている。");
	msg_recall_yoshino_notebook();
    message("　手記はここで終わっている。");
}
void msg_recall_yoshino_notebook(void){
	message("「人間のような悪意を持つ存在とも思えないが、"
			"奴は狙ったかのように、クリスマスイブの夕べに私たちを支配しようとした。"
			"妻と子供たちの意識はすでにない。"
			"私はまだ完全には支配されていないが、"
			"いつまで耐えられるか。%w");
	message("　奴が私を完全に支配しないのは、"
			"地下の機械を修理させるためらしい。"
			"思考力を失った操り人形では、そうした仕事は無理なのだろう。"
			"屋敷のあちこちに奴の監視の眼があり脱出は不可能だが、"
			"こちらの考えや行動がすべて見られているわけでもないようだ。"
			"なんとか奴の裏をかけないものか。%w");
	message("　図書室を探し回って、"
			"わずかながらマクドナルドが残した資料を見つけた。"
			"マクドナルドはサイクロトロン機関と呼んでいるが、"
			"私が知るサイクロトロンとは異なる装置のようだ。"
			"詳細は不明だが一種の動力炉であり、"
			"マクドナルドが奴を召喚した際、"
			"異界の門を開く動力源にしたようだ。%w");
	message("　わざと修理を遅らせながら資料を調べているが、"
            "魔術的な儀式については専門外でさっぱりわからない。"
			"こんなときに大佐がいてくれれば。"
            "昨夜は疲労のせいか、浴室の鏡に見知らぬ女性が映ったのを見て、"
            "錯乱して鏡を叩き割ってしまった。"
            "私もそろそろ危ないようだ。%w");
    message("　奴にこちらの意図を隠すのも限界だろう。"
            "機関の修理方法は、奴の目に触れないところに書いておく。"
            "自分の手で決着をつけたかったが、無念だ。"
            "もう一度皆で食卓を囲むことができたなら、私は」");
}

void msg_read_bathroom_1f_wall(void){
	message("「奴の支配力が強まってきた。"
			"もはやこれまでのようだ。");
	message(
			"　マクドナルドがサイクロトロン機関のエネルギーで"
            "異界の門を開いたのであれば、"
			"同じようにして仲間を呼び寄せることも可能だろう。"
            "私が機関の動かし方を理解したと"
			"奴が気づいたのであれば、"
			"生きたまま支配されるわけにもいくまい。"
			"しかし、このエネルギーで奴を送還することもできるはずだ。"
			"機関の燃料は」");
    message("メッセージはここで途切れている。");
}
void msg_read_mirror(void){
	message("「十分に残っているが、"
			"触媒としてカルシウムを投入する必要がある。"
			"多少の不純物なら炉の反応には影響ないから、"
			"カルシウムを含んでいればなんでもよい。");
	message("　万が一にも奴に利用されないよう、"
            "筐体内のケーブルはショートさせてある。"
			"ケーブル間を絶縁してから緑の始動ボタンを押せば、"
			"問題なく動作するはずだ」");
}
void msg_read_mcdonald_notebook(void){
	msg_recall_mcdonald_notebook();
    message("　手記はここで終わっている。");
}
void msg_recall_mcdonald_notebook(void){

    message("「死病に冒された妻を救うためとはいえ、"
            "魔術に頼ったのが過ちであった。");
    message("　儀式は成功し、現れたものはこう言った。"
            "代償と引き替えに、汝の望みを叶えてやると。"
            "悪魔の類としか思えなかったが、私は迷わず答えた。"
            "『妻の命を救ってくれれば館の中のものはすべてやる』と。"
            "それがすべての間違いだった。%w");
    message("　妻のためなら全財産も惜しくはなかったが、"
            "狡猾にも彼の者は、館の中の人間もすべて自分のものとしたのだ。"
            "使用人も妻も、そして私も。"
            "彼の者はたしかに妻を死から救ってくれたが、"
            "彼女の肉体はもはやあれの依代に過ぎない。"
            "魔法陣に守られたこの部屋を出た瞬間、"
            "私もまた彼の者の下僕と成り果てるだろう。%w");
    message("　外では傀儡と化した使用人たちが、"
            "私を嘲笑するかのように動き回っている。"
            "私自身にも彼の者の支配が及ぶゆえに"
            "万に一つの望みもないが、彼らが遠ざかるのを待ち、"
            "屋敷からの脱出を図るしかない。");
    message("　いや、そんな！あの音はなんだ。扉が！扉が！%w");
    message("　どうやら入り口を塞がれたようだ。"
            "もはや出ることもかなうまい。"
            "今となっては、残された時間に焦るあまり、"
            "送還に必要な旧神の印を入手できぬまま"
            "儀式を強行したことが悔やまれる。"
            "願わくは、この屋敷を訪れる誰かが、あれを」" );
}
void msg_read_leaflet(void){
    message("「『アーラムの先住民遺跡とその文化』");
    message("　アーラム近郊にも、北米大陸全体に見られるような先住民の"
            "遺跡が幾つかあることが知られている。"
            "しかし一方でこの地方には、"
            "忌まわしき姿の神を崇める異形の種族の伝説があり、"
            "既知のどの文化にも属さない奇妙な石器が出土する。"
            "ミスカトニック大学による最新の発掘調査の成果を報告する」");
}
void msg_read_letter(void){
    message("「儀式はあなたの書かれた方法で間違いありません。"
			"燻香も首尾良く入手されたとのこと。"
		    "しかし現代人が復元した紛い物では、"
            "滅び去りし種族と同じ儀式を行っても効果は望み薄です。"
			"あなたのお考えのように"
			"科学の力で魔法陣を強化すれば、"
			"門を開くだけの力が得られるかもしれません」"
			);
}
void msg_read_secret_room_book(void){
//    message("≪むまのきつ りやう きつにのをか なかくぼれいり くれんとう≫"
//    message("≪mumanokiz lyau kizni nooka nakak boleily qrento≫"
//    message("≪むうまのきっつ りゃう きっつに ぬうか なかっく ぼれいりい くれんと≫"
//    message("なにやらオカルトめいた内容でさっぱりわからないが、"
      message("オカルトじみた冒涜的な内容の本をぱらぱらめくっていくと、"
              "印のついたページが見つかった。%w");
	msg_recall_secret_room_book();
	message("肝心の呪文の部分が、虫に食われていて一部しか読み取れない。");
	message("呪文の下には、歪んだ星のような文様が描かれている。");
}


void msg_recall_secret_room_book(void){
	message("「極東の高僧が記した『無為の書』には、"
			"死者を蘇らせようとして"
			"異界の存在を呼び出してしまったとき、"
			"祭壇に灯をともし燻香を焚いたのち"
			"旧神の印を掲げて以下の呪文を唱えることで、"
			"眼前の忌まわしき者を送り還したとある。");
//    message("≪むまのき○ ○や○ ○○○○○○ ○か○○れいり くれんとう≫」");
//    message("≪むうまのきっつ りゃう きっつに ぬうか なかっく ぼれいりい くれんと≫」");
    if (get_term_input_mode() == im_english){
        message("≪mumanoki_ _y__ _____ _____ __k_k ___eily qrento≫」");
    }
    else {
        message("≪むうまのき○○ ○ゃ○ ○○○○ ○○○ ○か○○ ○れいりい くれんと≫」");
    }
}


void msg_read_bathroom_2f_wall(void){
	message("両端は不鮮明だが、判読できる部分は以下のように読める。");
    if (get_term_input_mode() == im_english){
        message("≪okiz lyau kizni nooka nakak bolei≫");
    }
    else {
        message("「のきっつ りゃう きっつに ぬうか なかっく ぼれい」");
    }
}


void msg_read_boiler_room_wall(void){
	msg_recall_boiler_room_wall();
    message("　手記はここで終わっている。");
}
void msg_recall_boiler_room_wall(void){
    message("「我ながら迂闊であった。"
            "屋敷内に召喚の魔法陣が見あたらないため、"
            "地下にあると踏んだのだが、"
            "ここで袋の鼠になろうとは。"
			"タカアキ達が消えてから15年以上たち、"
			"奴も休眠状態だと思っていたのだが、"
			"財宝目当てにここを訪れた連中が"
			"格好の餌になっていたようだ。"
			"%w");
    message("　来る前に調査したところでは、この地方の先住民には"
            "名状し難い支配者に仕える種族の存在が伝えられており、"
            "前世紀には開拓民が海岸の洞窟の奥で冒涜的な神殿を見たという"
            "記録も残っている。"
			"%w");
    message("　今は滅び去った種族の悍ましい神を"
            "マクドナルドが呼び出してしまったのであれば、"
            "その本体は、投影陣の仕掛けがあった庭園の真下だろう。"
			"幸い、こちらの切り札はオリオンのベルトにあり、今のところは安全だ。"
			"%w");
    message("　この部屋の装置は投影陣の動力源のようだが、"
			"故障しているようで私の手には負えない。"
			"タカアキならなんとかできたろうに。"
			"ここを出られたら、何か資料がないか彼の部屋を探してみよう。"
			"%w");
	message("　なんとしても上に戻り、魔法陣を見つけなければ。"
            "祭壇の炎に薫香をくべれば、魔法陣が起動するはずだ。"
			"あとはこの装置を動かすことができれば、"
            "投影陣の下で旧神の印を手に呪文を唱え、"
			"奴を送り還せるだろう。"
            "%w");
	message("　奴の触手は今もこの部屋の外で蠢いており、"
			"私を待ち受けている。"
			"ショットガン一丁では蟷螂の斧だが、"
			"ここは一か八かだ」");
}




void msg_touching_water(void){
    message("水面に手を伸ばすと、澱んだ水が動いたように見えた。");
    message("見間違いかと思ったそのとき、水面が急に盛り上がり、"
            "あなたの手を包み込んだ。"
            "生暖かい水が腕を這い上がり、"
            "逃げる間もなく全身を包み込む。");
}
void msg_shadow_appeared(void){
	message("遠くの方で金切り声のような叫びが上がり、何者かが廊下を近づいてくる音がする！");
}
void msg_shadow_behind_door(void){
	message("部屋のすぐ外に、何者かが立っている気配がする。"
			"気のせいか、肌寒い感じがする。");
}
void msg_shadow_approaching(void){
	message("廊下の向こうから、異様に歪んだ人の姿のような黒い影が近づいてきた！");
}
void msg_shadow_in_room(void){
    message("扉が開き、異様に歪んだ人の姿のような黒い影が入ってきた。"
            "影が発する冷気で、体が凍りつきそうだ。");
}
void msg_shadow_in_room_hiding(void){
    message("扉が開く音がして、何者かが部屋に入ってきたようだ。");
}
void msg_shadow_wandering(void){
	message("室内を歩きまわる足音が聞こえ、"
			"箪笥の隙間から体が凍りつくような冷気が入り込んでくる。");
}
void msg_touching_shadow(void){
    //    if (IsCurrentArea(ladys_room) && IsNewArea(corridor_2f_cr)){
    if (MovingTo(corridor_2f_cr)){
        message("いきなり黒い影があなたに覆い被さった。");
    }
    else {
        message("黒い影はあっという間に目の前に迫り、あなたに覆い被さった。");
    }
    message("視界が闇に閉ざされ、"
            "切り裂くような寒さに身が包まれる。");
}
void msg_shadow_gone(void){
	message("息を潜めていると、やがて足音は部屋を出て、廊下を遠ざかっていった。");
}

void msg_phone_call_started(void){
	message("入ったとたん、部屋の隅の電話が鳴り始めた。");
}
void msg_phone_call_continued(void){
	message("電話はまだ鳴り続けている。");
}
void msg_phone_call_stopped(void){
	message("電話は鳴りやんだ。");
}
void msg_answer_phone_call(void){
	message("受話器を耳に当てると、"
			"地の底に響くような冒涜的な声が聞こえてきた。");
	message("「馬鹿め、サンダースは死んだわ！」");
	message("それきり電話はうんともすんとも言わなくなった。");
}
void msg_clock_adjusting_guide(void){
    message("(「0715」のように数字4桁で入れてください。入力せずリターンキーで中止)");
}
void msg_clock_door_opened(void){
	message("壁の時計が鳴りだし、"
	  	    "文字盤の上の小窓からなにか飛び出してきた。");
}
void msg_clock_door_closed(void){
	message("時計が鳴り終わり、小窓が閉じた。");
}

void msg_attacked_on_2f(void){
    message("二階に足を踏み入れたとたん、視界が真っ暗になった。");
    message("身体が凍り付くような冷気に包まれ、動くことができない。");
    message("あなたの意識はそのまま途切れた…。");
}
void msg_moved_on_2f(void){
    message("気がつくと、見知らぬ部屋にいた。"
            "どうやら助かったようだ。");
}

void msg_secret_passage(void){
    message("本棚があった場所に%nlがある。", ni_passage);
}
void msg_locked_in_secret_room(void){
    message("図書室に戻ろうとしたが、隠し扉が閉まっている。");
    message("あなたは必死に開けようとしたが、"
            "こちら側からはどうしても開かなかった…。");
}
void msg_starved(void){
    message("…あれから何日たっただろうか。"
            "他に出口もなく、ここには水も食料もない。");
    message("迂闊にもあなたは、マクドナルドの轍を踏んでしまったようだ。");
 }

void msg_far_door_opened(void){
    message("遠くで扉が開く音がした。");
}
void msg_armor_moving_sound(void){
    message("ガシャガシャと音を立てて、何かが近づいてくる。");
}
void msg_armor_moving(void){
    message("ガシャガシャと音を立てて、鎧が近づいてくる。");
}
void msg_armor_into_guestroom(void){
    message("鎧は客室の扉を開けると、中に入っていった。");
}
void msg_armor_appeared(void){
    message("目の前に、刀を構えた%nlが現れた!", ni_armor);
}
void msg_killed_by_armor(void){
    message("鎧は無言で刀を振り下ろし、あなたは真っ二つになった。");
}
void msg_exiting_window(void){
    message("外に出ようと窓から身を乗り出したとき、"
            "勢いよく落ちてきた窓枠が、あなたの身体を真っ二つにした。");
}
void msg_feeling_magic_circle_power(void){
    message("気のせいか、魔法陣全体から不思議な力を感じる。");
}
void msg_looking_pocket_notebook(void){
    message("%nnの傍には%nlが落ちている。",
            ni_skeleton, ni_pocket_notebook);
}
void msg_no_light(void){
    message("真っ暗で何も見えない。"
            "灯りを見つけてきた方がよさそうだ。");
}

void message_dragon_too_high(noun_id_t nid){
    message("%nnに乗って手を伸ばしても、"
            "%nnは手の届かないところに逃げてしまう。", nid, ni_toy_dragon);
}
