#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "roman_conv.h"

typedef enum {
    v_invalid = 0,
    v_l = 1,
    v_x,
    v_a, v_i, v_u, v_e, v_o,
    v_k,
    v_s,
    v_t,
    v_n,
    v_h,
    v_m,
    v_y,
    v_r,
    v_w,

    v_g,
    v_z,
    v_d,
    v_b,
    v_p,

    v_c,
    v_f,
    v_j,
    v_q,
    v_v,
    v_minus,
} char_value_t;

#define IsVowel(v) ((v)<=v_o)

// char_valueを足し合わせるときにシフトするビット数
// char_value_tの最大値が納まるだけのビット数が必要
#define SHIFT 5
// 
#define STR_VALUE_MAX ((v_l<<(SHIFT*3))+(v_t<<(SHIFT*2))+(v_s<<SHIFT)+v_u)


typedef struct {
    unsigned char c;
    char_value_t cv;
} c_cv_t;
c_cv_t c_cv[] = {
    {'\0', v_invalid},
    {' ', v_invalid},
    {'a', v_a},
    {'i', v_i},
    {'u', v_u},
    {'e', v_e},
    {'o', v_o},
    {'k', v_k},
    {'s', v_s},
    {'t', v_t},
    {'n', v_n},
    {'h', v_h},
    {'m', v_m},
    {'y', v_y},
    {'r', v_r},
    {'w', v_w},

    {'g', v_g},
    {'z', v_z},
    {'d', v_d},
    {'b', v_b},
    {'p', v_p},

    {'c', v_c},
    {'f', v_f},
    {'j', v_j},
    {'l', v_l},
    {'q', v_q},
    {'v', v_v},
    {'-', v_minus},
};

typedef struct {
    char *hiragana;
    char *katakana;
    // ※ヴなど、ひらがなとカタカナで長さが異なる場合がある
    int hira_len;
    int kata_len;
} kana_t;

typedef int str_value_t;

typedef struct {
    str_value_t sv;
    kana_t kana;
} sv_kana_t;

#define SV_NN ((v_n<<SHIFT)+v_n)
#define SV_LTU ((v_l<<(SHIFT*2))+(v_t<<SHIFT)+v_u)

sv_kana_t sv_kana[] = {
    {v_a,      {"あ", "ア", 0}},
    {v_i,      {"い", "イ", 0}},
    {v_u,      {"う", "ウ", 0}},
    {v_e,      {"え", "エ", 0}},
    {v_o,      {"お", "オ", 0}},
    {(v_k<<SHIFT)+v_a,  {"か", "カ", 0}},
    {(v_k<<SHIFT)+v_i,  {"き", "キ", 0}},
    {(v_k<<SHIFT)+v_u,  {"く", "ク", 0}},
    {(v_k<<SHIFT)+v_e,  {"け", "ケ", 0}},
    {(v_k<<SHIFT)+v_o,  {"こ", "コ", 0}},
    {(v_s<<SHIFT)+v_a,  {"さ", "サ", 0}},
    {(v_s<<SHIFT)+v_i,  {"し", "シ", 0}},
    {(v_s<<SHIFT)+v_u,  {"す", "ス", 0}},
    {(v_s<<SHIFT)+v_e,  {"せ", "セ", 0}},
    {(v_s<<SHIFT)+v_o,  {"そ", "ソ", 0}},
    {(v_t<<SHIFT)+v_a,  {"た", "タ", 0}},
    {(v_t<<SHIFT)+v_i,  {"ち", "チ", 0}},
    {(v_t<<SHIFT)+v_u,  {"つ", "ツ", 0}},
    {(v_t<<SHIFT)+v_e,  {"て", "テ", 0}},
    {(v_t<<SHIFT)+v_o,  {"と", "ト", 0}},
    {(v_n<<SHIFT)+v_a,  {"な", "ナ", 0}},
    {(v_n<<SHIFT)+v_i,  {"に", "ニ", 0}},
    {(v_n<<SHIFT)+v_u,  {"ぬ", "ヌ", 0}},
    {(v_n<<SHIFT)+v_e,  {"ね", "ネ", 0}},
    {(v_n<<SHIFT)+v_o,  {"の", "ノ", 0}},
    {(v_h<<SHIFT)+v_a,  {"は", "ハ", 0}},
    {(v_h<<SHIFT)+v_i,  {"ひ", "ヒ", 0}},
    {(v_h<<SHIFT)+v_u,  {"ふ", "フ", 0}},
    {(v_h<<SHIFT)+v_e,  {"へ", "ヘ", 0}},
    {(v_h<<SHIFT)+v_o,  {"ほ", "ホ", 0}},
    {(v_m<<SHIFT)+v_a,  {"ま", "マ", 0}},
    {(v_m<<SHIFT)+v_i,  {"み", "ミ", 0}},
    {(v_m<<SHIFT)+v_u,  {"む", "ム", 0}},
    {(v_m<<SHIFT)+v_e,  {"め", "メ", 0}},
    {(v_m<<SHIFT)+v_o,  {"も", "モ", 0}},
    {(v_y<<SHIFT)+v_a,  {"や", "ヤ", 0}},
    {(v_y<<SHIFT)+v_u,  {"ゆ", "ユ", 0}},
    {(v_y<<SHIFT)+v_o,  {"よ", "ヨ", 0}},
    {(v_r<<SHIFT)+v_a,  {"ら", "ラ", 0}},
    {(v_r<<SHIFT)+v_i,  {"り", "リ", 0}},
    {(v_r<<SHIFT)+v_u,  {"る", "ル", 0}},
    {(v_r<<SHIFT)+v_e,  {"れ", "レ", 0}},
    {(v_r<<SHIFT)+v_o,  {"ろ", "ロ", 0}},
    {(v_w<<SHIFT)+v_a,  {"わ", "ワ", 0}},
    {(v_w<<SHIFT)+v_i,  {"うぃ", "ウィ", 0}},
    {(v_w<<SHIFT)+v_u,  {"う", "ウィ", 0}},
    {(v_w<<SHIFT)+v_e,  {"うぇ", "ウェ", 0}},
    {(v_w<<SHIFT)+v_o,  {"を", "ヲ", 0}},
    {(v_g<<SHIFT)+v_a,  {"が", "ガ", 0}},
    {(v_g<<SHIFT)+v_i,  {"ぎ", "ギ", 0}},
    {(v_g<<SHIFT)+v_u,  {"ぐ", "グ", 0}},
    {(v_g<<SHIFT)+v_e,  {"げ", "ゲ", 0}},
    {(v_g<<SHIFT)+v_o,  {"ご", "ゴ", 0}},
    {(v_z<<SHIFT)+v_a,  {"ざ", "ザ", 0}},
    {(v_z<<SHIFT)+v_i,  {"じ", "ジ", 0}},
    {(v_z<<SHIFT)+v_u,  {"ず", "ズ", 0}},
    {(v_z<<SHIFT)+v_e,  {"ぜ", "ゼ", 0}},
    {(v_z<<SHIFT)+v_o,  {"ぞ", "ゾ", 0}},
    {(v_d<<SHIFT)+v_a,  {"だ", "ダ", 0}},
    {(v_d<<SHIFT)+v_i,  {"ぢ", "ぢ", 0}},
    {(v_d<<SHIFT)+v_u,  {"づ", "づ", 0}},
    {(v_d<<SHIFT)+v_e,  {"で", "デ", 0}},
    {(v_d<<SHIFT)+v_o,  {"ど", "ド", 0}},
    {(v_b<<SHIFT)+v_a,  {"ば", "バ", 0}},
    {(v_b<<SHIFT)+v_i,  {"び", "ビ", 0}},
    {(v_b<<SHIFT)+v_u,  {"ぶ", "ブ", 0}},
    {(v_b<<SHIFT)+v_e,  {"べ", "ベ", 0}},
    {(v_b<<SHIFT)+v_o,  {"ぼ", "ボ", 0}},
    {(v_p<<SHIFT)+v_a,  {"ぱ", "パ", 0}},
    {(v_p<<SHIFT)+v_i,  {"ぴ", "ピ", 0}},
    {(v_p<<SHIFT)+v_u,  {"ぷ", "プ", 0}},
    {(v_p<<SHIFT)+v_e,  {"ぺ", "ペ", 0}},
    {(v_p<<SHIFT)+v_o,  {"ぽ", "ポ", 0}},

    {SV_NN,  {"ん", "ン", 0}},
    {v_minus,      {"ー", "ー", 0}},

    {(v_f<<SHIFT)+v_a,  {"ふぁ", "ファ", 0}},
    {(v_f<<SHIFT)+v_i,  {"ふぃ", "フィ", 0}},
    {(v_f<<SHIFT)+v_u,  {"ふ", "フ", 0}},
    {(v_f<<SHIFT)+v_e,  {"ふぇ", "フェ", 0}},
    {(v_f<<SHIFT)+v_o,  {"ふぉ", "フォ", 0}},
    {(v_j<<SHIFT)+v_a,  {"じゃ", "ジャ", 0}},
    {(v_j<<SHIFT)+v_i,  {"じ", "ジ", 0}},
    {(v_j<<SHIFT)+v_u,  {"じゅ", "ジュ", 0}},
    {(v_j<<SHIFT)+v_e,  {"じぇ", "ジェ", 0}},
    {(v_j<<SHIFT)+v_o,  {"じょ", "ジョ", 0}},
    {(v_l<<SHIFT)+v_a,  {"ぁ", "ァ", 0}},
    {(v_l<<SHIFT)+v_i,  {"ぃ", "ィ", 0}},
    {(v_l<<SHIFT)+v_u,  {"ぅ", "ゥ", 0}},
    {(v_l<<SHIFT)+v_e,  {"ぇ", "ェ", 0}},
    {(v_l<<SHIFT)+v_o,  {"ぉ", "ォ", 0}},
    {(v_x<<SHIFT)+v_a,  {"ぁ", "ァ", 0}},
    {(v_x<<SHIFT)+v_i,  {"ぃ", "ィ", 0}},
    {(v_x<<SHIFT)+v_u,  {"ぅ", "ゥ", 0}},
    {(v_x<<SHIFT)+v_e,  {"ぇ", "ェ", 0}},
    {(v_x<<SHIFT)+v_o,  {"ぉ", "ォ", 0}},
    {(v_v<<SHIFT)+v_a,  {"う゛ぁ", "ヴァ", 0}},
    {(v_v<<SHIFT)+v_i,  {"う゛ぃ", "ヴィ", 0}},
    {(v_v<<SHIFT)+v_u,  {"う゛", "ヴ", 0}},
    {(v_v<<SHIFT)+v_e,  {"う゛ぇ", "ヴェ", 0}},
    {(v_v<<SHIFT)+v_o,  {"う゛ぉ", "ヴォ", 0}},
    {(v_q<<SHIFT)+v_a,  {"くぁ", "クァ", 0}},
    {(v_q<<SHIFT)+v_i,  {"くぃ", "クィ", 0}},
    {(v_q<<SHIFT)+v_u,  {"く", "ク", 0}},
    {(v_q<<SHIFT)+v_e,  {"くぇ", "クェ", 0}},
    {(v_q<<SHIFT)+v_o,  {"くぉ", "クォ", 0}},

    {(v_b<<(SHIFT*2))+(v_y<<SHIFT)+v_a,  {"びゃ", "ビャ", 0}},
    {(v_b<<(SHIFT*2))+(v_y<<SHIFT)+v_i,  {"びぃ", "ビィ", 0}},
    {(v_b<<(SHIFT*2))+(v_y<<SHIFT)+v_u,  {"びゅ", "ビュ", 0}},
    {(v_b<<(SHIFT*2))+(v_y<<SHIFT)+v_e,  {"びぇ", "ビェ", 0}},
    {(v_b<<(SHIFT*2))+(v_y<<SHIFT)+v_o,  {"びょ", "ビョ", 0}},
    {(v_t<<(SHIFT*2))+(v_h<<SHIFT)+v_a,  {"てゃ", "テャ", 0}},
    {(v_t<<(SHIFT*2))+(v_h<<SHIFT)+v_i,  {"てぃ", "ティ", 0}},
    {(v_t<<(SHIFT*2))+(v_h<<SHIFT)+v_u,  {"てゅ", "テュ", 0}},
    {(v_t<<(SHIFT*2))+(v_h<<SHIFT)+v_e,  {"てぇ", "テェ", 0}},
    {(v_t<<(SHIFT*2))+(v_h<<SHIFT)+v_o,  {"てょ", "テョ", 0}},
    {(v_t<<(SHIFT*2))+(v_w<<SHIFT)+v_u,  {"とぅ", "トゥ", 0}},
    {(v_d<<(SHIFT*2))+(v_h<<SHIFT)+v_a,  {"でゃ", "デャ", 0}},
    {(v_d<<(SHIFT*2))+(v_h<<SHIFT)+v_i,  {"でぃ", "ディ", 0}},
    {(v_d<<(SHIFT*2))+(v_h<<SHIFT)+v_u,  {"でゅ", "デュ", 0}},
    {(v_d<<(SHIFT*2))+(v_h<<SHIFT)+v_e,  {"でぇ", "デェ", 0}},
    {(v_d<<(SHIFT*2))+(v_h<<SHIFT)+v_o,  {"でょ", "デョ", 0}},
    {(v_d<<(SHIFT*2))+(v_w<<SHIFT)+v_u,  {"どぅ", "ドゥ", 0}},
    {(v_c<<(SHIFT*2))+(v_h<<SHIFT)+v_a,  {"ちゃ", "チャ", 0}},
    {(v_c<<(SHIFT*2))+(v_h<<SHIFT)+v_i,  {"ち", "チ", 0}},
    {(v_c<<(SHIFT*2))+(v_h<<SHIFT)+v_u,  {"ちゅ", "チュ", 0}},
    {(v_c<<(SHIFT*2))+(v_h<<SHIFT)+v_e,  {"ちぇ", "チェ", 0}},
    {(v_c<<(SHIFT*2))+(v_h<<SHIFT)+v_o,  {"ちょ", "チョ", 0}},
    {(v_h<<(SHIFT*2))+(v_y<<SHIFT)+v_a,  {"ひゃ", "ヒャ", 0}},
    {(v_h<<(SHIFT*2))+(v_y<<SHIFT)+v_i,  {"ひぃ", "ヒィ", 0}},
    {(v_h<<(SHIFT*2))+(v_y<<SHIFT)+v_u,  {"ひゅ", "ヒュ", 0}},
    {(v_h<<(SHIFT*2))+(v_y<<SHIFT)+v_e,  {"ひぇ", "ヒェ", 0}},
    {(v_h<<(SHIFT*2))+(v_y<<SHIFT)+v_o,  {"ひょ", "ヒョ", 0}},
    {(v_s<<(SHIFT*2))+(v_h<<SHIFT)+v_a,  {"しゃ", "シャ", 0}},
    {(v_s<<(SHIFT*2))+(v_h<<SHIFT)+v_i,  {"し", "シ", 0}},
    {(v_s<<(SHIFT*2))+(v_h<<SHIFT)+v_u,  {"しゅ", "シュ", 0}},
    {(v_s<<(SHIFT*2))+(v_h<<SHIFT)+v_e,  {"しぇ", "シェ", 0}},
    {(v_s<<(SHIFT*2))+(v_h<<SHIFT)+v_o,  {"しょ", "ショ", 0}},
    {(v_s<<(SHIFT*2))+(v_y<<SHIFT)+v_a,  {"しゃ", "シャ", 0}},
    {(v_s<<(SHIFT*2))+(v_y<<SHIFT)+v_i,  {"しぃ", "シィ", 0}},
    {(v_s<<(SHIFT*2))+(v_y<<SHIFT)+v_u,  {"しゅ", "シュ", 0}},
    {(v_s<<(SHIFT*2))+(v_y<<SHIFT)+v_e,  {"しぇ", "シェ", 0}},
    {(v_s<<(SHIFT*2))+(v_y<<SHIFT)+v_o,  {"しょ", "ショ", 0}},
    {(v_k<<(SHIFT*2))+(v_y<<SHIFT)+v_a,  {"きゃ", "キャ", 0}},
    {(v_k<<(SHIFT*2))+(v_y<<SHIFT)+v_i,  {"きぃ", "キィ", 0}},
    {(v_k<<(SHIFT*2))+(v_y<<SHIFT)+v_u,  {"きゅ", "キュ", 0}},
    {(v_k<<(SHIFT*2))+(v_y<<SHIFT)+v_e,  {"きぇ", "キェ", 0}},
    {(v_k<<(SHIFT*2))+(v_y<<SHIFT)+v_o,  {"きょ", "キョ", 0}},
    {(v_g<<(SHIFT*2))+(v_y<<SHIFT)+v_a,  {"ぎゃ", "ギャ", 0}},
    {(v_g<<(SHIFT*2))+(v_y<<SHIFT)+v_i,  {"ぎぃ", "ギィ", 0}},
    {(v_g<<(SHIFT*2))+(v_y<<SHIFT)+v_u,  {"ぎゅ", "ギュ", 0}},
    {(v_g<<(SHIFT*2))+(v_y<<SHIFT)+v_e,  {"ぎぇ", "ギェ", 0}},
    {(v_g<<(SHIFT*2))+(v_y<<SHIFT)+v_o,  {"ぎょ", "ギョ", 0}},

    {(v_m<<(SHIFT*2))+(v_y<<SHIFT)+v_a,  {"みゃ", "ミャ", 0}},
    {(v_m<<(SHIFT*2))+(v_y<<SHIFT)+v_i,  {"みぃ", "ミィ", 0}},
    {(v_m<<(SHIFT*2))+(v_y<<SHIFT)+v_u,  {"みゅ", "ミュ", 0}},
    {(v_m<<(SHIFT*2))+(v_y<<SHIFT)+v_e,  {"みぇ", "ミェ", 0}},
    {(v_m<<(SHIFT*2))+(v_y<<SHIFT)+v_o,  {"みょ", "ミョ", 0}},
    {(v_n<<(SHIFT*2))+(v_y<<SHIFT)+v_a,  {"にゃ", "ニャ", 0}},
    {(v_n<<(SHIFT*2))+(v_y<<SHIFT)+v_i,  {"にぃ", "ニィ", 0}},
    {(v_n<<(SHIFT*2))+(v_y<<SHIFT)+v_u,  {"にゅ", "ニュ", 0}},
    {(v_n<<(SHIFT*2))+(v_y<<SHIFT)+v_e,  {"にぇ", "ニェ", 0}},
    {(v_n<<(SHIFT*2))+(v_y<<SHIFT)+v_o,  {"にょ", "ニョ", 0}},

    {(v_r<<(SHIFT*2))+(v_y<<SHIFT)+v_a,  {"りゃ", "リャ", 0}},
    {(v_r<<(SHIFT*2))+(v_y<<SHIFT)+v_i,  {"りぃ", "リィ", 0}},
    {(v_r<<(SHIFT*2))+(v_y<<SHIFT)+v_u,  {"りゅ", "リュ", 0}},
    {(v_r<<(SHIFT*2))+(v_y<<SHIFT)+v_e,  {"りぇ", "リェ", 0}},
    {(v_r<<(SHIFT*2))+(v_y<<SHIFT)+v_o,  {"りょ", "リョ", 0}},

    {(v_q<<(SHIFT*2))+(v_w<<SHIFT)+v_a,  {"くぁ", "クァ", 0}},
    {(v_q<<(SHIFT*2))+(v_w<<SHIFT)+v_i,  {"くぃ", "クィ", 0}},
    {(v_q<<(SHIFT*2))+(v_w<<SHIFT)+v_u,  {"くぅ", "クゥ", 0}},
    {(v_q<<(SHIFT*2))+(v_w<<SHIFT)+v_e,  {"くぇ", "クェ", 0}},
    {(v_q<<(SHIFT*2))+(v_w<<SHIFT)+v_o,  {"くぉ", "クォ", 0}},

    {(v_t<<(SHIFT*2))+(v_s<<SHIFT)+v_a,  {"つぁ", "ツァ", 0}},
    {(v_t<<(SHIFT*2))+(v_s<<SHIFT)+v_i,  {"つぃ", "ツィ", 0}},
    {(v_t<<(SHIFT*2))+(v_s<<SHIFT)+v_u,  {"つ",   "ツ",   0}},
    {(v_t<<(SHIFT*2))+(v_s<<SHIFT)+v_e,  {"つぇ", "ツェ", 0}},
    {(v_t<<(SHIFT*2))+(v_s<<SHIFT)+v_o,  {"つぉ", "ツォ", 0}},

    {(v_l<<(SHIFT*2))+(v_y<<SHIFT)+v_a,  {"ゃ", "ャ", 0}},
    {(v_l<<(SHIFT*2))+(v_y<<SHIFT)+v_u,  {"ゅ", "ュ", 0}},
    {(v_l<<(SHIFT*2))+(v_y<<SHIFT)+v_o,  {"ょ", "ョ", 0}},
    {SV_LTU,  {"っ", "ッ", 0}},
    //    {(v_l<<(SHIFT*2))+(v_t<<SHIFT)+v_u,  {"っ", "ッ", 0}},
    {(v_l<<(SHIFT*3))+(v_t<<(SHIFT*2))+(v_s<<SHIFT)+v_u,  {"っ", "ッ", 0}},
};

static char_value_t c2cv[256];
static kana_t *sv2kana[STR_VALUE_MAX+1];

static roman_conv_mode_t kanamode = rc_mode_hiragana;

void put_kana(char **kstr, kana_t *kana);
static char is_initialized = 0;


int roman_conv_init(void){
	int i;

    if (is_initialized) return 1;

    for (i = 0 ; i < sizeof(c_cv)/sizeof(c_cv_t) ; i++){
        assert(c2cv[c_cv[i].c] == 0);
        //                printf("c_cv %c %d\n", c_cv[i].c, c_cv[i].cv);
        c2cv[c_cv[i].c] = c_cv[i].cv;
    }
    for (i = 0 ; i < sizeof(sv_kana)/sizeof(sv_kana_t) ; i++){
        //        printf("sv_k %d %s\n", sv_kana[i].sv, sv_kana[i].kana.hiragana);
        sv_kana[i].kana.hira_len = strlen(sv_kana[i].kana.hiragana);
        sv_kana[i].kana.kata_len = strlen(sv_kana[i].kana.katakana);
        sv2kana[sv_kana[i].sv] = &sv_kana[i].kana;
    }
    is_initialized = 1;
    return 0;
}

void roman_conv_set_mode(roman_conv_mode_t mode){
    assert(mode == rc_mode_hiragana ||
           mode == rc_mode_katakana);
    kanamode = mode;
}

// rstrの先頭からローマ字変換を行った結果の文字列をkstrに格納する
// 完全に変換できなかったときは、残ったアルファベット(1バイト文字)部分の
// 先頭位置をleftstrに格納する
// 完全に変換できたときは0, それ以外なら非0を返す
// terminatedが非0のときは、入力が完了している(たとえば末尾がnなら「ん」にする)
// terminatedが0のときは、入力が不完全(たとえば末尾がnなら「ん」に変換しない)
int roman2kana(char* rstr, char *kstr, char **leftstr, int terminated){
    int cv, cv_next;
    int sv = 0;
	kana_t *kana;

    *kstr = '\0';

    while (*rstr != '\0'){
        cv = c2cv[(unsigned char)*rstr];
        if (cv == v_invalid) break;

        cv_next = c2cv[(unsigned char)*(rstr+1)];
        
        sv = (sv << SHIFT) + cv;
        // この時点で'nn'になっておらず、'n'の後続文字が'n'以外の子音なら、
        // ここで'nn'を読んだことにして'ん'に変換
        if (cv == v_n && sv != SV_NN){
            if ((cv_next != v_n && cv_next != v_y && !IsVowel(cv_next)) ||
                (cv_next == v_invalid && terminated != 0))
            {
                sv = (sv << SHIFT) + cv;
            }
        }
        // nnの場合を除き、子音が連続したら
        // 最初の子音(今回のcv)を'っ'に変換
        else if (cv == cv_next && !IsVowel(cv) && sv != SV_NN){ 
            sv = SV_LTU;
        }
        if (sv > STR_VALUE_MAX) return -1;
        kana = sv2kana[sv];
        rstr ++;
        if (kana != NULL){
            put_kana(&kstr, kana);
            sv = 0;
            *leftstr = rstr;
        }
    }
    if (*leftstr == rstr){
        return 0;
    }
    else {
        return -1;
    }
}

void put_kana(char **kstr, kana_t *kana){
    assert(kana != NULL);

    if (kanamode == rc_mode_hiragana){
        strcpy(*kstr, kana->hiragana);
        *kstr += kana->hira_len;
    }
    else {
        strcpy(*kstr, kana->katakana);
        *kstr += kana->kata_len;
    }
    
}
