#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "config.h"
#include "status.h"
#include "noun.h"
#include "message.h"
#include "graphic.h"
#include "file.h"
#include "term.h"

static status_t status;
static status_t achievements_status;

int flag_sequence_dragon_callback(flag_id_t new_flag, int time_passed);
int flag_sequence_shadow_callback(flag_id_t new_flag, int time_passed);
int flag_sequence_armor_to_guestroom_callback(flag_id_t new_flag,
                                              int time_passed);
int flag_sequence_armor_from_guestroom_callback(flag_id_t new_flag,
                                                int time_passed);
int flag_sequence_armor_to_library_callback(flag_id_t new_flag,
                                            int time_passed);
int flag_sequence_armor_from_library_callback(flag_id_t new_flag,
                                              int time_passed);

flag_sequence_t flag_sequence_dragon = {
    {
        //fi_dragon_library,
        fi_dragon_moved_corridor_2f_r,
        fi_dragon_moved_entrance_hall,
        fi_dragon_moved_guestroom_1f,
        fi_dragon_in_guestroom_1f,
        fi_dragon_shotdown,
        fi_invalid
    },
    0, 1, 0, flag_sequence_dragon_callback
};

flag_sequence_t flag_sequence_shadow = {
    {
        //        fi_start_shadow,
        fi_shadow_appeared,
        fi_shadow_behind_door,
        fi_shadow_in_room,
        fi_shadow_attack,
        fi_shadow_wandering,
        fi_shadow_gone,
        fi_invalid
    },
    0, 1, 1, flag_sequence_shadow_callback
};

flag_sequence_t flag_sequence_armor_to_guestroom = {
    {
        //        fi_start_armor_to_guestroom,
        fi_armor_left_study_to_guestroom,
        fi_armor_moving_to_guestroom,
        fi_armor_in_guestroom,
        fi_armor_restoring_guestroom,
        fi_invalid

    },
    0, 1, 1, flag_sequence_armor_to_guestroom_callback
};

flag_sequence_t flag_sequence_armor_from_guestroom = {
    {
        // to_guestroomのrestoring_guestroomでstartすると
        // そのターンでfromの1番目のフェーズも実行される。
        // 同じターンでmoving_fromに進まないように、ここにもrestoringを入れておく
        fi_armor_restoring_guestroom,
        fi_armor_moving_from_guestroom,
        fi_armor_returning_study,
        fi_invalid
    },
    0, 1, 1, flag_sequence_armor_from_guestroom_callback
};

flag_sequence_t flag_sequence_armor_to_library = {
    {
        fi_armor_in_library,
        fi_armor_restoring_library,
        fi_armor_crushed,
        fi_armor_reviving,
        fi_armor_revived,
        fi_armor_dead,
        fi_invalid

    },
    0, 1, 1, flag_sequence_armor_to_library_callback
};

flag_sequence_t flag_sequence_armor_from_library = {
    {
        // to_libraryのrestoring_guestroomでstartすると
        // そのターンでfromの1番目のフェーズも実行される。
        // 同じターンでmoving_fromに進まないように、ここにもrestoringを入れておく
        fi_armor_restoring_library,
        fi_armor_returning_study,
        fi_invalid

    },
    0, 1, 1, flag_sequence_armor_from_library_callback
};

flag_sequence_t *flag_sequence_table[] = {
    &flag_sequence_dragon,
    &flag_sequence_shadow,
    &flag_sequence_armor_to_guestroom,
    &flag_sequence_armor_to_library,
    &flag_sequence_armor_from_guestroom,
    &flag_sequence_armor_from_library,
};

#define FLAG_SEQUENCE_NUM (sizeof(flag_sequence_table)/sizeof(flag_sequence_t*))

point_t normal_points[] = {
    {fi_leaflet_taken, 1, 1},
    {fi_entered_house, 1, 5},
    {fi_bathtub_empty, 1, 5},
    {fi_mirror_taken, 1, 5},
    {fi_glasses_taken, 1, 5},
};
point_t maniac_points[] = {
    {fi_medal_taken,     1, 1},
    {fi_mailbox_has_closed,     1, 1},
    {fi_toilet_used, 1, 1},
    {fi_phone_call_answered, 1, 1},
};

#define MANIAC_POINT_NUM (sizeof(maniac_points)/sizeof(point_t))
#define NORMAL_POINT_NUM (sizeof(normal_points)/sizeof(point_t))

void status_init(void){
    int i, j;

    // ゲームフラグ数が予約数を超えていないかチェック
    assert(fi_end_regular_flags < fi_begin_door_flags);
    assert(fi_end_door_flags < fi_begin_discovered_flags);

    assert(fi_end_game_flags < fi_begin_clear_flags);

    // 各フラグシーケンスのフラグ数を求めてnumにセットする
    for (i = 0 ; i < FLAG_SEQUENCE_NUM ; i++){
        for (j = 0 ; j < FLAG_SEQUENCE_MAXLEN ; j++){
            //            printf("%d %d %d\n", i, j, flag_sequence_table[i]->flags[j]);
            if (flag_sequence_table[i]->flags[j] == fi_invalid){
                flag_sequence_table[i]->num = j;
                break;
            }
        }
        assert(j < FLAG_SEQUENCE_MAXNUM);
    }
 }


void clear_status(void){
    int i;
    flag_sequence_id_t fs;

    status.current_area = ai_title;
    status.item_num = 0;
    for (fs = fs_begin ; fs < fs_end ; fs++){
        clear_flag_sequence(fs);
    }
    for (i = 0 ; i < FLAG_CHAR_NUM ; i++){
        status.flags[i] = 0;
    }
}

void init_achievements_status(void){
    int i;
    for (i = 0 ; i < FLAG_CHAR_NUM ; i++){
        achievements_status.flags[i] = 0;
    }
}

area_id_t current_area(void){
    return status.current_area;
}

void set_area(area_id_t id){
    status.current_area = id;
}

int is_flag_on(flag_id_t fid){
    return (((status.flags[fid/8] >> fid%8) & 1) == 1);
}

int is_flag_off(flag_id_t fid){
    return (((status.flags[fid/8] >> fid%8) & 1) == 0);
}

void set_flag(flag_id_t fid){
    status.flags[fid/8] |= 1 << fid%8;
    //    draw_graphic();
}
void reset_flag(flag_id_t fid){
	if (is_flag_on(fid)){
        status.flags[fid/8] -=  1 << fid%8;
	}
    //    draw_graphic();
}
void toggle_flag(flag_id_t fid){
	if (is_flag_on(fid)){
		reset_flag(fid);
	}
	else {
		set_flag(fid);
	}
}

void test_and_set_flag(flag_id_t fid, char *msg_success, char *msg_failure){
    if (is_flag_off(fid)){
        set_flag(fid);
        if (msg_success){
            message(msg_success);
        }
    }
    else {
        if (msg_failure){
            message(msg_failure);
        }
    }
}
void test_and_reset_flag(flag_id_t fid, char *msg_success, char *msg_failure){
    if (is_flag_on(fid)){
        reset_flag(fid);
        message(msg_success);
    }
    else {
        message(msg_failure);
    }
}

int count_on_flags(int num, ...){
    int i, count = 0;
    va_list ap;
    flag_id_t fid;

    va_start(ap, num);
    for (i = 0 ; i < num ; i++){
        fid = va_arg(ap, flag_id_t);
        if (is_flag_on(fid)) count ++;
    }
    va_end(ap);
    return count;
}


int has_item(noun_id_t nid){
	int i;

	for (i = 0 ; i < status.item_num ; i++){
		if (status.items[i] == nid) return 1;
	}
    return 0;
}

void get_item(noun_id_t nid){
	status.items[status.item_num++] = nid;
    draw_graphic();
}
void lose_item(noun_id_t nid){
	int i, j;

	for (i = 0 ; i < status.item_num ; i++){
		if (status.items[i] == nid){
			for (j = i ; j < status.item_num-1 ; j++){
				status.items[j] = status.items[j+1];
			}
			status.item_num --;
			return;
		}
	}
}

int get_item_num(void){
	return status.item_num;
}
noun_id_t get_item_id(int ix){
	return status.items[ix];
}

void show_score(char *msg, point_t points[], int point_num){
    int i, score = 0, total_score = 0;
    
    for (i = 0 ; i < point_num ; i++){
        total_score += points[i].score;
        if (status.flags[points[i].id] == points[i].value){
            score += points[i].score;
        }
    }
    message("%s:%d/%d", msg, score, total_score);
}
void show_normal_score(void){
    show_score("現在の得点", normal_points, NORMAL_POINT_NUM);
}
#ifdef DUMMY
void show_maniac_score(void){
    show_score("マニア評価", maniac_points, MANIAC_POINT_NUM);
}
#endif

int load_data(void){
    return load_status(SAVE_FILE, &status);
}
int save_data(void){
    return save_status(SAVE_FILE, &status);
}
// achievementsファイルを読み込んでクリアフラグ部のみstatusにコピーする
int load_achievements_flags(void){
    int i, ret;

    ret = load_achievements();
    printf("copy achievements\n");
    for (i = fi_begin_clear_flags/8 ; i < FLAG_CHAR_NUM ; i++){
        printf("copy %d:%d->%d\n", i, status.flags[i], achievements_status.flags[i]);
        status.flags[i] = achievements_status.flags[i];
    }
    return ret;
}
int load_achievements(void){
    if (load_status(ACHIEVEMENTS_FILE, &achievements_status)){
        log_warn("Cannot read achievements file %s.\n",
                 ACHIEVEMENTS_FILE);
        init_achievements_status();
        return -1;
    }
    return 0;
}
int save_achievements(void){
    return save_status(ACHIEVEMENTS_FILE, &achievements_status);
}
// 今回のプレイのフラグをachievementのフラグにマージ(OR)
void merge_to_achievement_flags(void){
    int i;
    for (i = 0 ; i < FLAG_CHAR_NUM ; i++){
        achievements_status.flags[i] |= status.flags[i];
    }
}
// 達成度フラグを設定する
void set_achievement_flags(void){
    // 死因フラグは死亡時に設定済
    // クリア時のみ設定するフラグ
    if (Not(dead)){
        if (Is(armor_dead)){
            Set(clear_armor_dead);
        }
        else {
            Set(clear_armor_alive);
        }
        if (Is(treasure_chest_unlocked)){
            Set(clear_treasure_chest_unlocked);
        }
        else {
            Set(clear_treasure_chest_locked);
        }
        if (Is(portrait_has_removed)){
            Set(clear_portrait_disabled);
        }
        else {
            Set(clear_portrait_enabled);
        }
        if (Is(human_bone_put)){
            Set(clear_using_human_bone);
        }
        else {
            Set(clear_using_chicken_bone);
        }
        if (Is(phone_call_answered)){
            Set(clear_phone_call_answered);
        }
        else {
            Set(clear_phone_call_ignored);
        }
    }
}
int update_achievements(void){
    load_achievements();
    set_achievement_flags();
    merge_to_achievement_flags();
    return save_achievements();
}
int load_status(char *filename, status_t *status){
    FILE *fp;
    unsigned char file_major_version = 0;
    unsigned char file_minor_version = 0;
    int current_version = GetVersion(MAJOR_VERSION, MINOR_VERSION);
	int file_version;
    //	unsigned char id[APPLICATION_ID_SIZE];

    if ((fp = open_data(filename, "rb", APPLICATION_ID, APPLICATION_ID_SIZE,
                        &file_major_version, &file_minor_version)) == NULL)
    {
        return -1;
    }
    // 未知のバージョンなら読み込み中止
	file_version = GetVersion(file_major_version, file_minor_version);
    if (file_version < current_version || file_version > current_version){
        log_warn("Unknown file version %d.%d\n", 
                 file_major_version, file_minor_version);
        return -1;
    }
    fread(status, sizeof(status_t), 1, fp);
    fclose(fp);
    return 0;
}

int save_status(char *filename, status_t *status){
    FILE *fp;
    unsigned char file_major_version = MAJOR_VERSION;
    unsigned char file_minor_version = MINOR_VERSION;

    if ((fp = open_data(filename, "wb", APPLICATION_ID, APPLICATION_ID_SIZE,
                        &file_major_version, &file_minor_version)) == NULL)
    {
        return -1;
    }
    fwrite(status, sizeof(status_t), 1, fp);
    fclose(fp);
    return 0;
}
#ifdef DUMMY
void set_config_input(input_mode_t mode){
    input_mode = mode;
    set_term_input_mode(mode);
}
input_mode_t get_config_input(void){
    return input_mode;
}
#endif
// 指定したフラグシーケンスの系列フラグをすべて解除
void clear_flag_sequence(flag_sequence_id_t fs){
    int i;
    flag_sequence_t *fseq = flag_sequence_table[fs];
    flag_sequence_status_t *stat = &status.flag_sequence[fs];

    for (i = 0 ; i < fseq->num ; i++){
        reset_flag(fseq->flags[i]);
    }
    stat->phase = 0;
    stat->bits = 0;
}    
// 指定したフラグシーケンスを開始する。
// 同シーケンスが進行中でも初期化され、先頭のフラグのみ立つ
void start_flag_sequence(flag_sequence_id_t fs){
    flag_sequence_t *fseq = flag_sequence_table[fs];

    // 現在の進行状況に関わりなく系列フラグをいったんクリア
    clear_flag_sequence(fs);
    set_flag_sequence_active(fs);
    // 最初のフラグをON
    // auto系列の場合、process_flag_sequences()で進めてからコールバックされるので、
    // この時点ではフラグを立てない
    if (!fseq->is_auto){
        set_flag(fseq->flags[0]);
    }
    printf("started flag sequence %d\n", fs);
}
// フラグシーケンスを進行させる
void proceed_flag_sequence(flag_sequence_id_t fs){
    flag_sequence_t *fseq = flag_sequence_table[fs];
    flag_sequence_status_t *stat = &status.flag_sequence[fs];

    printf("%d %d %d\n", fs, stat->phase, fseq->num);
    assert (stat->phase < fseq->num);
    if (fseq->is_exclusive){
        reset_flag(fseq->flags[stat->phase]);
    }
    stat->phase ++;
    set_flag(fseq->flags[stat->phase]);
    printf("proceed flag sequence %d (%d->%d\n",
           fs, stat->phase-1,stat->phase);
    if (stat->phase == fseq->num-1){
        set_flag_sequence_complete(fs);
        reset_flag_sequence_active(fs);
        printf("complete.\n");
    }
}
// 指定したフラグまでフラグ系列を進める
// @note fsがアクティブでなければならず、fidがfsに含まれていなければならない
int jump_flag_sequence(flag_sequence_id_t fs, flag_id_t fid){
    flag_sequence_t *fseq = flag_sequence_table[fs];
    flag_sequence_status_t *stat = &status.flag_sequence[fs];
    int i, fid_pos = -1;
    

    assert(is_flag_sequence_active(fs));
    for (i = 0 ; i < fseq->num ; i++){
        if (fseq->flags[i] == fid){
            /*
            // autoの場合は次回のコールバック前に一つ進むので、一つ前にしておく
            if (fseq->is_auto){
                set_flag(fseq->flags[i-1]);
                stat->phase = i-1;
                fid_pos = i-1;
            }
            else {
            */
                set_flag(fseq->flags[i]);
                stat->phase = i;
                fid_pos = i;
                //            }
        }
        else {
            // exclusiveでないなら、ジャンプ先より前のフラグはすべて立てる
            if (!fseq->is_exclusive && fid_pos == -1){
                set_flag(fseq->flags[i]);
            }
            else {
                reset_flag(fseq->flags[i]);
            }
        }
    }
    assert(fid_pos >= 0);
    return fid_pos;
}

void abort_flag_sequence(flag_sequence_id_t fs){
    clear_flag_sequence(fs);
    reset_flag_sequence_active(fs);
    printf("end flag sequence %d\n", fs);
}

void process_flag_sequences(int time_passed){
    int fs;
    flag_sequence_t *fseq;
    flag_sequence_status_t *stat;

    for (fs = 0 ; fs < FLAG_SEQUENCE_NUM ; fs++){
        //        if (is_flag_sequence_active(fs) && !is_flag_sequence_complete(fs)){
        // アクティブなフラグ系列のみ処理
        if (is_flag_sequence_active(fs)){
            fseq = flag_sequence_table[fs];
            stat = &status.flag_sequence[fs];
            // auto系列かつ時間が経過している場合のみ
            if (fseq->is_auto && time_passed){
                printf("proceed auto flag sequence %d.\n", fs);
                // auto系列の場合、start時点ではフラグを立っていないので
                // ここで立てることで、
                // すぐ後のコールバックは初回で系列先頭のフラグが渡る
                if (stat->phase == 0 && is_flag_off(fseq->flags[0])){
                    set_flag(fseq->flags[0]);
                }
                else {
                    proceed_flag_sequence(fs);
                }
            }
            if (fseq->callback != NULL){
                printf("call flag_sequence_callback.\n");
                fseq->callback(fseq->flags[stat->phase], time_passed);
            }
        }
    }
}


/*
void proceed_auto_flag_sequences(void){
    int fs;
    for (fs = 0 ; fs < FLAG_SEQUENCE_NUM ; fs++){
        //        if (is_flag_sequence_active(fs) && !is_flag_sequence_complete(fs)){
        if (is_flag_sequence_active(fs)){
        }
    }
}
void callback_flag_sequences(int time_passed){
    int fs;
    for (fs = 0 ; fs < FLAG_SEQUENCE_NUM ; fs++){
        if (is_flag_sequence_active(fs)){
        }
    }
}
*/

#define SetFlagSequenceStatusBit(fs, bit) \
    ((&status.flag_sequence[(fs)])->bits |= 1 << (bit))
#define ResetFlagSequenceStatusBit(fs, bit) \
    ((&status.flag_sequence[(fs)])->bits &= 0xffff - (1 << (bit)))
#define IsFlagSequenceStatusBit(fs, bit) \
    ((&status.flag_sequence[(fs)])->bits & (1 << (bit)))

void set_flag_sequence_active(flag_sequence_id_t fs){
    SetFlagSequenceStatusBit(fs, FLAG_SEQUENCE_BIT_ACTIVE);
    //    flag_sequence_status_t *stat = &status.flag_sequence[fs];
    //    stat->bits |= 1 << FLAG_SEQUENCE_BIT_ACTIVE;
}    
void reset_flag_sequence_active(flag_sequence_id_t fs){
    ResetFlagSequenceStatusBit(fs, FLAG_SEQUENCE_BIT_ACTIVE);
    //    flag_sequence_status_t *stat = &status.flag_sequence[fs];
    //    stat->bits &= 0xffff - (1 << FLAG_SEQUENCE_BIT_ACTIVE);
}    
int is_flag_sequence_active(flag_sequence_id_t fs){
    return IsFlagSequenceStatusBit(fs, FLAG_SEQUENCE_BIT_ACTIVE);
    //    flag_sequence_status_t *stat = &status.flag_sequence[fs];
    //    return stat->bits & (1 << FLAG_SEQUENCE_BIT_ACTIVE);
}
void set_flag_sequence_complete(flag_sequence_id_t fs){
    SetFlagSequenceStatusBit(fs, FLAG_SEQUENCE_BIT_COMPLETE);
    //    flag_sequence_status_t *stat = &status.flag_sequence[fs];
    //    stat->bits |= 1 << FLAG_SEQUENCE_BIT_COMPLETE;
}    
int is_flag_sequence_complete(flag_sequence_id_t fs){
    return IsFlagSequenceStatusBit(fs, FLAG_SEQUENCE_BIT_COMPLETE);
    //    flag_sequence_status_t *stat = &status.flag_sequence[fs];
    //    return stat->bits & (1 << FLAG_SEQUENCE_BIT_COMPLETE);
}

/*
void set_flag_sequence_auto(flag_sequence_id_t fs){
    SetFlagSequenceStatusBit(fs, FLAG_SEQUENCE_BIT_AUTO);
}
void reset_flag_sequence_auto(flag_sequence_id_t fs){
    ResetFlagSequenceStatusBit(fs, FLAG_SEQUENCE_BIT_AUTO);
}
*/

int has_light(void){
    if ((Has(candle) && Is(candle_lit)) ||
        (Has(lantern) && Is(lantern_lit)))
    {
        return 1;
    }
    else {
        return 0;
    }
}


int flag_sequence_dragon_callback(flag_id_t new_flag, int time_passed){

    printf("DRAGON CALLBACK\n");
    switch(new_flag) {
    case fi_dragon_moved_corridor_2f_r:
        //        if (IsNewArea(corridor_2f_r) || IsNewArea(corridor_2f_cr)){
        // 書斎経由で廊下に出る可能性もあるので両方チェック
        if (MovingFrom(library) || MovingFrom(study)){
            message("廊下に出ると、何か小さな影が階段の方へ飛び去った。"
                    "それはそのまま階下へと姿を消した。");
            proceed_flag_sequence(fs_dragon);
        }
        break;
    case fi_dragon_moved_entrance_hall:
        //        if (IsNewArea(entrance_hall)){
        if (MovingTo(entrance_hall)){
            message("1階に下りると、先ほどの小さな影が飛び去るのが見えた。");
            proceed_flag_sequence(fs_dragon);
        }
        break;
    case fi_dragon_moved_guestroom_1f:
        //        if (IsNewArea(guestroom_1f)){
        if (MovingTo(guestroom_1f)){
            message("天井近くに、先ほどの鳥のようなものが浮いているのを見つけた。");
            proceed_flag_sequence(fs_dragon);
        }
        break;
    case fi_dragon_in_guestroom_1f:
        break;
    case fi_dragon_shotdown:
        break;
    default:
        assert(0);
    }
    return 0;
}


int flag_sequence_shadow_callback(flag_id_t new_flag, int time_passed){

    if (!time_passed) return 0;

    switch(new_flag) {
    case fi_shadow_appeared:
        msg_shadow_appeared();
        break;
    case fi_shadow_behind_door:
        //        if (IsCurrentArea(ladys_room)){
        //        if (IsCurrentArea(ladys_room) && (IsNewArea(invalid) || IsNewArea(wardrobe))){
        if (Staying(ladys_room) || MovingTo(wardrobe)){
            msg_shadow_behind_door();
        }
        else {
            msg_shadow_approaching();
        }
        break;
    case fi_shadow_in_room:
        // 婦人室にとどまっている場合のみ。入れ違いに出ようとしたときは攻撃される(下のelse節)
        //        if (IsCurrentArea(ladys_room) && IsNewArea(invalid)){
        if (Staying(ladys_room)){
            msg_shadow_in_room();
            Set(saw_shadow);
        }
        //        else if (IsCurrentArea(wardrobe) || IsNewArea(wardrobe)){
        else if (IsNextArea(wardrobe)){
            if (Is(wardrobe_opened)){
                msg_shadow_in_room();
                Set(saw_shadow);
            }
            else {
                // 洋服箪笥の中で扉を閉めていたら、影の姿は見えない
                msg_shadow_in_room_hiding();
            }
        } 
        else {
            jump_flag_sequence(fs_shadow, fi_shadow_attack);
            flag_sequence_shadow_callback(fi_shadow_attack, time_passed);
        }
        break;
    case fi_shadow_attack:
        // 攻撃されたとき、洋服箪笥の中で扉を閉めていたら助かる
        //        if (IsCurrentArea(wardrobe) && Not(wardrobe_opened) && Not(saw_shadow)){
        if (Staying(wardrobe) && Not(wardrobe_opened) && Not(saw_shadow)){
            jump_flag_sequence(fs_shadow, fi_shadow_wandering);
            flag_sequence_shadow_callback(fi_shadow_wandering, time_passed);
        }
        else {
            die_touching_shadow();
        }
        break;
    case fi_shadow_wandering:
        // うろついている間に扉を開けると捕まる
        if (Is(wardrobe_opened)){
            die_touching_shadow();
        }
        else {
            msg_shadow_wandering();
        }
        break;
    case fi_shadow_gone:
        // うろついている間に扉を開けると捕まる
        if (Is(wardrobe_opened)){
            die_touching_shadow();
        }
        else {
            msg_shadow_gone();
			Reset(save_forbidden);
        }
        break;
    default:
        assert(0);
        
    }
    return 0;
}


int flag_sequence_armor_to_guestroom_callback(flag_id_t new_flag, int time_passed){

    if (!time_passed) return 0;

    switch(new_flag) {
    case fi_armor_left_study_to_guestroom:
        msg_far_door_opened();
        Set(armor_moving);
        break;
    case fi_armor_moving_to_guestroom:
        if (Staying(guestroom_2f)){
            msg_armor_moving_sound();
        }
        else {
            msg_armor_moving();
        }
        break;
    case fi_armor_in_guestroom:
        if (Staying(guestroom_2f)){
            msg_armor_appeared();
        }
        else if (MovingFrom(guestroom_2f)){
            msg_armor_appeared();
            msg_armor_into_guestroom();
        }
        else {
            msg_armor_into_guestroom();
        }
        break;
    case fi_armor_restoring_guestroom:
        if (Staying(guestroom_2f)){
            die_killed_by_armor();
        }
        else {
            clear_flag_sequence(fs_armor_to_guestroom);        
            start_flag_sequence(fs_armor_from_guestroom);        
        }
        break;
    default:
        assert(0);
        
    }
    return 0;
}
        
int flag_sequence_armor_from_guestroom_callback(flag_id_t new_flag, int time_passed){
    if (!time_passed) return 0;

    switch(new_flag) {
    case fi_armor_restoring_guestroom:
        Reset(guestroom_2f_chair_moved);
        Reset(guestroom_2f_book_moved);
        Reset(guestroom_2f_table_moved);
        Reset(guestroom_2f_cabinet_opened);
        //        Reset(start_armor_to_guestroom);
        //        Reset(armor_to_guestroom);
        //        Reset(armor_in_guestroom);
        break;
    case fi_armor_moving_from_guestroom:
        if (IsNextArea(guestroom_2f) ||
            IsNextArea(corridor_2f_l) ||
            IsNextArea(corridor_2f_c))
        {
            message("鎧が客室を出て、書斎の方へ戻っていった。");
        }
        else if (IsNextArea(corridor_2f_cr) ||
                 IsNextArea(corridor_2f_r))
        {
            message("鎧が客室を出て、書斎の方へ戻ってきた。");
        }
        break;
    case fi_armor_returning_study:
        if (IsNextArea(study)){
            // 先に書斎について悪さをしていたら座らずにすぐ襲う
            if (Is(start_armor_to_study)){
                message("扉が開いて鎧が書斎に入ってくると、"
                        "あなたに向かってきた!");
                Set(armor_in_study);
                Reset(start_armor_to_study);
            }
            else {
                message("扉が開いて鎧が書斎に入ってくると、"
                        "元の場所に座った。");
                Reset(armor_moving);
            }
        }
        else {
            // 廊下にいるときだけ戻るのを目撃する
            if (IsNextArea(corridor_2f_l) ||
                IsNextArea(corridor_2f_c) ||
                IsNextArea(corridor_2f_cr) ||
                IsNextArea(corridor_2f_r))
            {
                message("鎧が書斎に入っていった。");
            }
            Reset(armor_moving);
        }
        break;
    default:
        assert(0);
        
    }

    return 0;
}


int flag_sequence_armor_to_library_callback(flag_id_t new_flag, int time_passed){

    if (!time_passed) return 0;

    switch(new_flag) {
    case fi_armor_in_library:
        message("書斎に座っていた鎧が動き出し、こちらにやってきた！");
        Set(armor_moving);
        if (Is(fossil_set)){
            //            Set(armor_crushed);
            jump_flag_sequence(fs_armor_to_library, fi_armor_crushed);
            flag_sequence_armor_to_library_callback(fi_armor_crushed,
                                                    time_passed);
        }
        break;
    case fi_armor_restoring_library:
        if (Staying(library)){
            die_killed_by_armor();
        }
        else {
            clear_flag_sequence(fs_armor_to_library);        
            start_flag_sequence(fs_armor_from_library);
        }
        break;
    case fi_armor_crushed:
        if (IsNextArea(library) || MovingFrom(library)){
            message("しかし床上の化石に蹴躓き、"
                    "がしゃんと大きな音を立てて転んだ。");
        }
        break;
    case fi_armor_reviving:
        if (IsNextArea(library) || MovingFrom(library)){
            message("床に倒れている鎧の手足が動きだし、"
                    "立ち上がろうともがき始めた。");
        }
        //        Reset(armor_crushed);
        //        Reset(armor_reviving);
        //        Set(armor_in_library);
        break;
    case fi_armor_revived:
        if (IsNextArea(library) || MovingFrom(library)){
            message("床に倒れていた鎧は立ち上がった！");
        }
        jump_flag_sequence(fs_armor_to_library, fi_armor_in_library);
        break;
    default:
        assert(0);
        
    }
    return 0;
}

int flag_sequence_armor_from_library_callback(flag_id_t new_flag, int time_passed){
    if (!time_passed) return 0;

    switch(new_flag) {
    case fi_armor_restoring_library:
        Reset(bookshelf_book_removed);
        Reset(bookshelf_unlocked);
        Reset(bookshelf_moved);
        break;
    case fi_armor_returning_study:
        if (IsNextArea(study)){
            // 先に書斎について悪さをしていたら座らずにすぐ襲う
            if (Is(start_armor_to_study)){
                message("図書室から戻ってきた鎧が、"
                        "あなたに向かってきた!");
                Set(armor_in_study);
                Reset(start_armor_to_study);
            }
            else {
                message("図書室から戻ってきた鎧が、"
                        "元の場所に座った。");
                Reset(armor_moving);
            }
        }
        else {
            if (MovingTo(library)){
                message("図書室にいた鎧が書斎に戻り、"
                        "元の場所に座った。");
            }
            Reset(armor_moving);
        }
        break;
    default:
        assert(0);
        
    }
    return 0;
}
