#ifndef STATUS_H
#define STATUS_H
#include <stdio.h>
#include "term.h"
#include "area.h"

#define ITEM_MAX 256
#define FLAG_SEQUENCE_MAXLEN 16


// 各種フラグのID。
// 一括して0で初期化できるように、初期状態がすべて偽になるように定義する。
// 消費しないアイテムは所持フラグで兼用できるため、ここで取得フラグは用意しない
// (スコア加算されるケースでは取得フラグを用意する)
// '※'はクリアに無関係なフラグ
// '！'は強制的にONになるイベントフラグ
typedef enum {
    fi_invalid = -1,
    fi_begin = 0,
    fi_begin_game_flags = 0,
	fi_save_forbidden = fi_begin_game_flags, // !セーブコマンドが禁止されている

    fi_entrance_unlocked,       // 玄関の扉が解錠されている
    fi_mailbox_opened,          // ※郵便受けが開いている
    fi_mailbox_has_closed,      // ※郵便受けを閉じたことがある(一度開けて閉じた)
    fi_leaflet_taken,           // ※チラシを入手
    fi_medal_taken,             // ※メダルを入手

	fi_newspaper_looked,        // ※新聞を見た(look hatに影響)

	fi_flowerpot_moved,         // ※植木鉢を動かした
    fi_flowerpot_destroyed,     // 植木鉢を壊した
	fi_brass_key_taken,         // 真鍮の鍵を入手
	fi_wooden_key_taken,        // ※木の鍵を入手
    fi_entered_house,           // 屋敷に入った

    //    fi_start_phone_call,        // ！電話が鳴り始めた
	fi_phone_call_continued,    // ！電話が鳴り続けている(以後ON)
	fi_stop_phone_call,         // ！電話が鳴り終わった
	fi_phone_call_stopped,      // ！電話が鳴り終わった(以後ON)
	fi_phone_call_answered,     // ※電話を使って声を聞いた
	fi_photo_checked,           // ※写真を見た
	fi_man_checked,             // ※写真の男性を見た
	fi_woman_checked,           // ※写真の女性を見た
	fi_girl_checked,            // ※写真の少女を見た
	fi_boy_checked,             // ※写真の少年を見た
	fi_clock_working,			// 時計が動作している(鎖を引いた)
	fi_clock_adjusted,          // 時計を○時ジャストに合わせた(時計が鳴るとOFFに戻る)
	fi_clock_striking,          // 時計が鳴る
	fi_clock_bird_appeared,     // 鳥が現れた(次ターンに解除)
	fi_toy_bird_taken,          // 玩具の鳥を入手
    fi_silver_key_found,        // 暖炉の燃え殻を調べて銀の鍵を発見した
    fi_silver_key_taken,        // 銀の鍵を入手した
    fi_fireplace_message_read,  // 暖炉のメッセージを読んだ
    fi_fossil_taken,            // 化石を入手した

    fi_card_taken,              // 大佐のメモが書かれたトランプをとった
	fi_billiard_ball_lost,      // ビリヤード台にビリヤードボールを入れた
	fi_essay_found,             // 少年の作文を見つけた
	fi_essay_taken,             // 少年の作文をとった





    fi_dragon_taken,
    fi_golden_key_taken,
    fi_guestroom_1f_chair_moved,
    fi_guestroom_1f_table_moved,
    fi_guestroom_1f_cabinet_opened,


    fi_bathtub_empty,           // 浴槽の名状しがたいものを流した
    fi_bathroom_1f_message_read,// 鏡の裏のメッセージを読んだ
    fi_mirror_taken,            // 鏡を入手
    fi_glasses_taken,           // 眼鏡を入手
    fi_duck_taken,              // アヒルを入手
    fi_toilet_used,             // ※トイレを使った

    fi_storage_table_moved,     // 物置の奥のテーブルを動かした
    fi_storage_drawers_closed,  // 物置の箪笥の引き出しをしめた
    fi_sponge_wet,              // スポンジを水につけて濡らした
    fi_fishing_rod_with_sponge, // 釣り竿がスポンジつきになっている
    fi_lantern_taken,           // ランタンを入手
    fi_lantern_lit,             // ランタンが灯っている。

    fi_oven_opened,             // オーブンが開いている
    fi_chicken_taken,           // オーブン内の鶏を取った(所持中とは限らない)

    fi_chicken_put,             // 食堂にチキンを置いた

    fi_dining_put_man,          // 食堂のそれぞれの席になにか置いてある
    fi_dining_put_woman,
    fi_dining_put_girl,
    fi_dining_put_boy,          // 初期状態で剣士があるので真であるべきだが
                                // システム的にフラグはすべて初期状態が偽である
                                // これだけfi_dining_notput_boyなどとすると
                                // 依代完成チェックが面倒なので、
                                // area.c:start_game()内で強制的に調整する

    fi_dining_glasses_put,      // 食堂に眼鏡を置いた
    fi_dining_necklace_put,     // 食堂にネックレスを置いた
    fi_dining_plush_cat_put,    // 食堂にぬいぐるみを置いた
    fi_dining_large_robot_put,  // 食堂に巨大ロボットを置いた

    fi_dining_toy_swordman_put, // 食堂に剣士の玩具を置いた
    fi_dining_toy_lion_put,     // 食堂にライオンの玩具を置いた
    fi_dining_toy_bird_put,     // 食堂に鳥の玩具を置いた
    fi_dining_robot_put,        // 食堂にロボットを置いた
    fi_dining_toy_dragon_put,   // 食堂にドラゴンの玩具を置いた

    fi_dining_fishing_rod_put,  // 食堂に釣り竿を置いた
    fi_dining_bottle_put,       // 
    fi_dining_fossil_put,       // 
    fi_dining_screwdriver_put,  // 
    fi_dining_girl_doll_put,    // 
    fi_dining_witch_doll_put,   // 
    fi_dining_marble_put,       // 
    fi_dining_poker_put,        // 
    fi_dining_billiard_ball_put,// 
    fi_dining_rubber_duck_put,  // 
    fi_dining_slingshot_put,  // 
    fi_dining_ladder_put,  // 
    fi_dining_diary_put,  // 
    fi_dining_score_put,  // 
    fi_dining_origami_plane_put,  // 
    fi_dining_essay_put,  // 
    fi_dining_paper_put,  // 
    fi_dining_lantern_put,  // 
    fi_dining_mirror_put,  // 
    fi_dining_medal_put,  // 

    fi_toy_swordman_taken,      // 剣士のおもちゃを取った
	fi_ghost_appeared,          // 食堂に一家の霊が現れた
	fi_candle_lit,              // 蝋燭がついた
	fi_candle_taken,            // 蝋燭を取った
	fi_chicken_bone_taken,      // 鳥の骨を取った

	fi_portrait_eyes_right,     // 肖像画の目が右向き
	fi_portrait_removed,        // 肖像画を外した
	fi_portrait_has_removed,        // 肖像画を外したことがある
    fi_start_armor_to_guestroom,       // 2F客室に鎧が出るイベント開始
    fi_armor_left_study_to_guestroom,  // 鎧が客室を出て2F客室に向かった
    fi_armor_moving_to_guestroom, // 2F客室に鎧が移動中
    fi_armor_in_guestroom,      // 2F客室に鎧が現れた
    fi_armor_restoring_guestroom, // 鎧は2F客室を復元中

    fi_armor_moving, // 鎧が初期位置を離れている

    fi_start_armor_from_guestroom,
    fi_armor_moving_from_guestroom,
    fi_armor_returning_study,


    fi_guestroom_2f_book_moved, // テーブルの上の本を動かした
    fi_logbook_found,           // 看護日誌を見つけた
    //    fi_logbook_taken,           // 看護日誌をとった
    fi_logbook_read,             // 看護日誌を読んだ
    fi_guestroom_2f_chair_moved, // 箪笥の前の椅子を動かした
    fi_guestroom_2f_table_moved, // 箪笥の前のテーブルを動かした
    fi_guestroom_2f_cabinet_opened,
    fi_sponge_taken,
    fi_bottle_taken,

	fi_washer_opened,           // 洗濯機を開けた
	fi_dryer_opened,            // 乾燥機を開けた
	fi_bronze_key_found,        // 青銅の鍵を見つけた
	fi_bronze_key_taken,        // 青銅の鍵を入手
    fi_dog_stood_up,            // 犬が立ち上がった 
    fi_dog_gone,                // 犬が逝ってしまった
    fi_belt_taken,              // 首輪を入手
    fi_fishing_rod_taken,       // 釣り竿を入手
    fi_poker_taken,             // 火かき棒を入手

    fi_attacked_on_2f,          // ！2Fで襲われたことがある

    fi_ladder_taken,            // 梯子を二段ベッドから外した(所持中とは限らない)
    fi_treasure_chest_moved,    // 宝箱を移動した
    fi_treasure_chest_unlocked, // 宝箱を解錠した
    fi_treasure_chest_broken,   // 宝箱の鍵を壊した
    fi_treasure_chest_opened,   // 宝箱を開けた
    fi_billiard_ball_taken,     // ビリヤードの玉を入手
    fi_slingshot_taken,         // 壊れたぱちんこを入手
	fi_slingshot_repaired,      // ぱちんこを修理した

    fi_mattress_moved,          // 二段ベッド上段のマットレスを動かした
    fi_diary_taken,             // 日記を入手
    fi_plush_cat_taken,         // 猫のぬいぐるみを入手

    fi_wardrobe_opened,			// 洋服箪笥の扉が開いている
    fi_saw_shadow,              // 影を見た(見つかったので死亡確定)
    fi_dressing_table_unlocked,
    fi_dressing_table_opened,
    fi_branch_taken,
    fi_necklace_taken,

    fi_start_shadow,            // 2Fの影、イベント系列開始
	fi_shadow_appeared,         // 影が出現
	fi_shadow_behind_door,      // 影が夫人部屋のドアの向こう
	fi_shadow_in_room,          // 影が室内に入ってきた
    fi_shadow_attack,           // 影が襲ってきた
    fi_shadow_wandering,        // 影が室内をさまよっている
    fi_shadow_gone,             // 影が行ってしまった

    fi_bedroom_drawers_opened,  // 寝室の箪笥の引き出しを開けた
    fi_marble_found,            // ベッドの下にビー玉を発見
    fi_marble_taken,            // ビー玉を入手

    fi_start_armor_to_study,    // 鎧が書斎内で襲う際の起動フラグ
                                // 書斎内で鎧が襲うイベントは
                                // フラグ系列を使っていないが、
                                // 他のフラグ系列で戻る途中(i.e. armor_moving)
                                // に書斎内をいじったときはこのフラグを立てておき、
                                // 戻り次第armor_in_studyに移行する
    fi_armor_in_study,

    fi_study_library_door_closed,
    fi_study_desk_unlocked,
    fi_study_desk_opened,
//    fi_amulet_taken,
    fi_screwdriver_taken,
    //    fi_postcard_taken,
    fi_postcard_read,
    //	fi_notebook_taken,
    fi_study_notebook_read,

    fi_armor_dead, 

    fi_armor_in_library,
    fi_armor_restoring_library,
    fi_bookshelf_book_removed,
    fi_bookshelf_unlocked,
    fi_bookshelf_moved,
    //    fi_easy_necromancy_taken,
    fi_easy_necromancy_read,  // 降霊術を読んだ
    fi_library_ladder_set,
    fi_origami_plane_taken,
    fi_fossil_set,            // 化石を図書室に置いた
    fi_armor_crushed,         // 鎧が転んだ
    fi_armor_reviving,        // 転んだ鎧が復活中
    fi_armor_revived,         // 転んだ鎧が立ち上がった

	fi_pocket_notebook_dropped, // !死体を調べて手帳が落ちた
    fi_secret_room_note_read,   // !マクドナルドの遺書を読んだ
    fi_candle_put,              // 祭壇に蝋燭を配置
    fi_magic_circle_invoked,    // 魔法陣本体を起動
    fi_secret_room_letter_read, // !手紙を読んだ
    fi_secret_room_book_read,   // !本を読んだ
    fi_human_bone_taken,        // マクドナルドの骨を入手

    //    fi_dragon_library,        // ドラゴンが図書室に出現
    fi_dragon_moved_corridor_2f_r,  // ドラゴンが2F廊下にいる
    fi_dragon_moved_entrance_hall,  // ドラゴンが玄関ホールにいる
    fi_dragon_moved_guestroom_1f,   // ドラゴンが1F客室にいる
    fi_dragon_in_guestroom_1f,   // ドラゴンが1F客室にいる
    fi_dragon_shotdown,       // ドラゴンが撃ち落とされた

    fi_toy_lion_taken,
    fi_witch_doll_taken,

	fi_score_found,
	fi_score_taken,
    fi_girl_doll_taken,

	fi_score_set,
	fi_score_played,
	fi_steel_key_found,
	fi_steel_key_taken,

    fi_mirror_put,
    fi_bathroom_ghost_appeared,
	fi_bathroom_2f_ghost_seen,
    fi_bathroom_2f_message_found,
    fi_bathroom_2f_message_read,

//	    fi_amulet_put,
//    fi_memo_taken,

	fi_hall_dining_door_unlocked,
    fi_hall_right_door_unlocked,
    fi_landing_door_unlocked,
    fi_music_room_door_unlocked,


	fi_cycrotron_panel_open,
	fi_cycrotron_small_door_open,
	fi_cycrotron_round_door_open,
	fi_chicken_bone_put,
	fi_human_bone_put,
	fi_rubber_duck_put,
    fi_cycrotron_activated,
    fi_boiler_room_note_read,

	fi_hat_taken,                  // 大佐の帽子を拾った

    fi_temple_spell_cast,          // 神殿で正しい呪文を唱えた
	fi_great_old_one_gone,         // 支配者を送還した
    fi_temple_unsunken,            // 神殿の水が引いた
	                               // 支配者送還と連動だが、水が引くタイミングを
								   // 制御できるよう別フラグにする
    fi_temple_ceiling_destroyed,   // 神殿の天蓋が破壊され、クルマが落下した
	fi_temple_burning,             // ガソリンに火がついた

    fi_temple_exploded,            // 崖下に出てから、神殿内で爆発した
    fi_house_burning,              // 屋敷に引火した
    fi_the_end,                    // エンディングロゴを出すか
    fi_dead, // 死亡した(死亡の瞬間に設定する)
             // ※ゲーム再スタート時に他のフラグと共にクリアされる
    fi_achievements_door_opened,

    fi_end_regular_flags,

    fi_begin_door_flags = 512,
    // 各ドアを開けたことがあるかどうか
    fi_hall_dining_door_opened = fi_begin_door_flags,
    fi_hall_left_door_opened,
    fi_hall_right_door_opened,
    fi_bathroom_door_opened,
	fi_dining_living_door_opened,
	fi_dining_kitchen_door_opened,
    fi_kitchen_corridor_door_opened,
    fi_laundry_corridor_door_opened,
    fi_living_corridor_door_opened,
	fi_guestroom_1f_door_opened,
    fi_storage_door_opened,
	fi_playroom_door_opened,
    fi_landing_door_opened,
    fi_boys_room_door_opened,
    fi_girls_room_door_opened,
    fi_ladys_room_door_opened,
    fi_guestroom_2f_door_opened,
    fi_study_corridor_door_opened,
    fi_library_corridor_door_opened,
    fi_bedroom_door_opened,
    fi_music_room_door_opened,
    fi_bathroom_2f_door_opened,
	fi_music_room_right_door_opened,
	fi_balcony_door_opened,
    fi_laundry_cellar_door_opened,
    fi_boiler_room_door_opened,

    fi_end_door_flags,

    fi_begin_discovered_flags = 768,
    // 各エリアに入ったことがあるか
    fi_title_discovered,
    fi_game_over_discovered,
	fi_ending_discovered,
    fi_achievements_discovered,

    fi_mailbox_discovered,
    fi_entrance_discovered,
    fi_opened_window_discovered,
    fi_washtub_discovered,
    fi_backdoor_discovered,
    fi_under_tree_discovered,
    fi_garden_discovered,

    fi_entrance_hall_discovered,
    fi_corridor_1f_fl_discovered,
    fi_corridor_1f_bl_discovered,
    fi_corridor_1f_fr_discovered,
    fi_corridor_1f_mr_discovered,
    fi_corridor_1f_br_discovered,
    fi_bathroom_discovered,
    fi_dining_discovered,
    fi_kitchen_discovered,
    fi_laundry_discovered,
    fi_living_discovered,
    fi_guestroom_1f_discovered,
	fi_storage_discovered,
	fi_playroom_discovered,

    fi_landing_discovered,
    fi_corridor_2f_l_discovered,
    fi_corridor_2f_c_discovered,
    fi_corridor_2f_cr_discovered,
    fi_corridor_2f_r_discovered,
	
    fi_boys_room_discovered,
    fi_bed_discovered,
    fi_girls_room_discovered,
    fi_guestroom_2f_discovered,
    fi_study_discovered,
    fi_library_discovered,
    fi_bookshelf_discovered,
    fi_bedroom_discovered,
    fi_ladys_room_discovered,
    fi_wardrobe_discovered,
    fi_secret_room_discovered,
    fi_music_room_discovered,
    fi_bathroom_2f_discovered,
	fi_balcony_discovered,
    fi_balcony_down_discovered,

    fi_cellar_discovered,
    fi_boiler_room_discovered,
    fi_corridor_0f_discovered,
    fi_temple_discovered,
	fi_temple_ceiling_discovered,
	fi_cave_entrance_discovered,
    fi_cave_discovered,
    fi_under_cliff_discovered,

    fi_working_discovered,

    fi_end_discovered_flags,
    fi_end_game_flags = fi_end_discovered_flags,

    // クリア・ゲームオーバー時に設定されるフラグ(達成度用)

    // @note 今後フラグを追加してもachievement情報の互換性が取れるように
    // begin_clear_flagsの位置を固定しておく
    // クリアフラグのみコピーできるように、必ず8の倍数にする
    fi_begin_clear_flags = 1024,
    // 死因
    fi_death_by_touching_water,  // 浴室の水に触れた
    fi_death_by_touching_shadow, // 2Fの影に触れた
    fi_death_by_armor,           // 鎧に斬り殺された
    fi_death_by_exiting_window,  // 窓で真っ二つになった
    fi_death_by_starvation,      // 隠し部屋で餓死
    fi_death_by_tentacle,        // 達成度部屋で触手に殺された

    // 以下はクリア時のみ設定される(クリアしないなら達成は容易だから)
    // クリア方法が複数ある箇所は、それぞれチェック
    fi_clear_armor_alive,              // 鎧を倒していない
    fi_clear_armor_dead,               // 鎧を倒した
    fi_clear_treasure_chest_unlocked,  // 宝箱を解錠した
    fi_clear_treasure_chest_locked,    // 宝箱を解錠せずに鍵を壊した
    fi_clear_portrait_disabled,        // 肖像画を外さなかった
    fi_clear_portrait_enabled,         // 肖像画を外した
    fi_clear_using_human_bone,         // マクドナルドの骨でサイクロトロンを起動
    fi_clear_using_chicken_bone,       // 鶏の骨でサイクロトロンを起動
    fi_clear_phone_call_answered,      // 居間の電話に出た
    fi_clear_phone_call_ignored,       // 居間の電話を無視した


    // 同種行動をすべてやっているか集計
    fi_clear_reading_all_docs,         // 文書類をすべて読んだ
    fi_clear_closed,                   // 引き出しなどすべて閉めている

    fi_end_clear_flags,

    /*
      達成度ネタ
      しらみつぶし度v
        - トイレを使った
        - たからばこを移動した
      - 
      - 文書をすべて読んだ?
      - ダミーアイテムをすべてとった

      - achievementsの部屋でなんかすると支配者が出てきて死ぬ
        -> killed by achievements

     */

    fi_end
} flag_id_t;

typedef enum {
    fs_begin = 0,
    fs_dragon = 0, // 金の鍵をドラゴンが持ち去る
    fs_shadow,     // 首飾りをとると影が現れる
    // 鎧イベントは出現と帰還を別系列にする
    // 複数の鎧イベントが干渉する可能性があり、
    // 帰還するまで別イベントでの出現を保留する必要がある
    // そこで、
    // - 各系列は鎧の位置と無関係に発動させる
    // - auto進行時に鎧が未帰還なら進行しない(callback内で戻す)
    // で対応する
    // この際、系列Aのフラグを系列Bで参照したとき、どのフラグがONになっているかが
    // 系列A, Bの処理順に依存してしまう
    // そこで、他系列でチェックする「帰還済かどうか」をすべて別系列に分離し、
    // A, Bの処理順に異存なく双方から同様に見えるようにする

    fs_armor_to_guestroom,    // 2F客室をいじると鎧が現れる
    fs_armor_to_library,      // 図書室をいじると鎧が現れる
    fs_armor_from_guestroom,  // 2F客室から鎧が戻る
    fs_armor_from_library,    // 図書室から鎧が戻る
    fs_end
} flag_sequence_id_t;

typedef struct {
    flag_id_t flags[FLAG_SEQUENCE_MAXLEN];
    int num; // フラグ数(status_init()で自動計算)
    int is_exclusive; // 非0なら、flags[phase]のみON。0ならそこまですべてON
    int is_auto;      // 非0なら、ターン毎に自動的に進行する
    int (*callback)(flag_id_t new_flag, int time_passed);
} flag_sequence_t;

typedef struct {
    unsigned char phase; // アクティブなら、現在flags[phase]まで立っている
    unsigned char bits;
} flag_sequence_status_t;

// 系列進行中なら非0
#define FLAG_SEQUENCE_BIT_ACTIVE 0
// 非アクティブのとき、一連のイベントが最後まで完了しているなら非0、
//                    途中で中断してイベント連鎖全体がキャンセルされていたら0
#define FLAG_SEQUENCE_BIT_COMPLETE 1
//// 非0なら、ターン毎に自動的に進行する
//#define FLAG_SEQUENCE_BIT_AUTO   2

typedef struct {
    flag_id_t id;
    int value;
    int score;
} point_t;

#define FLAG_CHAR_NUM (fi_end/8+1)

#define FLAG_SEQUENCE_MAXNUM 128

typedef struct {
	// area_id_tだがセーブ時のエンディアン問題を避けるためcharとする
	// ※areag_idが255を超えないことが前提
	//    area_id_t current_area;
    unsigned char current_area;
    // 持ち物の個数
	unsigned char item_num;
    // 持ち物のnoun_id
    unsigned char items[ITEM_MAX];
    // 各フラグ系列の状態
    flag_sequence_status_t flag_sequence[FLAG_SEQUENCE_MAXNUM];
    // 各種フラグ
    unsigned char flags[FLAG_CHAR_NUM];
} status_t;


#define Is(f) (is_flag_on(fi_ ## f))
#define Not(f) (is_flag_off(fi_ ## f))
#define Set(f) set_flag(fi_ ## f)
#define Reset(f) reset_flag(fi_ ## f)
#define Toggle(f) toggle_flag(fi_ ## f)
#define TestAndSet(f, msg1, msg2) test_and_set_flag(fi_ ## f, msg1, msg2)
#define TestAndReset(f, msg1, msg2) test_and_reset_flag(fi_ ## f, msg1, msg2)
#define Has(f) (has_item(ni_ ## f))
#define HasNot(f) (!has_item(ni_ ## f))
#define IsActive(fs) (is_flag_sequence_active(fs_ ## fs))
#define NotActive(fs) (!is_flag_sequence_active(fs_ ## fs))
#define Complete(fs) (is_flag_sequence_complete(fs_ ## fs))
#define IsCurrentArea(ai) (current_area() == (ai_ ## ai))
#define IsNewArea(ai) (new_area() == (ai_ ## ai))
#define MovingTo(ai) (IsNewArea(ai))
#define MovingFrom(ai) (IsCurrentArea(ai) && !IsNewArea(invalid))
#define Staying(ai) (IsCurrentArea(ai) && IsNewArea(invalid))
#define IsNextArea(ai) (MovingTo(ai) || Staying(ai))

void status_init(void);
void clear_status(void);
void init_achievements_status(void);
area_id_t current_area(void);
void set_area(area_id_t id);
int is_flag_on(flag_id_t fid);
int is_flag_off(flag_id_t fid);
void set_flag(flag_id_t fid);
void reset_flag(flag_id_t fid);
void toggle_flag(flag_id_t fid);
void test_and_set_flag(flag_id_t fid, char *msg_success, char *msg_failure);
void test_and_reset_flag(flag_id_t fid, char *msg_success, char *msg_failure);
int count_on_flags(int num, ...);
int has_item(noun_id_t nid);
void get_item(noun_id_t nid);
void lose_item(noun_id_t nid);
int get_item_num(void);
noun_id_t get_item_id(int ix);
void show_score(char *msg, point_t points[], int point_num);
void show_normal_score(void);
#ifdef DUMMY
void show_maniac_score(void);
#endif

int load_data(void);
int save_data(void);
int load_achievements_flags(void);
int load_achievements(void);
int save_achievements(void);
int update_achievements(void);
int load_status(char *filename, status_t *status);
int save_status(char *filename, status_t *status);

void clear_flag_sequence(flag_sequence_id_t fs);
void start_flag_sequence(flag_sequence_id_t fs);
void proceed_flag_sequence(flag_sequence_id_t fs);
int jump_flag_sequence(flag_sequence_id_t fs, flag_id_t fid);
void abort_flag_sequence(flag_sequence_id_t fs);
void process_flag_sequences(int time_passed);
//void proceed_auto_flag_sequences(void);
//void callback_flag_sequences(int time_passed);
void set_flag_sequence_active(flag_sequence_id_t fs);
void reset_flag_sequence_active(flag_sequence_id_t fs);
int is_flag_sequence_active(flag_sequence_id_t fs);
void set_flag_sequence_complete(flag_sequence_id_t fs);
int is_flag_sequence_complete(flag_sequence_id_t fs);
//void set_flag_sequence_auto(flag_sequence_id_t fs);
//void reset_flag_sequence_auto(flag_sequence_id_t fs);

int has_light(void);



#endif
