#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include "config.h"
#include "uty.h"



static char *exe_dir;

void init_uty(char *arg0){
    exe_dir = get_exe_dir(arg0);
}


// ARGV[0]を渡すと、実行ファイルのディレクトリを返す
char *get_exe_dir(char *argv0){
    static char dir[EXE_DIR_MAX];
    char *sp = dir;
    char *ep;

    ep = strrchr(argv0, '/');
    // 絶対・相対パスつきで起動していたら、パス部分を返す
    // Win32版のargv[0]はフルパスなので、こちらで処理される
    if (ep != NULL){
        while (argv0 != ep+1){
            //            printf("%p %p %c\n", sp, ep, *argv0);
            *sp = *argv0;
            sp ++;
            argv0 ++;
        }
        *sp = '\0';
        return dir;
    }
    // 実行ファイル名のみの場合はPATHを順にチェック
    else {
        sp = getenv("PATH");
        ep = strchr(sp, ':');
        while (ep != NULL){
            strncpy(dir, sp, ep-sp);
            dir[ep-sp] = '/';
            strcpy(dir + (ep - sp + 1), argv0);
            //            printf("|%s|\n", dir);
            // 試したパスに実行ファイル名のファイルが実行属性付きであれば
            // それが答えと見なす
            if (access(dir, X_OK) == 0){
                dir[ep-sp+1] = '\0';
                return dir;
            }
            sp = ep + 1;
            ep = strchr(sp, ':');
        }
        return NULL;
    }
}

// pathで示されるディレクトリを作成
// ※pathの最下層のディレクトリのみ作成する。
//   それより上のディレクトリが存在しなければ失敗する
// 成功時は0、失敗時は非0を返す
int make_dir(char *path){
    return mkdir(path, MAKEDIR_MODE);
}

// ディレクトリpathが存在するなら非0を返す
int check_dir(char *path){
    struct stat buf;
    // stat()に失敗したらチェック失敗
    if (stat(path, &buf) != 0){
        return 0;
    }
    else {
        return S_ISDIR(buf.st_mode);
    }
}
