#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "config.h"
#include "verb.h"
#include "layout.h"
#include "message.h"
#include "status.h"
#include "history.h"
#include "term.h"

static verb_t verbs[] = {
  {vi_invalid, {"", "", ""}, ""},
  {vi_forward, {"forward", "すすむ", "ススム"}, "f"},
  {vi_backward, {"backward", "もどる", "モドル"}, "b"},
  {vi_left, {"left", "ひだり", "ヒダリ"}, "l"},
  {vi_right, {"right", "みぎ", "ミギ"}, "r"},
  {vi_up, {"up", "あがる", "アガル"}, "u"},
  {vi_down, {"down", "おりる", "オリル"}, "d"},
  {vi_enter, {"enter", "はいる", "ハイル"}, NULL},
  {vi_exit, {"exit", "でる", "デル"}, NULL},
  {vi_inventory, {"inventory", "もちもの", "モチモノ"}, "i"},
  {vi_look, {"look", "みる", "ミル"}, NULL},
  {vi_examine, {"examine", "しらべる", "シラベル"}, NULL},
  {vi_recall, {"recall", "おもいだす", "オモイダス"}, NULL},
  {vi_get, {"get", "とる", "トル"}, NULL},
  {vi_put, {"put", "おく", "オク"}, NULL},
  {vi_use, {"use", "つかう", "ツカウ"}, NULL},
  {vi_open, {"open", "ひらく", "ヒラク"}, NULL},
  {vi_close, {"close", "とじる", "トジル"}, NULL},
  {vi_hit, {"hit", "たたく", "タタク"}, NULL},
  {vi_kick, {"kick", "ける", "ケル"}, NULL},
  {vi_destroy, {"break", "こわす", "コワス"}, NULL},
  {vi_move, {"move", "うごかす", "ウゴカス"}, NULL},

  {vi_remove, {"remove", "どける", "ドケル"}, NULL},
  {vi_push, {"push", "おす", "オス"}, NULL},
  {vi_pull, {"pull", "ひく", "ヒク"}, NULL},
  {vi_touch, {"touch", "さわる", "サワル"}, NULL},
  {vi_turn, {"turn", "まわす", "マワス"}, NULL},
  {vi_read, {"read", "よむ", "ヨム"}, NULL},
  {vi_throw, {"throw", "なげる", "ナゲル"}, NULL},
  {vi_combine, {"combine", "がったい", "ガッタイ"}, NULL},
  {vi_dismantle, {"dismantle", "ぶんかい", "ブンカイ"}, NULL},
  {vi_adjust, {"adjust", "あわせる", "アワセル"}, NULL},
  {vi_pray, {"pray", "いのる", "イノル"}, NULL},
  {vi_talk, {"talk", "はなす", "ハナス"}, NULL},
  {vi_cast, {"cast", "となえる", "トナエル"}, NULL},
  {vi_eat, {"eat", "たべる", "タベル"}, NULL},
  {vi_drink, {"drink", "のむ", "ノム"}, NULL},
  {vi_wait, {"wait", "まつ", "マツ"}, NULL},
  {vi_shoot, {"shoot", "うつ", "ウツ"}, NULL},
  {vi_erase, {"erase", "けす", "ケス"}, NULL},
  {vi_give, {"give", "あげる", "アゲル"}, NULL},
  {vi_show, {"show", "みせる", "ミセル"}, NULL},
  {vi_help, {"help", "へるぷ", "ヘルプ"}, NULL},
  {vi_load, {"load", "ろーど", "ロード"}, NULL},
  {vi_save, {"save", "せーぶ", "セーブ"}, NULL},
  {vi_config, {"config", "せってい", "セッテイ"}, NULL},
  {vi_quit, {"quit", "おわる", "オワル"}, NULL},
  // vi_quitより後はhelp verbで表示しない
  {vi_score, {"score", "とくてん", "トクテン"}, NULL},
  {vi_start, {"start", "はじめる", "ハジメル"}, NULL},
  {vi_title, {"title", "たいとる", "タイトル"}, NULL},
#ifdef DEBUG
  {vi_debug, {"debug", "でばっぐ", "デバッグ"}, NULL},
  {vi_test, {"test", "てすと", "テスト"}, NULL},
  {vi_teleport, {"teleport", "てれぽーと", "テレポート"}, NULL},
  {vi_summon, {"summon", "しょうかん", "ショウカン"}, NULL},
  {vi_flag, {"flag", "ふらぐ", "フラグ"}, NULL},
#endif
};
#define VERB_NUM (sizeof(verbs)/sizeof(verb_t))

static esynonym_t esynonyms[] = {
    {"escape", vi_exit},
    {"search", vi_examine},
    {"destroy", vi_destroy},
    {"take", vi_get},
    {"set", vi_put},
    {"call", vi_talk},
    {"assemble", vi_combine},
    {"disassemble", vi_dismantle},
    {"climb", vi_up},
    {"bye", vi_quit},

    {"play", vi_dont_care},
};

static jsynonym_t jsynonyms[] = {
    {"まえ", "マエ", vi_forward},
    {"うしろ", "ウシロ", vi_backward},
    {"うえ", "ウエ", vi_up},
    {"した", "シタ", vi_down},
    {"のぼる", "ノボル", vi_up},
    {"くだる", "クダル", vi_down},
    {"ちかづく", "チカヅク", vi_forward},

	{"にげる", "ニゲル", vi_exit},
	{"も", "モ", vi_inventory},
	{"い", "イ", vi_inventory},
	{"あいてむ", "アイテム", vi_inventory},
    {"てにいれる", "テニイレル", vi_get},
    {"にゅうしゅ", "ニュウシュ", vi_get},
    {"あける", "アケル", vi_open},
    {"しめる", "シメル", vi_close},
    {"ひっぱる", "ヒッパル", vi_pull},
	{"ふれる", "フレル", vi_touch},
	{"なぐる", "ナグル", vi_hit},
	{"けとばす", "ケトバス", vi_kick},
	{"よぶ", "ヨブ", vi_talk},
	{"いう", "イウ", vi_talk},
    {"なでる", "なでる", vi_touch},

	{"くみあわせる", "クミアワセル", vi_combine},
	{"くみたてる", "クミタテル", vi_combine},
    {"ばらす", "バラス", vi_dismantle},
    {"はずす", "ハズス", vi_dismantle},
	{"どかす", "ドカス", vi_remove},
	{"わたす", "ワタス", vi_give},
	{"あたえる", "アタエル", vi_give},

	{"すこあ", "スコア", vi_score},
	{"すたーと", "スタート", vi_start},
	{"かいし", "カイシ", vi_start},
	{"やめる", "ヤメル", vi_quit},
	{"しゅうりょう", "シュウリョウ", vi_quit},
	{"さようなら", "サヨウナラ", vi_quit},
	{"さよなら", "サヨナラ", vi_quit},
	{"さらば", "サラバ", vi_quit},
	{"もどす", "モドス", vi_dont_care},
	{"つける", "ツケル", vi_dont_care},
	{"めくる", "メクル", vi_dont_care},
	{"えんそう", "エンソウ", vi_dont_care},
#ifdef DEBUG
    {"てんい", "テンイ", vi_teleport},
#endif
    //    {vi_},
};

#define ESYNONYM_NUM (sizeof(esynonyms)/sizeof(esynonym_t))
#define JSYNONYM_NUM (sizeof(jsynonyms)/sizeof(jsynonym_t))
    
static verb_trans_t etrans[] = {
    {"play", ni_piano, vi_use},
    {"get", ni_telephone, vi_use},
    {"set", ni_clock, vi_adjust},
    {"shoot", ni_slingshot, vi_use},
};
static verb_trans_t jtrans[] = {
    {"とる", ni_telephone, vi_use},
    {"でる", ni_telephone, vi_use},
    {"つける", ni_range, vi_use},
    {"つける", ni_oven, vi_use},
    {"つける", ni_tv, vi_use},
    {"つける", ni_lantern, vi_use},
    //    {"もどす", ni_ladder, vi_put},
    {"ひく", ni_piano, vi_use},
    {"えんそう", ni_piano, vi_use},
    {"はずす", ni_portrait, vi_remove},
    {"はずす", ni_screw, vi_remove},
    {"めくる", ni_mattress, vi_remove},
    {"ひらく", ni_origami_plane, vi_dismantle},
    {"うつ", ni_slingshot, vi_use},
    {"ひらく", ni_wastepaper, vi_examine},
    //    {"", ni_, vi },
};
#define ETRANS_NUM (sizeof(etrans)/sizeof(verb_trans_t))
#define JTRANS_NUM (sizeof(jtrans)/sizeof(verb_trans_t))

static dic_t verb_dic;
static verb_id_t trans_table[WORD_MAX_NUM][ni_end];

int vid2verb_ix[vi_end];

static history_t verb_hist;


void verb_init(void){
    int i, j, ix;
    verb_id_t vid;

    init_dic(&verb_dic);
    init_history(&verb_hist);    
    
    // 基本動詞を辞書に登録
	// ※先頭のダミーは飛ばすので1から回す
    for (i = 1 ; i < VERB_NUM ; i++){

		vid = verbs[i].id;
        if (vid2verb_ix[vid] != 0){
            printf("システムエラー: "
                   "動詞「%s」のID(%d)が"
                   "動詞「%s」のIDと重複しています\n", 
                   verbs[i].word[0], vid, verbs[vid2verb_ix[vid]].word[0]);
            exit(1);
        }
        //        printf("set vid:%d ix:%d\n", vid, i);
        vid2verb_ix[vid] = i;
		for (j = 0 ; j < 3 ; j++){
            register_dic(&verb_dic, verbs[i].word[j], verbs[i].id);
        }
        if (verbs[i].shortcut != NULL){
            register_dic(&verb_dic, verbs[i].shortcut, verbs[i].id);
        }
    }
    // 類義語を辞書に登録
    for (i = 0 ; i < ESYNONYM_NUM ; i++){
        register_dic(&verb_dic, esynonyms[i].word, esynonyms[i].id);
    }
    for (i = 0 ; i < JSYNONYM_NUM ; i++){
        register_dic(&verb_dic, jsynonyms[i].hiragana, jsynonyms[i].id);
        register_dic(&verb_dic, jsynonyms[i].katakana, jsynonyms[i].id);
    }
    // 目的語別意味変換テーブルの作成
    for (i = 0 ; i < ETRANS_NUM ; i++){
        ix = get_verb_ix(etrans[i].word);
        if (ix <= 0){
            printf("Error: trans for Unknown verb '%s'.\n", etrans[i].word);
            exit(1);
        }
        trans_table[ix][etrans[i].nid] = etrans[i].new_vid;
    }
    for (i = 0 ; i < JTRANS_NUM ; i++){
        // ひらがなの登録
        ix = get_verb_ix(jtrans[i].word);
        if (ix < 0){
            printf("Error: trans for Unknown verb '%s'.\n", jtrans[i].word);
            exit(1);
        }
        trans_table[ix][jtrans[i].nid] = jtrans[i].new_vid;
        // カタカナの登録
        vid = word_ix2id(ix, &verb_dic); // ひらがなでvidを取得
        for (j = 1 ; j < VERB_NUM ; j++){
            if (verbs[j].id == vid){
                ix = get_verb_ix(verbs[j].word[im_katakana]);
                if (ix < 0){
                    printf("Error: trans for Unknown verb '%s'.\n",
                           verbs[j].word[im_katakana]);
                    exit(1);
                }
                break;
            }
        }
    }
    // チェックした単語がヒストリに残るのでヒストリを初期化
    init_history(&verb_hist);
}
int get_verb_ix(char *word){
    return get_word_ix(word, &verb_dic);
}

verb_id_t get_verb_id(char *verb, noun_id_t nid){
    int ix;
    verb_id_t vid = vi_unknown, new_vid;

    ix = get_verb_ix(verb);
    register_verb_history(ix);
    if (ix >= 0){
        new_vid = trans_table[ix][nid];
        if (new_vid){
            vid = new_vid;
        }
        else {
            vid = word_ix2id(ix, &verb_dic);
        }
    }
    return vid;
}
/*
char *vid2verb(verb_id_t vid){
    int i;
    input_mode_t mode = get_config_input();

    for (i = 0 ; i < VERB_NUM ; i++){
        if (verbs[i].id == vid){
            return verbs[i].word[mode];
        }
    }
    return NULL;
}
*/
char *vid2verb(verb_id_t vid){
    input_mode_t mode = get_term_input_mode();
	int ix = vid2verb_ix[vid];

	if (ix > 0){
        return verbs[ix].word[mode];
    }
    return NULL;
}



void register_verb_history(int verb_ix){
    register_history(verb_ix, &verb_hist);
}
char *get_history_verb(void){
	history_entry_t entry = verb_hist.entries[verb_hist.ix];
	if (entry.ix == INVALID_WORD_IX) return NULL;
	return ix2word(entry.ix, &verb_dic);
};
void prev_history_verb(void){
	prev_history(&verb_hist);
}
void next_history_verb(void){
	next_history(&verb_hist);
}

 
void show_help_verb(void){
    char buf[CONSOLE_CHR_WIDTH * CONSOLE_CHR_HEIGHT], *p;
    int i;
    input_mode_t mode = get_term_input_mode();
    char *word;

    message("以下の動詞が使用できます。");
    buf[0] = '\0';
    p = buf;
	// ※先頭のダミーは飛ばすので1から回す
    for (i = 1 ; i < VERB_NUM ; i++){
        word = verbs[i].word[mode];
        if (verbs[i].shortcut != NULL && mode == im_english){
            //            printf("test|%s|%s|\n", word, verbs[i].shortcut);
            *p++ = verbs[i].shortcut[0];
            *p++ = '(';
            strcpy(p, word+1);
            p += strlen(word+1);
            *p++ = ')';
        }
        else {
            strcpy(p, word);
            p += strlen(word);
        }
        *p++ = ' ';
        if (verbs[i].id == vi_quit) break;


        /*
        if (verbs[i].shortcut != 0 && mode == im_english){
            *p++ = verbs[i].shortcut[0];
            *p++ = '(';
            strcpy(p, &verbs[i].word[im_english][1]);
            p += strlen(&verbs[i].word[im_english][1]);
            *p++ = ')';
        }
        */
    }
    *p = '\0';
    puts(buf);
    message(buf);
}

void show_help_basic(void){
    message("「動詞」もしくは「動詞 名詞」の形式で入力してください。");
    message("「%v(f)」「%v(b)」「%v(l)」「%v(r)」で前後左右、"
            "「%v」「%v」で上下に移動できます。「%v」で持ち物を確認できます。",
            vi_forward,
            vi_backward,
            vi_left,
            vi_right,
            vi_up, vi_down,
            vi_inventory);
    message("「%v」「%v」「%v」で設定変更、説明表示、保存ができます。",
    vi_config, vi_help, vi_save);
    //    message("設定は「%v」、その他のコマンドは「%v」で表示できます。", 
    //            vi_config, vi_help);
    //    message("ゲームの途中経過は「%v」で保存できます。", 
    //            vi_save);
}

void show_help_tips(void){
    message("・「%v」できる状況では、デストラップ(ハマり)は存在しません。",
            vi_save);
	message("・目的語(名詞)なしの「%v」で、現在地の説明を見ることができます。",
            vi_look);
    message("・「%v」は時間が経過しません。「%v」は詳しく調べます。", 
            vi_look, vi_examine);
//    message("・入力方式を問わず「f」「b」「l」「r」で前後左右に移動できます。");
    message("・提示される動詞・名詞のみでクリア可能ですが、"
            "類義語も受け付けます。");
    message("・文字が書かれたものは「%v 名詞」で読めます。", vi_read);
}

void show_help_key(void){
    message("英字・数字・スペース: コマンド入力");
    message("リターン            : コマンド決定");
    message("[PageUp][PageDown]  : メッセージのロールバック");
    message("↑↓                : ヒストリ(過去に入力した動詞・名詞を呼び出す)");
    message("F1                  : 入力方式切替");
}
void show_help_assistance(void){
    message("F2                  : ミニマップの表示ON/OFF");
    message("F3                  : ミニマップの表示位置移動");
    message("F4                  : ミニマップの非透過/半透過");
    message("おもいだす          : 過去に読んだ本などの内容を再表示");
}
